<?php
require __DIR__ . '/../vendor/autoload.php';

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Factory\AppFactory;
use GuzzleHttp\Exception\GuzzleException;
use InfluxDB2\Client;
use InfluxDB2\Model\WritePrecision;
use InfluxDB2\Point;

/**
 * Instantiate App
 *
 * In order for the factory to work you need to ensure you have installed
 * a supported PSR-7 implementation of your choice e.g.: Slim PSR-7 and a supported
 * ServerRequest creator (included with Slim PSR-7)
 */
$app = AppFactory::create();

/**
 * The routing middleware should be added earlier than the ErrorMiddleware
 * Otherwise exceptions thrown from it will not be handled by the middleware
 */
$app->addRoutingMiddleware();

/**
 * Add Error Middleware
 *
 * @param bool $displayErrorDetails
 *            -> Should be set to false in production
 * @param bool $logErrors
 *            -> Parameter is passed to the default ErrorHandler
 * @param bool $logErrorDetails
 *            -> Display error details in error log
 * @param LoggerInterface|null $logger
 *            -> Optional PSR-3 Logger
 *            
 *            Note: This middleware should be added last. It will not handle any exceptions/errors
 *            for middleware added after it.
 */
$errorMiddleware = $app->addErrorMiddleware(true, true, true);

$app->setBasePath('/api2');

// Define app routes
$app->get('/hello/{name}', function (Request $request, Response $response, $args) {
    $name = $args['name'];
    $response->getBody()
        ->write("Hello, $name");
    return $response;
});

// Define app routes

// /directory/ : structure de l'annuaire (parcs)

$app->get('/directory/parcs', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    if (empty($params['name'])) {
        $params['name'] = "*";
    }
    $res = search_parcs($config, $params['name']);
    if (count($res) > 0) {
        $payload = json_encode($res, JSON_PRETTY_PRINT);
    } else {
        $payload = json_encode([
            "vide"
        ]);
    }
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'application/json')
        ->withStatus(201);
});

// Define app routes
$app->get('/directory/machines', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";

    $machines = list_members_parc($config, $params['name'], false);
    if (count($machines) > 0) {
        // foreach ($machines as $k => $machine) {
        // foreach ($machine['memberof'] as $kk => $g) {
        // $machines[$k]['memberof'][$kk] = ldap_dn2cn($g);
        // }
        // }
        $payload = json_encode($machines, JSON_PRETTY_PRINT);
    } else {
        $payload = json_encode([
            "vide"
        ]);
    }
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'application/json')
        ->withStatus(201);
});

$app->get('/directory/etabs', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    if (empty($params['name'])) {
        $params['name'] = "*";
    }
    $data = search_etabs($config, $params['name']);
    if (count($data) > 0) {
        $payload = json_encode($data, JSON_PRETTY_PRINT);
    } else {
        $payload = json_encode([
            "vide"
        ]);
    }
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'application/json')
        ->withStatus(201);
});

$app->get('/status/machine', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    $machine = search_machine($config, $params['name'], true);
    $machine = array_merge($machine, get_machine_status($config, $params['name']));
    if (is_array($machine)) {
        $payload = json_encode($machine, JSON_PRETTY_PRINT | JSON_INVALID_UTF8_IGNORE);
        $response->getBody()
            ->write($payload);
        return $response->withHeader('Content-Type', 'application/json')
            ->withStatus(201);
    } else {
        return $response->withStatus(204);
    }
});

$app->get('/status/remote', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    require "remote.inc.php";
    $machine = search_machine($config, $params['name'], true);
    $machine = remote_status($config, $config['login'], $machine);
    if (is_array($machine)) {
        $payload = json_encode($machine, JSON_PRETTY_PRINT | JSON_INVALID_UTF8_IGNORE);
        $response->getBody()
            ->write($payload);
        return $response->withHeader('Content-Type', 'application/json')
            ->withStatus(201);
    } else {
        return $response->withStatus(204);
    }
});

$app->get('/status/wpkg', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    include "wpkg_lib.php";
    include "wpkg_libsql.php";
    include "wpkg_lib_admin.php";

    $liste_hosts = info_postes($config);
    ksort($liste_hosts);
    $name = $params['name'];
    if (! array_key_exists($name, $liste_hosts)) {
        $name = key($liste_hosts);
    }
    $res = wpkg_poste_info($config, $name);
    // $res = $parc;
    if (is_array($res)) {
        $payload = json_encode($res, JSON_PRETTY_PRINT | JSON_INVALID_UTF8_IGNORE);
        $response->getBody()
            ->write($payload);
        return $response->withHeader('Content-Type', 'application/json')
            ->withStatus(200);
    } else {
        return $response->withStatus(204);
    }
});

$app->get('/stats/week', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    include ("stats_libsql.php");
    include "wpkg_lib.php";
    include "wpkg_libsql.php";
    
    $payload = generate_stats_week($config, $params['name']);
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'image/png')
        ->withStatus(201);
});

// a appeler sur le serveur de l'étab
$app->get('/status/parc', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    require "parcs.inc.php";
    include "wpkg_lib.php";
    include "wpkg_libsql.php";
    include "wpkg_lib_admin.php";
    

    $res = parc_local_status($config, $params['name']);
    // $res = $parc;
    if (is_array($res)) {
        $payload = json_encode($res, JSON_PRETTY_PRINT | JSON_INVALID_UTF8_IGNORE);
    } else {
        $payload = json_encode([
            "vide"
        ]);
    }
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'application/json')
        ->withStatus(200);
});

$app->get('/action/machines', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    require "fonc_outils.inc.php";
    $id = md5(serialize($params));
    $list = apcu_fetch($id);
    if (empty($list)) {
        if (! empty($params['name'])) {
            $etab = $params['etab'] ?? true;
            $recurse = ! empty($params['recurse']);
            $c = etab_suffix($etab);
            if (! preg_match("/" . $c . "/", $params['name'])) {
                $params['name'] .= $c;
            }
            $list = list_members_parc(get_config($config, true, $etab), $params['name'], true, $recurse);
        }
        if (! empty($params['list'])) {
            $list = json_decode($params['list']);
        }
        if (! empty($params['machine'])) {
            $list = [
                search_machine($config, $params['machine'], true)
            ];
        }
    }
    $action = $params['action'];
    async_action($config, $action, $list);
    apcu_store($id, $list);
    if (is_array($list)) {
        $payload = json_encode($list, JSON_PRETTY_PRINT | JSON_INVALID_UTF8_IGNORE);
    } else {
        $payload = json_encode([
            "vide"
        ]);
    }
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'application/json')
        ->withStatus(200);
});

$app->post('/action/machine', function (Request $request, Response $response, $args) {
    $params = (array) $request->getParsedBody();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    require "fonc_outils.inc.php";

    $machine = json_decode($params['machine'], true);
    $res = start_poste_async($config, $params['action'], $machine, false);
    $payload = json_encode($machine, JSON_PRETTY_PRINT);
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'application/json')
        ->withStatus($res);
});

$app->get('/action/parcs', function (Request $request, Response $response, $args) {
    $params = $request->getQueryParams();
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";
    require "fonc_outils.inc.php";

    $parc = list_members_parc($config, $params['name']);
    foreach ($parc as $machine) {
        $res[$machine] = get_machine_status($config, $machine);
    }
    if (count($res) > 0) {
        $payload = json_encode($res, JSON_PRETTY_PRINT);
    } else {
        $payload = json_encode([
            "vide"
        ]);
    }
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'application/json')
        ->withStatus(201);
});

$app->get('/error/{code}', function (Request $request, Response $response, $args) {
    require "config.inc.php";
    $config = get_config();
    require "ldap.inc.php";
    require "actions.inc.php";
    require "logs.inc.php";

    $res = error_message($args['code']);
    if (count($res) > 0) {
        $payload = json_encode($res, JSON_PRETTY_PRINT);
    } else {
        $payload = json_encode([
            "vide"
        ]);
    }
    $response->getBody()
        ->write($payload);
    return $response->withHeader('Content-Type', 'application/json')
        ->withStatus(201);
});

// Run app
$app->run();
?>