<?php

/*
 * Fonctions permettant la gestion des quotas disque
 *
 * Projet Sambaedu
 *
 * Auteur Denis Bonnenfant
 *
 * Licence GPL
 *
 * Principe :
 * - les quotas par groupe/utilisateur sont enregistrés dans la table mysql quota
 * - la valeur effective correspond au max des enregistrements
 * - l'état actuel du système de fichiers est caché dans un tableau apcu, mis à jour toutes les 5 minutes maxi.
 *
 * lecture :
 *
 * - il suffit de lire repquota($device)[$user]
 *
 * ecriture :
 *
 * - en cas de modification dans la base sql, on recalcule les nouveaux quotas effectifs et on met à jour le fs si besoin.
 *
 * avertissement en cas de dépassement :
 *
 * - un script de logon gpo interroge http://se4/action.php avec POST action=quota et user=%username%. Une commande est renvoyée :
 * ouverture dans un navigateur d'une page d'info ?
 *
 */
function disk_quota($user)
{
    $quota = $out = $val = array();
    exec("sudo quota   -u $user  -F xfs -p -v --show-mntpoint --hide-device", $out);
    if (count($out) > 2) {
        for ($l = 2; $l <= 3; $l ++) {
            $overfill = false;
            $val = preg_split("/\s+/", $out[$l]);
            if (preg_match("/^([0-9]+)\*$/", $val[2], $match)) {
                $val[2] = $match[1];
                $val[5] = floor(($val[5] - time()) / 86400);
                $overfill = true;
                if ($val[2] > $val[4]) {
                    $val[5] = 0;
                }
            }
            $quota[] = array(
                'nom' => $user,
                'utilise' => $val[2],
                'quotasoft' => $val[3],
                'quotahard' => $val[4],
                'grace' => $val[5],
                'partition' => $val[1],
                'overfill' => $overfill
            );
        }
    }
    return $quota;
}

function info_quota($part)
{
    $out = $val = $info = array();
    exec("sudo xfs_quota -x -c 'state -u' " . $part, $out);
    $info['partition'] = $part;
    $info['etat'] = false;
    $info['grace'] = 0;
    foreach ($out as $ligne) {
        $val = [];
        $val1 = [];
        if (preg_match("/^Blocks grace time: \[([0-9]+) days\]$/", $ligne, $val)) {
            $info['grace'] = $val[1];
        } elseif (preg_match("/^  Enforcement: (.*)$/", $ligne, $val1)) {
            if ($val1[1] == "ON") {
                $info['etat'] = true;
            } else {
                $info['etat'] = false;
            }
        }
    }
    return $info;
}

/**
 * retourne une tableau avec les quotas disque de tous les utilisateurs en dépassement
 * uniquement pour xfs !
 *
 * pas très rapide avec quota, utiliser xfs_quota va beaucoup vite
 *
 *
 * le tableau est caché 5 minutes (si possible)
 *
 *
 * @return array $table[$nom][]
 */
function repquota($store = false)
{
    $quotas = [];
    if (! $quotas = apcu_fetch('quotas')) {
        if (! $store) {
            return [];
        } else {
            $quotas = $m = array();
            $parts = [
                '/home',
                '/var/sambaedu'
            ];
            foreach ($parts as $part) {
                $out = $val1 = $val2 = array();
                exec("sudo xfs_quota -x -c 'report -a -N' " . $part, $out);
                foreach ($out as $ligne) {
                    if (preg_match("/^(.*)\[(.*)\]$/", $ligne, $val1)) {
                        $val2 = preg_split("/\s+/", $val1[1]);
                        if (! empty($val1[2]) && ($val1[2] != "--------") && ($val2[3] != 0)) {
                            if (preg_match("/^([0-9]+)\s+day.*/", $val1[2], $m)) {
                                $grace = $m[1];
                            } else {
                                $grace = 0;
                            }
                            $quotas[] = array(
                                'nom' => $val2[0],
                                'utilise' => $val2[1],
                                'quotasoft' => $val2[2],
                                'quotahard' => $val2[3],
                                'grace' => $grace,
                                'partition' => $part,
                                'overfill' => true
                            );
                        }
                    }
                }
            }
            apcu_store('quotas', $quotas, 300);
        }
    }
    return $quotas;
}

/**
 * liste les quotas des utilisateurs d'un groupe pour une partition
 *
 * @param unknown $config
 * @param unknown $group
 * @param unknown $part
 */
function group_quota($config, $group, $part)
{
    $quotas = array();
    // $list = search_group($config, $group);
    $list = search_people_group($config, $group);
    foreach ($list as $user) {
        $quota = disk_quota($user['cn']);
        foreach ($quota as $ligne) {
            if ($ligne['partition'] == $part) {
                $quotas[] = $ligne;
            }
        }
    }
    return $quotas;
}

/**
 * retourne un tableau avec tous les quotas enregistrés dans la base pour la partition.
 *
 * @param unknown $config
 * @param unknown $part
 * @return unknown[]
 */
function list_quotas($config, $part)
{
    $authlink = db_connect($config);
    $query = mysqli_prepare($authlink, "SELECT nom, quotasoft, quotahard, `partition` FROM quotas WHERE `partition`=? ORDER BY nom DESC");
    mysqli_stmt_bind_param($query, "s", $part);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_nom, $res_quotasoft, $res_quotahard, $res_partition);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $quotas = array();
    if ($num_rows != 0) {
        while (mysqli_stmt_fetch($query)) {
            $quotas[] = array(
                "nom" => $res_nom,
                "quotasoft" => $res_quotasoft,
                "quotahard" => $res_quotahard,
                "partition" => $res_partition
            );
        }
    }
    mysqli_stmt_close($query);
    return $quotas;
}

/**
 * calcule le quota effectif a partir de la base mysql
 *
 * - si un quota utilisateur est defini il gagne (0 = pas de quota)
 * - sinon le plus grand groupe d'appartenance gagne (0 = pas de quota)
 *
 * CREATE TABLE IF NOT EXISTS `quotas` (
 * `nom` varchar(30) default NULL,
 * `quotasoft` mediumint(9) default NULL,
 * `quotahard` mediumint(9) default NULL,
 * `partition` varchar(20) default NULL
 * ) ENGINE=MyISAM DEFAULT CHARSET=utf8;
 *
 *
 * @param unknown $config
 * @param string $name
 * @param string $part
 * @return string[]|number[]|unknown
 */
function get_quota($config, string $name, string $part)
{
    require_once "ent.inc.php";
    $authlink = db_connect($config);
    $quota = array(
        'nom' => "defaut",
        'partition' => $part,
        'quotasoft' => 0,
        'quotahard' => 0
    );
    $noms = array(
        $name
    );
    $user = search_user($config, $name);
    if (count($user) > 0) {
        foreach ($user['memberof'] as $groupdn) {
            $noms[] = matiere_name(ldap_dn2cn($groupdn));
        }
    }
    $query = "SELECT * FROM quotas WHERE `partition`='" . $part . "' AND nom IN (";
    for ($k = 0; $k < count($noms); $k ++) {
        $query .= "'" . $noms[$k] . "'";
        if ($k < count($noms) - 1) {
            $query .= ", ";
        }
    }
    $query .= ");";
    if ($res = mysqli_query($authlink, $query)) {
        $quota = array(
            'nom' => "defaut",
            'partition' => $part,
            'quotasoft' => 0,
            'quotahard' => 0
        );
        while ($row = mysqli_fetch_assoc($res)) {
            if ($row['nom'] == $name) { // quota explicite utilisateur
                $quota = $row;
                break;
            } elseif ($row['quotahard'] == 0) { // pas de quota : héritage du groupe
                $quota = $row;
                break;
            } elseif ($row['quotahard'] >= $quota['quotahard']) { // heritage du plus grand quota de groupe
                $quota = $row;
            }
        }
        mysqli_free_result($res);
    }
    mysqli_close($authlink);
    return $quota;
}

/**
 * fixe le quota xfs
 *
 * @param unknown $config
 * @param unknown $quota
 * @return boolean
 */
function set_quota_disk($config, $quota)
{
    $ret = false;
    if (isset($quota['grace'])) {
        system("sudo xfs_quota -x -c 'timer -u " . $quota['grace'] . "d' " . $quota['partition'], $ret);
    } else {
        system("sudo xfs_quota -x -c 'limit -u bsoft=" . $quota['quotasoft'] . "m bhard=" . $quota['quotahard'] . "m " . $quota['nom'] . "' " . $quota['partition'], $ret);
    }
    if ($ret == 0) {
        $ret = true;
    }
    return $ret;
}

/**
 * fixe le quota dans la base pour l'utilisateur ou le groupe
 * quotahard = 0 -> on force pas de quota
 * quotahard = -1 -> suppression du quota (l'heritage s'applique)
 * un quota utilisateur est prioritaire sur les quotas herités des groupes
 *
 * @param unknown $config
 * @param array $new_quota
 */
function set_quota($config, array $new_quota)
{
    $authlink = db_connect($config);

    $quota = get_quota($config, $new_quota['nom'], $new_quota['partition']);
    if ($new_quota['quotahard'] == - 1) {
        // on veut supprimer le quota explicite de l'utilisateur
        $query = mysqli_prepare($authlink, "DELETE FROM quotas WHERE nom=? AND `partition`=?");
        mysqli_stmt_bind_param($query, "ss", $quota['nom'], $quota['partition']);
        mysqli_stmt_execute($query);
        mysqli_stmt_close($query);
    } else {
        $quota = get_quota($config, $new_quota['nom'], $new_quota['partition']);

        if ($quota != $new_quota) {

            if ($quota['nom'] == $new_quota['nom']) {
                $query = mysqli_prepare($authlink, "UPDATE quotas SET quotasoft=?, quotahard=? WHERE nom=? AND `partition`=?");
                mysqli_stmt_bind_param($query, "iiss", $new_quota['quotasoft'], $new_quota['quotahard'], $quota['nom'], $quota['partition']);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
            } else {
                $query = mysqli_prepare($authlink, "INSERT INTO quotas VALUES (?,?,?,?)");
                mysqli_stmt_bind_param($query, "siis", $new_quota['nom'], $new_quota['quotasoft'], $new_quota['quotahard'], $quota['partition']);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
            }
        }
    }
    mysqli_close($authlink);

    $group = search_people_group($config, $new_quota['nom']);
    if (count($group) > 0) {
        // groupe, il faut actualiser les quotas pour tous les utilisateurs du groupe
        foreach ($group as $user) {
            $user = $user['cn'];
            $quota = get_quota($config, $user, $new_quota['partition']);
            $quota['nom'] = $user;
            set_quota_disk($config, $quota);
        }
    } else {
        $quota = get_quota($config, $new_quota['nom'], $new_quota['partition']);
        set_quota_disk($config, $quota);
    }
}

function affiche_quotas($quotas, $partit)
{
    if ($partit == "/home") {
        $disque = "K";
    } else {
        $disque = "U";
    }
    echo "<h2>Liste des quotas utilisateurs sur $partit : <u onmouseover=\"this.T_SHADOWWIDTH=5;this.T_STICKY=1;return escape('Visualisation de tous les utilisateurs en période de gr&#226;ce (orange) ou ayant leur période de gr&#226;ce expirée (rouge): ces derniers ne peuvent plus rien écrire sur $partit d\'o&#151; maints dysfontionnements possibles...')\"><img name=\"action_image2\"  src=\"../elements/images/system-help.png\"></u></h2>";
    echo "<TABLE align='center' class='table-bordered'>\n";
    echo "<TR><TD class='menuheader'> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Utilisateur&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; </TD>
			<TD class='menuheader'> &nbsp;&nbsp;&nbsp;Espace utilisé &nbsp;(Mo)&nbsp;&nbsp;&nbsp; </TD>
			<TD class='menuheader'> &nbsp;&nbsp;&nbsp;&nbsp;Quota fixé &nbsp;(Mo)&nbsp;&nbsp;&nbsp;</TD>
			<TD class='menuheader'> &nbsp;&nbsp;&nbsp;Délai de gr&#226;ce &nbsp;(Jours)&nbsp;&nbsp;&nbsp; </TD></TR>";
    foreach ($quotas as $user) {

        if (($partit == "/home") && ($user['partition'] == "/home")) {
            $repere1 = "popuprecherche('../annu/people.php?cn=";
            $repere2 = "','popuprecherche','width=800,height=500');";
            $quotasoft = round($user['quotasoft'] / 1024);
            if (! $user['overfill'] && ($quotasoft == 0)) {
                $quotasoft = "Aucun";
            }
            echo "<tr align='center'><td>
                <a name=ancre_" . $user['nom'] . "_alerte href=#ancre_" . $user['nom'] . "_alerte onclick=" . $repere1 . $user['nom'] . $repere2 . "> " . $user['nom'] . "</a></td>
                <td> " . round($user['utilise'] / 1024) . "</td>
                <td> " . $quotasoft . "</td>";
            if ($user['overfill']) {
                if ((int) $user['grace'] == "0") {
                    $color = "#FF0000";
                } else {
                    $color = "#FF8C00";
                }
                echo "<td bgcolor=" . $color . ">" . $user['grace'] . "</td></tr>";
            } else {
                echo "<td> OK </td></tr>";
            }
            // exec("sed -i /tmp/result_quota -e \"s!REPERE1 !popuprecherche(\'stats_user.php?partition=$partit\&uid=!\"");
            // exec("sed -i /tmp/result_quota -e \"s! REPERE2!','popuprecherche','width=800,height=500');!\"");
            // system("cat /tmp/result_quota");
            // exec("rm /tmp/result_quota");
        } elseif (($partit == "/var/sambaedu") && ($user['partition'] == "/var/sambaedu")) {
            $quotasoft = round($user['quotasoft'] / 1024);
            echo "<tr align='center'><td>
                <a " . $user['nom'] . "> " . $user['nom'] . "</a></td>
                <td> " . round($user['utilise'] / 1024) . "</td>
                <td> " . $quotasoft . "</td>";
            if ($user['overfill']) {
                if ((int) $user['grace'] == "0") {

                    $color = "#FF0000";
                } else {
                    $color = "#FF8C00";
                }
                echo "<td bgcolor=" . $color . ">" . $user['grace'] . "</td></tr>";
            } else {
                echo "<td> OK </td></tr>";
            }
        }
    }
    echo "</table>";
}

?>
