<?php

/**
 * Fonctions pour l'import ENT
 *
 * @Projet SambaEdu
 * @Auteurs Equipe Sambaedu
 * @Licence Distribue sous la licence GPL
 * @Repertoire: includes/
 * file: ent.inc.php
 *
 * /
 * /**
 *
 * remplace les accents
 *
 * @Parametres chaine a traiter
 * @return la chaine sans accents
 *
 */
function remplace_accents($chaine)
{
    setlocale(LC_ALL, 'fr_FR.UTF8');
    $retour = iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE', $chaine);
    $retour = preg_replace("/\?/", "", "$retour");
    return $retour;
}

/**
 * Cette méthode prend une chaîne de caractères et s'assure qu'elle est bien retournée en UTF-8
 * Attention, certain encodages sont très similaire et ne peuve pas être théoriquement distingué sur une chaine de caractere.
 * Si vous connaissez déjà l'encodage de votre chaine de départ, il est préférable de le préciser
 *
 * @param string $str
 *            La chaine à encoder
 * @param string $encoding
 *            L'encodage de départ
 * @return string La chaine en utf8
 * @throws Exception si la chaine n'a pas pu être encodée correctement
 */
function ensure_utf8($str, $from_encoding = null)
{
    if ($str === null || $str === '') {
        return $str;
    } else if ($from_encoding == null && detect_utf8($str)) {
        return $str;
    }

    if ($from_encoding != null) {
        $encoding = $from_encoding;
    } else {
        $encoding = detect_encoding($str);
    }
    $result = null;
    if ($encoding !== false && $encoding != null) {
        if (function_exists('mb_convert_encoding')) {
            $result = mb_convert_encoding($str, 'UTF-8', $encoding);
        }
    }
    if ($result === null || ! detect_utf8($result)) {
        throw new Exception('Impossible de convertir la chaine vers l\'utf8');
    }
    return $result;
}

/**
 * Cette méthode prend une chaîne de caractères et teste si elle ne contient que
 * de l'ASCII 7 bits ou si elle contient au moins une suite d'octets codant un
 * caractère en UTF8
 *
 * @param string $str
 *            La chaine à tester
 * @return boolean
 */
function detect_utf8($str)
{
    // Inspiré de http://w3.org/International/questions/qa-forms-utf-8.html
    //
    // on s'assure de bien opérer sur une chaîne de caractère
    $str = (string) $str;
    // La chaîne ne comporte que des octets <= 7F ?
    $full_ascii = true;
    $i = 0;
    while ($full_ascii && $i < strlen($str)) {
        $full_ascii = $full_ascii && (ord($str[$i]) <= 0x7F);
        $i ++;
    }
    // Si oui c'est de l'utf8 sinon on cherche si la chaîne contient
    // au moins une suite d'octets valide en UTF8
    if ($full_ascii)
        return true;
    else
        return preg_match('#[\xC2-\xDF][\x80-\xBF]#', $str) || // non-overlong 2-byte
        preg_match('#\xE0[\xA0-\xBF][\x80-\xBF]#', $str) || // excluding overlongs
        preg_match('#[\xE1-\xEC\xEE\xEF][\x80-\xBF]{2}#', $str) || // straight 3-byte
        preg_match('#\xED[\x80-\x9F][\x80-\xBF]#', $str) | // excluding surrogates
        preg_match('#\xF0[\x90-\xBF][\x80-\xBF]{2}#', $str) || // planes 1-3
        preg_match('#[\xF1-\xF3][\x80-\xBF]{3}#', $str) || // planes 4-15
        preg_match('# \xF4[\x80-\x8F][\x80-\xBF]{2}#', $str); // plane 16
}

/**
 * Cette méthode prend une chaîne de caractères et détecte son encodage
 *
 * @param string $str
 *            La chaine à tester
 * @return l'encodage ou false si indétectable
 */
function detect_encoding($str)
{
    // on commence par vérifier si c'est de l'utf8
    if (detect_utf8($str)) {
        return 'UTF-8';
    }

    // on va commencer par tester ces encodages
    static $encoding_list = array(
        'UTF-8',
        'ISO-8859-15',
        'windows-1251'
    );
    foreach ($encoding_list as $item) {
        if (function_exists('iconv')) {
            $sample = @iconv($item, $item, $str);
            if (md5($sample) == md5($str)) {
                return $item;
            }
        } else if (function_exists('mb_detect_encoding')) {
            if (@mb_detect_encoding($str, $item, true)) {
                return $item;
            }
        }
    }

    // la méthode précédente n'a rien donnée
    if (function_exists('mb_detect_encoding')) {
        return mb_detect_encoding($str);
    } else {
        return false;
    }
}

/**
 * Cette méthode prend une chaîne de caractères et s'assure qu'elle est bien retournée en ASCII
 * Attention, certain encodages sont très similaire et ne peuve pas être théoriquement distingué sur une chaine de caractere.
 * Si vous connaissez déjà l'encodage de votre chaine de départ, il est préférable de le préciser
 *
 * @param string $chaine
 *            La chaine à encoder
 * @param string $encoding
 *            L'encodage de départ
 * @return string La chaine en ascii
 */
function ensure_ascii($chaine, $encoding = '')
{
    if ($chaine == null || $chaine == '') {
        return $chaine;
    }

    $chaine = ensure_utf8($chaine, $encoding);
    $str = null;
    if (function_exists('iconv')) {
        // test : est-ce que iconv est bien implémenté sur ce système ?
        $test = 'c\'est un bel ete' === iconv("UTF-8", "ASCII//TRANSLIT//IGNORE", 'c\'est un bel été');
        if ($test) {
            // on utilise iconv pour la conversion
            $str = @iconv("UTF-8", "ASCII//TRANSLIT//IGNORE", $chaine);
        }
    }
    if ($str === null) {
        // on utilise pas iconv pour la conversion
        $translit = array(
            'Á' => 'A',
            'À' => 'A',
            'Â' => 'A',
            'Ä' => 'A',
            'Ã' => 'A',
            'Å' => 'A',
            'Ç' => 'C',
            'É' => 'E',
            'È' => 'E',
            'Ê' => 'E',
            'Ë' => 'E',
            'Í' => 'I',
            'Ï' => 'I',
            'Î' => 'I',
            'Ì' => 'I',
            'Ñ' => 'N',
            'Ó' => 'O',
            'Ò' => 'O',
            'Ô' => 'O',
            'Ö' => 'O',
            'Õ' => 'O',
            'Ú' => 'U',
            'Ù' => 'U',
            'Û' => 'U',
            'Ü' => 'U',
            'Ý' => 'Y',
            'á' => 'a',
            'à' => 'a',
            'â' => 'a',
            'ä' => 'a',
            'ã' => 'a',
            'å' => 'a',
            'ç' => 'c',
            'é' => 'e',
            'è' => 'e',
            'ê' => 'e',
            'ë' => 'e',
            'í' => 'i',
            'ì' => 'i',
            'î' => 'i',
            'ï' => 'i',
            'ñ' => 'n',
            'ó' => 'o',
            'ò' => 'o',
            'ô' => 'o',
            'ö' => 'o',
            'õ' => 'o',
            'ú' => 'u',
            'ù' => 'u',
            'û' => 'u',
            'ü' => 'u',
            'ý' => 'y',
            'ÿ' => 'y'
        );
        $str = strtr($chaine, $translit);
    }
    if (function_exists('mb_convert_encoding')) {
        $str = @mb_convert_encoding($str, 'ASCII', 'UTF-8');
    }
    return $str;
}

/**
 * Le csv format utilisé correspond à celui utilisé par l'ENT Openent :
 * première ligne avec les champs :
 * "externalId";"Id Siecle";"type";"lastname";"firstName";"originalLogin";"login","code";"functions";"structures";"allClasses";"childrens";"parents";"birthDate";"sex";"allGroups"
 * id : identifiant unique ENT
 * Id Siecle : numero eleve siecle (eleves uniquement)
 * + Nom : nom en majuscule => TEST
 * + Prénom : prénom => Elève
 * + Type : Student|Teacher|Personnel|Relative|Guest
 * originalLogin : login d'origine, prenom.nom(x) x étant un nombre pour les doublons.
 * login : login simplifié chois par l'utilisateur => repris en priorité par Sambaedu
 * code : mot de passe initial => repris par sambaedu si il existe
 * functions : Personnel administatif => array()
 * structures : nom de(s) l'établissemnt(s) en majuscules ,séparés par des virgules => array()
 * + allClasses : classe(s), séparées par des virgules => array()
 * allGroups : groupe(s), séparés par des virgules
 * birthDate : date de naissance AAAAMMJJ
 * sex : sexe M/F
 *
 * seuls les champs avec + sont obligatoires
 *
 * Des mappages sont possibles pour d'autres types d'imports, voir ci-dessous.
 * La même structure est possible en json ou xml
 */
function map_ent_fields($keys, &$type = "")
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    if (empty($type))
        $type = "csv";
    $map['csv_openent'] = array(
        "Id" => "externalId",
        "Identifiant SIECLE" => "Id Siecle",
        "Type" => "type",
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "Identifiant" => "originalLogin",
        "Login pesonnalisé (Alias)" => "login",
        "Code d'activation" => "code",
        "Fonctions" => "functions",
        "Établissements" => "structures",
        "Classes" => "allClasses",
        "Matieres" => "allMatieres",
        "Enfants" => "childrens",
        "Parents" => "parents"
    );
    $map['csv_pronote'] = array(
        "NOM" => "lastName",
        "PRENOM" => "firstName",
        "LOGIN" => "pronoteLogin",
        "MDP" => "code",
        "CLASSES" => "allClasses",
        "GROUPES" => "allGroups",
        // "FONCTION" => "type",
        "STATUT" => "type",
        "EMAIL" => "email",
        "CIVILITE" => "sex",
        "DATE_NAISS" => "birthDate",
        "SEXE" => "sex",
        "MAT ENSEIGN" => "allMatieres"
    );

    $map['csv_pronote_CAS'] = array(
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "Identifiant PRONOTE-ENT" => "Id Pronote"
    );

    $map['csv_pronote_eleve'] = array(
        "NOM" => "lastName",
        "PRENOM" => "firstName",
        "EMAIL" => "email",
        "CIVILITE" => "sex",
        "DATE_NAISS" => "birthDate",
        "N NATIONAL" => "Id Siecle",
        "CLASSE" => "allClasses",
        "GROUPES" => "allGroups",
        "SEXE" => "sex"
    );

    $map['kosmos'] = array(
        "Identifiant ENT" => "externalId",
        "sexe" => "sex",
        "année de naissance" => "birthDate",
        "Profil" => "type",
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "classe" => "allClasses"
    );
    $map['csv_oZe'] = array(
        "Login" => "email",
        "Profils" => "type",
        "Nom" => "lastName",
        "Prenom" => "firstName",
        "Classes" => "allClasses",
        "MDP" => "code"
    );
    $map['asm_student'] = array(
        "sis_username" => "login",
        "person_id" => "Id ASM",
        "student_id" => "Id ASM",
        "instructor_id" => "Id ASM",
        "person_number" => "Id GPEI",
        "first_name" => "firstName",
        "last_name" => "lastName",
        "location_id" => "structure",
        "email_address" => "email"
    );
    $map['asm_teacher'] = array(
        "sis_username" => "login",
        "person_id" => "Id ASM",
        "instructor_id" => "Id ASM",
        "person_number" => "Id GPEI",
        "first_name" => "firstName",
        "last_name" => "lastName",
        "location_id" => "structure",
        "email_address" => "email"
    );
    $map['csv'] = array(
        "sexe" => "sex",
        "année de naissance" => "birthDate",
        "Profil" => "type",
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "classe" => "allClasses"
    );
    $map['csv_siecle'] = array(
        "Id" => "externalId",
        "Identifiant SIECLE" => "Id Siecle",
        "Type" => "type",
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "Classes" => "allClasses",
        "Groupes" => "allGroups",
        "Date de naissance" => "birthDate",
        "Sexe" => "sex"
    );
    $map['csv_sts'] = array(
        "Id" => "externalId",
        "Type" => "type",
        "Nom" => "lastName",
        "Prénom" => "firstName",
        "Classes" => "allClasses",
        "Date de naissance" => "birthDate",
        "Sexe" => "sex"
    );

    $ret = array();
    foreach ($keys as $index => $key) {
        // détection d'un fichier pronote eleve
        if ($type == "csv_pronote" && $key == "CLASSE") {
            $type = "csv_pronote_eleve";
        }
    }
    foreach ($keys as $index => $key) {
        if (isset($map[$type][$key])) {
            $ret[$index] = $map[$type][$key];
        } else {
            $ret[$index] = $key;
        }
    }
    return $ret;
}

function map_profil($profil, $type = "", $reverse = false)
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    if (empty($type))
        $type = "csv";
    $map['csv_openent'] = array(
        "Elève" => "Student",
        "Élève" => "Student",
        "Enseignant" => "Teacher",
        "Personnel" => "Personnel",
        "Invité" => "Guest",
        "Parent" => "Relative"
    );
    $map['kosmos'] = array(
        "Eleve" => "Student",
        "Professeur" => "Teacher",
        "Personnel" => "Personnel",
        "Parent" => "Relative",
        "Autre" => "Personnel"
    );
    $map['csv'] = array(
        "Eleve" => "Student",
        "Professeur" => "Teacher",
        "Personnel" => "Personnel"
    );
    $map['csv_pronote'] = array(
        "ENS" => "Teacher",
        "Définitif" => "Teacher"
    );
    $map['csv_oZe'] = array(
        "Elève" => "Student",
        "Enseignant" => "Teacher",
        "Administratif" => "Personnel"
    );
    $map['manuel'] = array(
        "Eleves" => "Student",
        "Profs" => "Teacher",
        "Administratifs" => "Personnel"
    );
    $map['csv_siecle'] = array(
        "ELEVE" => "Student"
    );
    $map['csv_sts'] = array(
        "ENS" => "Teacher",
        "DIR" => "Personnel",
        "EDU" => "Personnel"
    );
    if ($reverse) {
        $map[$type] = array_flip($map[$type]);
    }
    if (isset($map[$type][$profil])) {
        $ret = $map[$type][$profil];
    } else {
        $ret = $profil;
    }
    return $ret;
}

function map_classe($classes, $type = "")
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    $ret = array();
    if (empty($type)) {
        $type = "csv";
    }
    if ($type == "csv_openent" || $type == "csv" || $type == "csv_group" || $type == "csv_oZe" || $type == "csv_pronote" || $type == "csv_pronote_eleve" || $type == "csv_siecle" || $type == "csv_sts") {
        if (count($classes) > 0) {
            foreach ($classes as $classe) {
                $ret[]['name'] = trim($classe);
            }
        }
        if ($type == "csv_pronote_eleve") {
            $ret[]['name'] = trim($classes[0]);
        }
    } elseif ($type == "kosmos") {
        if (count($classes) > 0) {
            foreach ($classes as $classe) {
                $classe = explode("~", $classe);
                if (isset($classe[1]))
                    $ret[]['name'] = $classe[1];
            }
        }
    }
    return $ret;
}

/**
 * mise en correspondance des noms de matières.
 *
 * l'export EDT raccourcit les noms, ce qui provoque des doublons si les amtières existent par ailleurs.
 * Ne pas utiliser dans le cas général; Cette fonction ne devrait être utilisée que pour l'import EDT
 *
 * @param unknown $config
 * @param unknown $ent
 */
function map_matiere($config, $list, $type = "")
{
    $mat = [];
    foreach ($list as $ent) {
        $ent = matiere_name(trim($ent));
        if ($type == "csv_pronote") {
            $matieres = cache_fetch('matieres');
            if (! isset($matieres[$ent])) {
                $matieres[$ent] = $ent;
                foreach (search_ad($config, "*", "matiere") as $m) {
                    if (strlen($ent) > 8 && strlen($ent) < strlen($m['cn'])) {
                        $match = ($ent == substr($m['cn'], 0, strlen($ent)));
                    } else {
                        $match = ($ent == $m['cn']);
                    }
                    if ($match) {
                        $matieres[$ent] = $m['cn'];
                        $mat[] = $m['cn'];
                        break;
                    }
                }
                cache_store('matieres', $matieres);
            } else {
                $mat[]['name'] = $matieres[$ent];
            }
            return $mat;
        } else {
            $mat[]['name'] = $ent;
        }
    }
    return $mat;
}

function map_groups($groups, $type = "")
{
    // table de correspondance :
    // on ne prend pas en compte les parties de classe .p1-.p9
    $ret = array();
    if (empty($type)) {
        $type = "csv";
        foreach ($groups as $group) {
            if (! preg_match("/.*[0-9]p\.?[1-3]/i", $group))
                $ret[]['name'] = trim($group);
        }
    }
    return $ret;
}

function map_birthdate($date, $type = "")
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    // format par défaut AAAAMMJJ
    // la fonction fait la conversion vers se format si besoin
    if (empty($type)) {
        $type = "csv";
    }
    switch ($type) {
        case "csv_pronote":
        case "csv_pronote_eleve":
        case "gpei":
        case "siecle":
            $ddn = explode("/", $date);
            return implode("", array_reverse($ddn));
            break;
        default:
            // AAAA-MM-JJ
            return preg_replace("/-/", "", $date);
    }
}

function map_sex($sex)
{
    // table de correspondance :
    // champ dans l'ent d'origine => champ utilisé pour le traitement sambaedu
    switch ($sex) {
        case "Mme":
        case "Mlle":
        case "F":
        case "MM":
        case "2": // siecle_sts
            $sex = "F";
            break;
        default:
            $sex = "M";
    }
    return $sex;
}

function cmp_classe($a, $b)
{
    $ca = count($a['allClasses']);
    $cb = count($b['allClasses']);
    if ($ca > $cb)
        return - 1;
    elseif ($cb > $ca)
        return 1;
    else
        return 0;
}

function remplace_separateurs($chaine)
{
    $retour = strtr($chaine, array(
        "'" => ' ',
        '-' => ' ',
        '_' => ' '
    ));

    return $retour;
}

/**
 * * simplifie le nom
 * - enleve l'apostrophe, les tirets...
 * - colle les particules
 * -
 *
 * @param string $name
 * @return string
 */
function simplify_name($name = "", $short = false)
{
    $name = strtolower(remplace_accents($name));
    $name = remplace_separateurs($name);
    $length = strlen($name);
    $mots = explode(" ", $name);

    if (strlen($mots[0]) < 3 && ! empty($mots[1])) {
        // on colle la particule (l el le al n mc....)
        $mots[1] = $mots[0] . $mots[1];
        array_shift($mots);
        $length --;
    }
    foreach ($mots as $n => $mot) {
        if (empty($mot)) {
            unset($mots[$n]);
        }
    }
    if ($length < 12) {
        if ($short) {
            $name = substr(implode("", $mots), 0, 12);
        } else {
            $name = substr(implode("-", $mots), 0, 12);
        }
    } else {
        $name = substr(array_shift($mots), 0, 12);
    }
    return $name;
}

function simplify_firstname($name = "", $short = false, $openent = false)
{
    $name = strtolower(remplace_accents($name));
    $name = remplace_separateurs($name);
    $length = strlen($name);
    $mots = explode(" ", $name);

    if (strlen($mots[0]) < 3 && ! empty($mots[1])) {
        // on colle la particule (l el le al n mc....)
        $mots[1] = $mots[0] . $mots[1];
        array_shift($mots);
        $length --;
    }
    foreach ($mots as $n => $mot) {
        if (empty($mot)) {
            unset($mots[$n]);
        }
    }
    if ($short) {
        $name = "";
        foreach ($mots as $mot) {
            $name .= substr($mot, 0, 1);
        }
    } else {
        if ($length < 12) {
            if ($openent) {
                $name = substr(implode("", $mots), 0, 12);
            } else {
                $name = substr(implode("-", $mots), 0, 12);
            }
        } else {
            $name = substr(array_shift($mots), 0, 12);
        }
    }
    return $name;
}

function create_key()
{
    if (! file_exists("/etc/sambaedu/sambaedu-pubkey.pem")) {
        // Configuration settings for the key
        $config = array(
            "private_key_bits" => 512,
            "private_key_type" => OPENSSL_KEYTYPE_RSA
        );

        // Create the private and public key
        $res = openssl_pkey_new($config);

        // Extract the private key into $private_key
        openssl_pkey_export($res, $private_key);
        file_put_contents("/etc/sambaedu/sambaedu-privkey.pem", $private_key);
        file_put_contents("/etc/sambaedu/sambaedu-pubkey.pem", openssl_pkey_get_details($res)['key']);
    }
}

function encode_birthdate(string $attribute)
{
    create_key();
    $public_key = file_get_contents("/etc/sambaedu/sambaedu-pubkey.pem");
    $encrypted = "";
    openssl_public_encrypt($attribute, $encrypted, $public_key);
    return base64_encode($encrypted);
}

function decode_birthdate(string $encrypted)
{
    if (empty($encrypted)) {
        return "";
    }
    create_key();
    $private_key = file_get_contents("/etc/sambaedu/sambaedu-privkey.pem");
    $attribute = "";
    openssl_private_decrypt(base64_decode($encrypted), $attribute, $private_key);
    return $attribute;
}

function doublon_cn($cn)
{
    $ret = [
        'cn' => $cn,
        'nb' => ""
    ];
    if (preg_match("/^([a-z-.]*)([0-9]*)/", $cn, $m)) {
        $ret['cn'] = $m[1];
        $ret['nb'] = $m[2];
    }
    return $ret;
}

function mk_cn($config, $nom, $prenom, $cnPolicy = "", $nb = 0)
{
    /*
     * # Valeurs de l'cnPolicy
     * # 0: prenom.nom tronqué a 19
     * # 1: prenom.nom tronque a 19 sans -
     * # 2: p.nom tronque a 15
     * # 3: pnom tronque a 8
     * # 4: nomp tronque a 8
     * # 5: nomprenom tronque a 18
     */
    if (empty($cnPolicy))
        $cnPolicy = $config['cnPolicy'];

    switch ($cnPolicy) {
        case 0:
            $nom = simplify_name($nom);
            $prenom = simplify_firstname($prenom);
            if ($nb > 0) {
                $nom .= $nb;
            }
            if (strlen($nom) + strlen($prenom) > 18) {
                $prenom = substr($prenom, 0, 18 - strlen($nom));
            }
            if (! empty($prenom)) {
                $cn = $prenom . "." . $nom;
            } else {
                $cn = $nom;
            }
            break;
        case 1: // suppression des - pour openent
            $nom = simplify_name($nom, true);
            $prenom = simplify_firstname($prenom, false, true);
            if ($nb > 0) {
                $nom .= $nb;
            }
            if (strlen($nom) + strlen($prenom) > 18) {
                $prenom = substr($prenom, 0, 18 - strlen($nom));
            }
            if (! empty($prenom)) {
                $cn = $prenom . "." . $nom;
            } else {
                $cn = $nom;
            }
            break;
        case 2:
            $nom = simplify_name($nom);
            $prenom = simplify_firstname($prenom, true);

            if ($nb > 0) {
                $nom .= $nb;
            }
            if (! empty($prenom)) {
                $cn = $prenom . "." . $nom;
            } else {
                $cn = $nom;
            }
            break;
        case 3:
            $nom = simplify_name($nom, true);
            $prenom = simplify_firstname($prenom, true);

            if ($nb == 0) {
                $debut_nom = substr($nom, 0, 8 - strlen($prenom));
            } else {
                $debut_nom = substr($nom, 0, (8 - strlen($prenom) - strlen($nb))) . $nb;
            }
            if (! empty($prenom)) {
                $cn = $prenom . $debut_nom;
            } else {
                $cn = $debut_nom;
            }
            break;
        case 4:
            $nom = simplify_name($nom, true);
            $prenom = simplify_firstname($prenom, true);

            if ($nb == 0) {
                $debut_nom = substr($nom, 0, 8 - strlen($prenom));
            } else {
                $debut_nom = substr($nom, 0, (8 - strlen($prenom) - strlen($nb)));
            }
            if (! empty($prenom)) {
                $cn = $debut_nom . $prenom;
            } else {
                $cn = $nom;
            }
            if ($nb > 0) {
                $cn .= $nb;
            }
            break;
        case 5:
            $nom = simplify_name($nom, true);
            $prenom = simplify_firstname($prenom, false, true);

            if ($nb > 0) {
                if (strlen($nom) + strlen($prenom) + strlen($nb) > 17) {
                    $prenom = substr($prenom, 0, 17 - strlen($nom) - strlen($nb));
                }
                $cn = $nom . $prenom . $nb;
            } else {
                if (strlen($nom) + strlen($prenom) > 17) {
                    $prenom = substr($prenom, 0, 17 - strlen($nom));
                }
                $cn = $nom . $prenom;
            }
            break;
        default:
            $cn = false;
    }

    // Pour faire disparaitre les caracteres speciaux restants:
    $cn = strtolower(preg_replace("/[^a-z0-9.-]/", "", $cn));

    // Pour eviter les _ en fin d'UID... pb avec des connexions machine de M$7
    $cn = preg_replace("/_*$/", "", $cn);
    return $cn;
}

function oauth_login($config)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'clientId' => $config['openent_oauth2_id'], // The client ID assigned to you by the provider
        'clientSecret' => $config['openent_oauth2_pass'], // The client password assigned to you by the provider
        'redirectUri' => $config['openent_oauth2_redirect_uri'],
        'urlAuthorize' => $config['openent_uri'] . $config['openent_oauth2_auth'],
        'urlAccessToken' => $config['openent_uri'] . $config['openent_oauth2_token'],
        'urlResourceOwnerDetails' => $config['openent_uri'] . $config['openent_oauth2_userinfo'],
        'scopes' => [
            'userinfo'
        ]
    ];
    set_oauth_proxy($config, $opt);
    $provider = new \League\OAuth2\Client\Provider\GenericProvider($opt);
    // Fetch the authorization URL from the provider; this returns the
    // urlAuthorize option and generates and applies any necessary parameters
    // (e.g. state).
    try {
        $authorizationUrl = $provider->getAuthorizationUrl();
        $accessToken = $provider->getAccessToken('authorization_code', [
            'code' => $_GET['code']
        ]);

        // Get the state generated for you and store it to the session.
        $_SESSION['oauth2state'] = $provider->getState();

        return $authorizationUrl;
    } catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {

        // Failed to get the access token or user details.
        return "erreur de connexion Oauth2 : " . $e->getMessage();
    }
}

function oauth_callback($config)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'clientId' => $config['openent_oauth2_id'], // The client ID assigned to you by the provider
        'clientSecret' => $config['openent_oauth2_pass'], // The client password assigned to you by the provider
        'redirectUri' => $config['openent_oauth2_redirect_uri'],
        'urlAuthorize' => $config['openent_uri'] . $config['openent_oauth2_auth'],
        'urlAccessToken' => $config['openent_uri'] . $config['openent_oauth2_token'],
        'urlResourceOwnerDetails' => $config['openent_uri'] . $config['openent_oauth2_userinfo'],
        'scopes' => [
            'userinfo'
        ]
    ];
    set_oauth_proxy($config, $opt);

    $provider = new \League\OAuth2\Client\Provider\GenericProvider($opt);

    // If we don't have an authorization code then get one

    if (! isset($_GET['code'])) {

        // Fetch the authorization URL from the provider; this returns the
        // urlAuthorize option and generates and applies any necessary parameters
        // (e.g. state).
        $authorizationUrl = $provider->getAuthorizationUrl();

        // Get the state generated for you and store it to the session.
        $_SESSION['oauth2state'] = $provider->getState();

        // Redirect the user to the authorization URL.
        header('Location: ' . $authorizationUrl);
        exit();

        // Check given state against previously stored one to mitigate CSRF attack
    } elseif (empty($_GET['state']) || (isset($_SESSION['oauth2state']) && $_GET['state'] !== $_SESSION['oauth2state'])) {

        // $html .="state : " . $_GET['state'] . "<br>";
        // $html .="<pre>" . print_r($_SESSION, true) . "</pre>";
        unset($_SESSION['oauth2state']);
        exit('Invalid state');
    } else {

        try {
            $time = time();
            $existingAccessToken = apcu_fetch($_GET['state']);

            // $existingAccessToken = $_SESSION['accesstoken'] ?? false;
            if (! empty($existingAccessToken) && $existingAccessToken->hasExpired()) {
                $accessToken = $provider->getAccessToken('refresh_token', [
                    'refresh_token' => $existingAccessToken->getRefreshToken()
                ]);
            } else {
                // Try to get an access token using the authorization code grant.
                $accessToken = $provider->getAccessToken('authorization_code', [
                    'code' => $_GET['code']
                ]);
            }
            apcu_store($_GET['state'], $accessToken);
            $time = time() - $time;
            $message = "getAccessToken: " . $time . "s\n";
            error_log($message, 3, "/tmp/profil.log");
            // We have an access token, which we may use in authenticated
            // requests against the service provider's API.
            // $html .=$accessToken->getToken() . "\n";
            // $html .=$accessToken->getRefreshToken() . "\n";$global = $_POST['global'] ?? false;

            // $html .=$accessToken->getExpires() . "\n";
            // $html .=($accessToken->hasExpired() ? 'expired' : 'not expired') . "\n";

            // Using the access token, we may look up details about the
            // resource owner.
            $time = time();
            $resourceOwner = $provider->getResourceOwner($accessToken);
            $userinfo = $resourceOwner->toArray();
            $time = time() - $time;
            $message = "getResourceOwner: " . $time . "s\n";
            error_log($message, 3, "/tmp/profil.log");
            $time = time();
            // le login peut-il être différent de celui de l'ent ?
            // OUI ! incohérence entre userinfo, et directory, qui ne renvoient pas la même valeur. Du coup on est obligé d'aller chercgher dans l'ad
            $filter = "(&(objectclass=user)(|(title=*" . $userinfo['externalId'] . ")(cn=" . $userinfo['login'] . ")))";
            $user = search_ad($config, $filter, "filter", $config['dn']['people']);
            $time = time() - $time;
            $message = "search_ad: " . $time . "s\n";
            error_log($message, 3, "/tmp/profil.log");

            $_SESSION['login_ent'] = $userinfo['login'];
            // $_SESSION['login'] = $userinfo['login'];
            $_SESSION['login'] = $user[0]['cn'];
            $_SESSION['auth'] = "ent";
            // }
            $_SESSION['accesstoken'] = $accessToken;
            $_SESSION['hasPw'] = $userinfo['hasPw'] ?? NULL;
            ;
            $_SESSION['forceChangePassword'] = $userinfo['forceChangePassword'] ?? NULL;
            // header("Location:" . $config['se4_url']);
            header("Location:../index.html");
        } catch (\League\OAuth2\Client\Provider\Exception\IdentityProviderException $e) {

            // Failed to get the access token or user details.
            exit("erreur de connexion Oauth2 : " . $e->getMessage());
        }
    }
}

/**
 * connecte l'utilisateur, et met en place le cookie d'auth si il n'existe pas et par défaut renvoie les infos utilisateur.
 * le cookie peut être passé ensuite pour faire d'autres requètes sur l'api
 *
 * @param unknown $config
 * @param unknown $cookies
 * @param string $route
 *            : chemin relatif d'une requète en GET sur l'api.
 * @param unknown $login
 * @param unknown $passwd
 *
 * @return boolean|mixed : résultat de la requète
 */
function check_ent_passwd($config, &$cookies, $route = NULL, $login = "", $passwd = "", &$client = NULL)
{
    $ent = false;
    if (empty($route)) {
        $route = "userbook/api/person";
    }
    if ($config['ent_type'] == "openent") {
        if (! $cookies) {
            $init = true;
            $cookies = new GuzzleHttp\Cookie\CookieJar();
        } else {
            $init = true; // false;
        }
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        if (empty($config['openent_kc_auth'])) {
            $opt = [
                'base_uri' => $config['openent_uri'],
                'cookies' => $cookies
            ];
            curl_proxy_options($config, $opt);
            $client = new GuzzleHttp\Client($opt);
            try {
                $response = true;
                if ($init) {
                    $response = $client->request('POST', 'auth/login', [
                        'form_params' => [
                            'email' => $login,
                            'password' => $passwd
                        ],
                        'connect_timeout' => 10
                    ]);
                }
                if ($response) {
                    $response = $client->request('GET', $route);
                    (string) $data = $response->getBody();
                    $ent = json_decode($data, true);
                    if (@$ent['status'] != "ok") {
                        $ent = false;
                    }
                } else {
                    $ent = false;
                }
            } catch (\GuzzleHttp\Exception\RequestException $e) {
                $ent = false;
            }
        } else {
            $opt = [
                'base_uri' => $config['openent_uri'],
                'cookies' => $cookies
            ];
            curl_proxy_options($config, $opt);
            $client = new GuzzleHttp\Client($opt);
            try {
                if (! $init) {
                    $response = true;
                } else {
                    $response = $client->request('GET', 'auth/openid/login?callback=' . urlencode($config['openent_uri'] . $route), [
                        'connect_timeout' => 10
                    ]);
                    if ($response) {
                        $html = new DOMDocument();
                        libxml_use_internal_errors(true);
                        $html->loadHTML((string) $response->getbody());
                        libxml_clear_errors();
                        $url = $html->getElementById("kc-form-login")->getAttribute("action");
                        // echo $url;
                        $response = $client->request('POST', $url, [
                            'form_params' => [
                                'username' => $login,
                                'password' => $passwd,
                                'credentialId' => ""
                            ],
                            'connect_timeout' => 10
                        ]);
                        (string) $data = $response->getBody();
                        $ent = json_decode($data, true);
                        if (@$ent['status'] != "ok") {
                            $ent = false;
                            // echo "<pre>" . print_r((string) $response->getbody(), true) . "</pre>";
                        }
                    }
                }
                if (! $response) {
                    $ent = false;
                }
            } catch (\GuzzleHttp\Exception\RequestException $e) {
                $ent = false;
            }
        }
        return $ent;
    }
    return true;
}

/**
 * change le password ent de l'utilisateur
 *
 * cette fonction suppose que le mdp d'origine est connu.
 * en cas de perte l'utilisateur doit untiliser la fonction mot de passe perdu de l'ent.
 *
 * @param unknown $config
 * @param unknown $login
 * @param unknown $passwd
 * @return boolean|mixed
 */
function change_ent_passwd($config, $login, $passwd, $newpasswd)
{
    if ($config['ent_type'] == "openent") {
        $cookies = false;
        if (check_ent_passwd($config, $cookies, "", $login, $passwd)) {
            require_once (dirname(__FILE__) . '/../vendor/autoload.php');
            $opt = [
                'base_uri' => $config['openent_uri'],
                'cookies' => $cookies
            ];
            curl_proxy_options($config, $opt);
            $client = new GuzzleHttp\Client($opt);
            try {
                $token = "";
                foreach ($cookies as $cookie) {
                    if ($cookie['Name'] == "XSRF-TOKEN") {
                        $token = $cookie['Value'];
                    }
                }
                $response = $client->request('POST', 'auth/reset', [
                    'form_params' => [
                        'oldPassword' => $passwd,
                        'password' => $newpasswd,
                        'confirmPassword' => $newpasswd
                    ],
                    'headers' => [
                        'X-XSRF-TOKEN' => $token
                    ]
                ]);
                if ($response)
                    return true;
            } catch (\GuzzleHttp\Exception\RequestException $e) {
                return false;
            }
        }
    }
    return false;
}

/**
 * reinit le password ent de l'utilisateur
 *
 * cette fonction bascule lent en mode mdp perdu et renvoie le code d'activation
 * et le stocke dans apcu 'ent'
 *
 * @param unknown $config
 * @param unknown $originallogin
 * @param unknown $passwd
 * @return boolean|code
 */
function reinit_ent_passwd($config, $originallogin)
{
    $code = false;
    if (isset($config['ent_type']) && $config['ent_type'] == "openent") {
        $html = "";
        sync_ent($config);
        if ($ent = cache_fetch('ent')) {
            $key = array_search($originallogin, array_column($ent, 'originalLogin'));
            if (! empty($ent[$key]['code'])) {
                $code = $ent[$key]['code'];
            }
            if ($client = openent_admin_logon($config, $html)) {
                try {
                    $token = "";
                    foreach ($client->getConfig('cookies')->toArray() as $cookie) {
                        if ($cookie['Name'] == "XSRF-TOKEN") {
                            $token = $cookie['Value'];
                        }
                    }
                    $response = $client->request('POST', 'auth/generatePasswordRenewalCode', [
                        'form_params' => [
                            'login' => $originallogin
                        ],
                        'headers' => [
                            'X-XSRF-TOKEN' => $token
                        ]
                    ]);
                    if ($response) {
                        (string) $data = $response->getBody();
                        $msg = json_decode($data, true);
                        $ent[$key]['code'] = $msg['renewalCode'];
                        cache_store('ent', $ent);
                        $code = $msg['renewalCode'];
                    }
                } catch (\GuzzleHttp\Exception\RequestException $e) {
                    trigger_error(print_r($e, true));
                    $code = false;
                }
            }
        }
    }
    return $code;
}

/**
 * reinitialise le mot de passe dans l'ent et dans l'ad.
 * Si pas de methode de reinit dans l'ad, les stratégies de mot de passe s'appliquent
 *
 * @param unknown $config
 * @param array $user
 * @return string $html
 */
function reinit_passwd($config, array $user, array &$listing = [], bool $force = false, bool $change = true, $batch = false)
{
    $html = "";
    if (! empty($user['cn']) && ($force || $user['pwdlastset'] == 0)) {
        $rules = get_password_rule($config);
        $length = $rules['min-pwd-length'] + 3;
        if ($rules['complexity'] == "on")
            $complexity = true;
        else
            $complexity = false;
        $originallogin = explode("@", @$user['userprincipalname'])[0] ?? $user['cn'];
        $pwdPolicy = $config['pwdPolicy'];
        // $change = true;
        if ($userpwd = reinit_ent_passwd($config, $originallogin)) {
            if ($complexity) {
                $html = "Mot de passe provisoire ENT de <b>" . $user['cn'] . "</b> réinitialisé par l'ENT : <b>" . $userpwd . "</b> . Code d'accès aux ordinateurs : <b>" . $userpwd . "-#ENT#</b> .<br>\n";
            } else {
                $html = "Mot de passe provisoire de <b>" . $user['cn'] . "</b> réinitialisé par l'ENT : <b>" . $userpwd . "</b> .<br>\n";
            }
            if ($originallogin != $user['cn']) {
                if (empty($config['ent_educonnect'])) {
                    $html .= "Identifiant à utiliser pour la première connexion ENT : <b>" . $originallogin . "</b><br>";
                }
                $html .= "Identifiant à utiliser pour la première connexion aux ordinateurs du lycée : <b>" . $originallogin . "@" . $config['domain'] . "</b><br>";
                if (! empty($config['openent_uri']) && $config['openent_uri'] == "https://ent.iledefrance.fr/") {
                    $html .= "Identifiant à utiliser pour la première connexion aux ordinateurs portables : <b>" . $originallogin . "@monlycee.net</b><br>";
                }
            }

            $mode = "ent";
        } else {
            $mode = "sambaedu";
            switch ($pwdPolicy) {
                case 0: // date de naissance sans expiration (DANGER)
                case 1:
                    if (! empty($user['date']) && strlen($user['date']) == 8) {
                        $userpwd = $user['date'];
                        $html = "Mot de passe de <b>" . $user['cn'] . "</b> réinitialisé à la date de naissance : <b>" . $userpwd . "</b><br>\n";
                        if (! empty($config['no_passwd_change'])) {
                            $change = false;
                        }
                    } else {
                        $userpwd = create_random_password($length, $complexity);
                        $html = "Pas de date de naissance : mot de passe de <b>" . $user['cn'] . "</b> réinitialisé à : <b>" . $userpwd . "</b><br>\n";
                    }
                    break;
                case 2: // aleatoire
                         // case 3: // code initial, on met aléatoire
                    $userpwd = create_random_password($length, $complexity);
                    $html = "Mot de passe de <b>" . $user['cn'] . "</b> réinitialisé à : <b>" . $userpwd . "</b><br>\n";
                    break;
                default: // code initial
                    if (! $change) {
                        $userpwd = create_random_password($length, $complexity);
                        $html = "Mot de passe de <b>" . $user['cn'] . "</b> réinitialisé à : <b>" . $userpwd . "</b><br>\n";
                    } else {
                        $userpwd = $config['activation_code'] ?? create_random_password($length, $complexity);
                        $html = "Mot de passe de <b>" . $user['cn'] . "</b> réinitialisé à : <b>" . $userpwd . "</b><br>\n";
                    }
            }
        }
        $upe = $userpwd;
        if ($complexity && $mode == "ent") {
            // HACK : les pso ne fonctionnent pas :(
            $userpwd .= "-#ENT#";

            // on desactive la complexité et on met le code ENT.
            // $res = set_password_activation_rule($config);
            // $res &= apply_user_pso($config, $user['cn']);
            // if (! $res) {
            // $html .= "ERREUR, desactivation des contraintes de mot de passe impossible !";
            // }
            // $res = usersetpassword($config, $user['cn'], $userpwd, $change);
            // if (! $res) {
            // $html .= "ERREUR, setpassword impossible !";
            // } else {
            // $res &= apply_user_pso($config, $user['cn']);
            // }
        }
        $res = false;
        if (! $batch) {
            $res = usersetpassword($config, $user['dn'], $userpwd, $change, "", false, $html);
        }
        if ($res || $batch) {

            $listing[$user['dn']] = [
                'nom' => $user['nom'],
                'pre' => $user['prenom'],
                'cla' => implode(", ", list_classes($config, $user)) ?? "",
                'cn' => $user['cn'],
                'or' => $originallogin,
                'change' => $change,
                'mode' => $mode,
                'pwde' => $upe, // mot de passe ent
                'pwd' => $userpwd, // mot de passe sambaedu
                'structure' => $user['etab']
            ];
        } else {
            $html .= "ERREUR, mot de passe AD non réinitialisé !";
        }
    }
    return ($html);
}

/**
 * reintialise le mdp d'une liste d'utilisateurs
 *
 * @param array $config
 * @param array $users
 * @return string
 */
function reinit_all_passwords(array $config, array $users, array &$listing = [], $force = false)
{
    $match = cache_fetch('match');
    $html = "";
    foreach ($users as $user => $val) {
        foreach ($match as $ad_user) {
            if ($user == $ad_user['ad']['cn']) {
                $html .= reinit_passwd($config, $ad_user['ad'], $listing, $force);
                break;
            }
        }
    }
    return $html;
}

/**
 * Authentification sur openent, si besoin en MFA
 *
 * @param array $config
 * @param string $html
 *            message
 * @param bool $sms
 *            : si false, keepalive pour rafraichir la session
 * @param bool $ajax
 *            : true si appel par ajax pour affichage dialogue
 * @param string $code
 *            code SMS
 * @return boolean|\GuzzleHttp\Client
 */
function openent_admin_logon(array $config, ?string &$html = "", bool $sms = true, bool $ajax = false, string $code = "")
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $file = "/etc/sambaedu/cookies_ent.json";
    $cookies = new GuzzleHttp\Cookie\FileCookieJar($file, true);
    $opt = [
        'base_uri' => $config['openent_uri'],
        'cookies' => $cookies
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        // validation de la phase 2 : auth MFA sms
        if (apcu_fetch('openent_mfa_1') && ! apcu_fetch('openent_mfa_2') && $sms) {
            // auth sms pas encore faite
            if (empty($code)) {
                $code = $_POST['mfa_code'] ?? "";
            }
            if (! empty($code)) {
                // trigger_error("Code présent " . $code);
                // code entré dans le formulaire
                $token = "";
                foreach ($cookies->toArray() as $cookie) {
                    if ($cookie['Name'] == "XSRF-TOKEN") {
                        $token = $cookie['Value'];
                    }
                }
                // test du code
                $response = $client->request('POST', 'auth/user/mfa/code', [
                    'json' => [
                        'key' => $code
                    ],
                    'headers' => [
                        'X-XSRF-TOKEN' => $token,
                        'content-type' => "application/json;charset=utf-8"
                    ]
                ]);
                $data = $response->getBody()->getContents();
                $mfa = json_decode($data, true);

                switch ($mfa['state']) {
                    case "pending":
                        // apcu_delete("openent_mfa_2");
                        if ($ajax) {
                            $html .= "Attente du code : " . $mfa['retries'] . " essais restants, il reste " . round($mfa['$ttl'] / 60, 0) . " minutes";
                        } else {
                            $html .= "Attente du code : " . $mfa['retries'] . " essais restants, il reste " . round($mfa['$ttl'] / 60, 0) . " minutes<br>";
                            $html .= "<input name=mfa_code type=text value=''><br>";
                        }
                        return false;
                    // break;
                    case "outdated":
                        $response = $client->request('GET', 'auth/user/mfa/code');
                        // apcu_delete("openent_mfa_2");

                        $html .= "Entrer le code reçu par SMS : ";
                        if (! $ajax) {
                            $html .= "<input name=mfa_code type=text value=''><br>\n";
                        }
                        return false;
                    // break;
                    case "valid":
                        apcu_store("openent_mfa_2", $code);
                        // set_config($config, "mfa_code", $code, "ent");
                        $cookies->save($file);
                        $html .= "Code SMS valide.";
                        if (! $ajax) {
                            $html .= "<br>";
                        }
                        return $client;
                    default:
                        return $mfa;
                }
                // $html .= "Code : " . $code . " " . $mfa['state'] . "<br>";
            } else {
                // trigger_error("Code absent " . $code);
                // pas de code reçu mais authentifiaction ADML faite
                // demande du code sms
                $response = $client->request('GET', 'auth/user/mfa/code');
                $html .= "Entrer le code reçu par SMS : ";
                if (! $ajax) {
                    $html .= "<input name=mfa_code type=text value=''><br>\n";
                }
                return false;
            }
        } else {
            // test authentification ADML
            $response = $client->request('GET', 'admin');
            if (! str_contains((string) $response->getBody(), "<title>Console d'administration</title>")) {
                // Auth pas faite
                apcu_delete("openent_mfa_1");
                apcu_delete("openent_mfa_2");
                $cookies->clear(); // clearSessionCookies();

                if ((! empty($config['openent_mfa'])) && (! $sms)) {
                    // keepalive KO
                    $url = $config['se4_url'] ?? "http://" . $config['se4fs_name'];
                    $html .= "Erreur cookie ENT : validité expirée.<br/>";
                    $html .= "Se connecter sur <a href=\"" . $url . "/admin\">l'interface admin</a> pour déclencher l'envoi du SMS et saisir le nouveau code.";
                    return false;
                }
                // auth ADML
                $client = NULL;
                $ent = check_ent_passwd($config, $cookies, "", $config['openent_user'], $config['openent_user_password'], $client);
                if ($ent) {
                    $cookies->save($file);
                    apcu_store("openent_mfa_1", 1);
                    // $html .= "<pre>" . print_r($ent, true) . "</pre>";
                    if (empty($config['openent_mfa'])) {
                        // Auth directe OK
                        apcu_store("openent_mfa_2", 1);
                        return $client;
                    } else {
                        // MFA demandée
                        // demande du code sms
                        $response = $client->request('GET', 'auth/user/mfa/code');
                        $html .= "Entrer le code reçu par SMS : ";
                        if (! $ajax) {
                            $html .= "<input name=mfa_code type=text value=''><br>\n";
                        }
                        // pas encore auth
                        return false;
                    }
                } else {
                    $html .= "Echec de la vérification du mot de passe<br>";
                    return false;
                }
            } else {
                // Auth directe OK
                apcu_store("openent_mfa_1", 1);
                apcu_store("openent_mfa_2", 1);
                return $client;
            }
        }
    } catch (\GuzzleHttp\Exception\RequestException | \GuzzleHttp\Exception\ConnectException $e) {
        trigger_error("Erreur Guzzle openent_admin_logon : " . $e);
        // on ne demande pas le code si le serveur ENT est en panne !
        return true;
    }
    return false;
}

function ask_ent_mfa_code($config)
{
    $html = "";
    $msg = "";
    if (! openent_admin_logon($config, $msg, true, true)) {
        // openent_admin_logon($config, $msg2, true);
        $html .= '
            <script>
            var code = prompt("Saisie du code d\'authentification ENT\n' . $msg . '\n");
            $.ajax(
            {
                type: "POST",
                url: "/annu/post_mfa_ent.php",
                data: { "mfa_code":code },
                success: function() {
                    alert("Envoi du code au SE4FS OK");
                }
            });
            </script>';
    }
    return $html;
}

/**
 * fixe l'alias de login dans l'ent; renvoie les données ou false.
 *
 * @param unknown $config
 * @param unknown $login
 * @param unknown $passwd
 * @return boolean|mixed
 */
function change_ent_alias($config, &$client, $ad_users, &$user, &$html, $manu = false)
{
    $originallogin = $user['ent']['originalLogin'] ?? "";
    if (isset($config['ent_type']) && $config['ent_type'] == "openent" && ! $manu) {
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');

        if (empty($client)) {
            $client = openent_admin_logon($config, $html);
        }
        $force_alias = $config['ent_force_alias'] ?? "0";
        if ($client) {
            $new = false;
            $increment = 0;
            while (! $new && $increment < 20) {
                if ($force_alias == 1) {
                    $new_login = $user['ad']['cn'];
                    $new = true;
                } elseif ($user['ent']['login'] != $originallogin && empty($config['ent_remove_alias']) && strlen($user['ent']['login']) < 20) {
                    $html .= "Conservation de l'alias ENT " . $user['ent']['login'] . " déjà en place.<br>\n";
                    return true;
                } else {
                    $new_login = mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName'], "", $increment);
                }
                if ($new_login == $user['ent']['login']) {
                    return true;
                } elseif ($new_login == $user['ent']['originalLogin']) {
                    $html .= "Le login d'origine ENT est déjà  : " . $new_login . ", effacement l'alias.<br>\n";
                    $new_login = "";
                }
                while (! empty($new_login) && in_array($new_login, array_column($ad_users, "cn")) && $force_alias == 0) {
                    $new_login = mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName'], "", ++ $increment);
                }
                if ($new_login == $user['ent']['login'] || $new_login == $originallogin) {
                    $html .= "L'alias ENT est déjà $new_login. Pas de changement d'alias <br>\n";
                    return true;
                }
                try {
                    $token = "";
                    foreach ($client->getConfig('cookies')->toArray() as $cookie) {
                        if ($cookie['Name'] == "XSRF-TOKEN") {
                            $token = $cookie['Value'];
                        }
                    }

                    $response = $client->request('PUT', 'directory/user/' . $user['ent']['id'], [
                        'json' => [
                            'loginAlias' => $new_login
                        ],
                        'headers' => [
                            'X-XSRF-TOKEN' => $token
                        ]
                    ]);
                    if ($response) {
                        (string) $data = $response->getBody();
                        $msg = json_decode($data, true);
                        if ($msg['id'] == $user['ent']['id']) {
                            $user['ent']['login'] = $new_login;
                            $html .= "Alias de login ENT changé : " . $new_login . "<br>\n";
                            return true;
                        }
                    }
                } catch (GuzzleHttp\Exception\ClientException $e) {
                    $html .= "Changement d'alias $new_login impossible.<br>\n";
                    if (isset($config['debug']) && $config['debug'] > 0) {
                        $html .= \GuzzleHttp\Psr7\Message::toString($e->getRequest());
                        $html .= \GuzzleHttp\Psr7\Message::toString($e->getResponse()) . "<br>\n";
                    }
                    $increment ++;
                }
            }
        }
        return false;
    } else {
        $new = false;
        $increment = 0;
        while (! $new && $increment < 20) {
            if ($user['ent']['login'] != $originallogin) {
                $html .= "Conservation de l'alias ENT déjà en place.<br>\n";
                return true;
            } else {
                $new_login = mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName'], "", $increment);
            }
            if ($new_login == $user['ent']['login']) {
                return true;
            } elseif ($new_login == $originallogin) {
                $html .= "Le login d'origine ENT est déjà  : " . $new_login . ", effacement l'alias.<br>\n";
                $new_login = "";
            }
            while (! empty($new_login) && in_array($new_login, array_column($ad_users, "cn")) && $force_alias == 0) {
                $new_login = mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName'], "", ++ $increment);
            }
            if ($new_login == $user['ent']['login'] || $new_login == $originallogin) {
                $html .= "L'alias ENT est déjà $new_login. Pas de changement d'alias <br>\n";
                return true;
            } else {
                $user['ent']['login'] = $new_login;
                $html .= "Alias de login ENT changé : " . $new_login . "<br>\n";
                return true;
            }
            $increment ++;
        }
        return false;
    }
}

/**
 * verrouille deverouille le compte si besoin
 *
 * @param
 *            $config
 * @param
 *            ent_user
 * @param
 *            block
 * @return etat
 */
function lock_ent_user($config, &$client, &$user, $block, &$html)
{
    if ($user['ent']['blocked'] != $block) {

        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        if (empty($client)) {
            $client = openent_admin_logon($config, $html);
        }
        try {
            foreach ($client->getConfig('cookies')->toArray() as $cookie) {
                if ($cookie['Name'] == "XSRF-TOKEN") {
                    $token = $cookie['Value'];
                }
            }
            $response = $client->request('PUT', "auth/block/" . $user['ent']['id'], [
                'json' => [
                    'block' => $block
                ],
                'headers' => [
                    'X-XSRF-TOKEN' => $token
                ]
            ]);
            if ($response) {
                $user['ent']['blocked'] = $block;
                return true;
            } else {
                $html .= "erreur lors de l'operation $block sur " . $user['ent']['login'] . "<br>\n";
            }
        } catch (\GuzzleHttp\Exception\TransferException $e) {
            $html .= "erreur lors de l'operation $block sur " . $user['ent']['login'] . "<br>\n";
            $html .= \GuzzleHttp\Psr7\Message::toString($e->getRequest());
            $html .= \GuzzleHttp\Psr7\Message::toString($e->getResponse());
        }

        return false;
    } else {
        return true;
    }
}

/**
 * on évite tous les caractères problèmatiques dans les noms de matières
 *
 * @param string $name
 * @return string
 */
function matiere_name($name)
{
    // supprime les doubles (ou plus) espaces, s'assure que les majuscules sont sans accents et pas de caratères bizarres
    return trim(preg_replace('/\s+/', ' ', strtr(ensure_ascii(remplace_accents($name)), array(
        "'" => " ",
        "(" => " ",
        ")" => " ",
        ".-" => "-",
        "." => " ",
        "," => " ",
        "/" => " ",
        "&" => " ET ",
        "#" => ""
    ))));
}

/**
 * décode et effectue le pré-traitement du fichier JSON ENT
 *
 * @param string $data
 * @return array $ent tableau des utilisateurs
 */
function ent_json_decode($data)
{
    $ent = json_decode($data, true);
    $keys = array_keys(array_column($ent, "type"), "Relative");
    foreach ($keys as $key) {
        unset($ent[$key]);
    }
    $keys = array_keys(array_column($ent, "type"), "Teacher");
    foreach ($keys as $key) {
        $ent[$key]["allMatieres"] = [];
        foreach ($ent[$key]["allClassesSubject"] as $ClassesSubject) {
            $names = array_column($ent[$key]["allMatieres"], "name");
            $name = matiere_name($ClassesSubject["name"]);
            if (! is_null($ClassesSubject["name"]) && ! in_array($name, $names)) {
                $ent[$key]["allMatieres"][] = array(
                    "name" => $name,
                    "id" => ""
                );
            }
        }
        if (! is_null($ent[$key]["aafFunctions"])) {
            foreach ($ent[$key]["aafFunctions"] as $fonction) {
                $names = array_column($ent[$key]["allMatieres"], "name");
                $name = matiere_name($fonction);
                if ($name != "SANS OBJET" && ! in_array($name, $names)) {
                    $ent[$key]["allMatieres"][] = array(
                        "name" => $name,
                        "id" => ""
                    );
                }
            }
        }
    }
    foreach ($ent as $k => $user) {
        $ent[$k]['firstName_norm'] = simplify_firstname($user['firstName'], false, true);
        $ent[$k]['lastName_norm'] = simplify_name($user['lastName'], true);
        $ent[$k]['structure'] = $ent[$k]['structure'] ?? 0;
    }
    usort($ent, "cmp_classe");
    return $ent;
}

/**
 * récupère la base utilisateurs dans l'ENT
 * les identifiants d'un administrateur doivent être saisis dans un ficiher de conf
 *
 * @param unknown $config
 * @return tableau des utilisateurs
 */
function sync_ent($config, $force = false, $cron = false)
{
    if (apcu_fetch("match_lock")) {
        return "Une importation est déjà en cours !<br>\n";
    }
    $html = "";
    $ent = cache_fetch('ent');
    if (@$config['ent_type'] == "openent") {
        if (! $ent || count($ent) == 0 || $force) {
            if ($client = openent_admin_logon($config, $html, ! $cron)) {
                try {
                    $response = $client->request('GET', 'directory/export/users');
                    $data = $response->getBody()->getContents();
                    // $html .= $response->getStatusCode();
                    // $html .= "<pre>" . print_r($response, true) . "</pre>";
                    if (! empty($data)) {
                        $ent = ent_json_decode($data);
                    } else {
                        $html .= "Erreur lors de la récupération des données ENT, 2FA nécessaire<br>\n";
                        cache_store('ent', array());
                        apcu_delete("openent_mfa_1");
                        apcu_delete("openent_mfa_2");
                        $file = "/etc/sambaedu/cookies_ent.json";
                        $cookies = new GuzzleHttp\Cookie\FileCookieJar($file, true);
                        $cookies->clearSessionCookies();
                        $cookies->__destruct();
                        if (! $cron) {
                            $html .= form_ent_next("nouveau_force");
                        }
                    }
                    if (count($ent) > 0) {
                        // $html .= "<pre>" . print_r($ent, true) . "</pre>";
                        cache_store('ent', $ent);
                        $html .= count($ent) . " utilisateurs importés.<br>\n";
                        if (! $cron) {
                            $html .= form_ent_next("match");
                        }
                    } else {
                        $html .= "Erreur lors de la récupération des données ENT, 2FA nécessaire.<br>\n";
                        cache_store('ent', array());
                        apcu_delete("openent_mfa_1");
                        apcu_delete("openent_mfa_2");
                        $file = "/etc/sambaedu/cookies_ent.json";
                        $cookies = new GuzzleHttp\Cookie\FileCookieJar($file, true);
                        $cookies->clearSessionCookies();
                        $cookies->__destruct();
                        if (! $cron) {
                            $html .= form_ent_next("nouveau_force");
                        }
                    }
                } catch (GuzzleHttp\Exception\ClientException $e) {
                    $html .= "Erreur lors de la connexion à l'ENT.<br>\n";
                    $html .= "Exception : " . $e->getMessage();
                    // $html .= GuzzleHTTP\Psr7\Message::toString($e->getRequest());
                    // $html .= GuzzleHTTP\Psr7\Message::toString($e->getResponse());
                    cache_store('ent', array());
                    apcu_delete("openent_mfa_1");
                    apcu_delete("openent_mfa_2");
                }
            } else {
                $html .= "erreur lors de la connexion 2FA à l'ENT<br>\n";
                if (! $cron) {
                    $html .= form_ent_next("nouveau_force");
                }
            }
        }
    }
    // $html .= "<pre>" . $err . "</pre>";
    return $html;
}

/**
 * récupère la base utilisateurs au format json
 *
 * @param unknown $configmessage
 *            de sortie
 */
function read_ent_json($config, $filename, &$data, $manu = false)
{
    $html = "";
    if (@$config['ent_type'] == "openent" || $manu) {
        $json = file_get_contents($filename);
        $ent = ent_json_decode($json);
        if (count($ent) > 0) {
            $html .= count($ent) . " utilisateurs synchronisés.<br>\n";
            $data = array_merge($data, $ent);
        } else {
            $html .= "Erreur lors de la récupération des données ENT. <br>\n" . print_r($data, true);
        }
    }
    return $html;
}

function sync_ent_groups($config)
{
    if (apcu_fetch("match_lock")) {
        return "Une importation est déjà en cours !<br>\n";
    }
    $n = $g = 0;
    $html = "";
    $ent = cache_fetch('ent');
    if (@$config['ent_type'] == "openent") {

        // url liste des groupes : https://ent.iledefrance.fr/directory/group/admin/list?type=FunctionalGroup&structureId=0f851f90-9dff-47d0-979e-0feec63db6b6
        // url liste d'un groupe : https://ent.iledefrance.fr/directory/user/group/039ba279-4cb8-4f70-a5a2-53c51e708366
        if ($ent && count($ent) > 0) {
            $id_etab = $ent[0]['structures'][0]['id'];
            if ($client = openent_admin_logon($config, $html)) {
                try {
                    $response = $client->request('GET', 'directory/group/admin/list', [
                        'query' => [
                            'type' => "FunctionalGroup",
                            'StructureId' => $id_etab
                        ]
                    ]);
                    if ($response) {
                        (string) $data = $response->getBody();
                        $ent_groups = json_decode($data, true);
                        $n = 0;
                        $g = count($ent_groups);
                        if ($g > 0) {
                            foreach ($ent_groups as $group) {
                                $response = $client->request('GET', 'directory/user/group/' . $group['id']);
                                if ($response) {
                                    (string) $data = $response->getBody();
                                    $group_data = json_decode($data, true);
                                    foreach ($group_data as $member) {
                                        $key = array_keys(array_column($ent, 'id'), $member['id'])[0];
                                        // if (! preg_match("/.*[0-9]p\.?[1-3]|.* [0-9]{2}/i", $group['name'])) {
                                        if (! preg_match("/.* [0-9]{2}/i", $group['name'])) {
                                            $ent[$key]['allGroups'][] = [
                                                'name' => $group['name'],
                                                'id' => $group['id']
                                            ];
                                            $n ++;
                                        }
                                    }
                                }
                            }
                            cache_store('ent', $ent, $config['ent_timeout']);
                        } else {
                            $html .= "Erreur lors de la récupération des données ENT. <br>\n" . print_r($data, true);
                            cache_store('ent', $ent, $config['ent_timeout']);
                        }
                    } else {
                        $html .= "Erreur lors de la récupération des données ENT, pas de réponse du serveur.<br>\n";
                        cache_store('ent', $ent, $config['ent_timeout']);
                    }
                    // } catch (\GuzzleHttp\Exception\TransferException $e) {
                    // $html .= "Erreur lors de la connexion à l'ENT.<br>\n";
                    // $html .= \GuzzleHttp\Psr7\Message::toString($e->getRequest());
                    // $html .= \GuzzleHttp\Psr7\Message::toString($e->getResponse());
                    // cache_store('ent', $ent, $config['ent_timeout']);
                    // }
                } catch (GuzzleHttp\Exception\ClientException $e) {
                    $html .= "Erreur lors de la connexion à l'ENT.<br>\n";
                    $html .= "Exception : " . $e->getMessage();
                    // $html .= GuzzleHTTP\Psr7\Message::toString($e->getRequest());
                    // $html .= GuzzleHTTP\Psr7\Message::toString($e->getResponse());
                    cache_store('ent', array(), $config['ent_timeout']);
                }
            }
            $html .= $g . " groupes synchronisés.<br>\n";
            $html .= $n . " associations aux groupes synchronisées.<br>\n";
        }
    }
    // $html .= "<pre>" . print_r($ent, true) . "</pre>";
    // $html .= "<pre>" . $err . "</pre>";
    return $html;
}

/**
 * récupère les infos dans un ficihier csv.
 * Attention format UTF-8 obligatoire
 *
 * @param unknown $config
 * @param unknown $csv_file
 * @return array|unknown|unknown[]|string|array|unknown
 */
function import_ent($config, $csv_file, $file_type, &$data, $manu = false)
{
    $html = "";
    require_once "gpo.inc.php";
    $contents = file_get_contents($csv_file);
    $enc = detect_utf_encoding($contents);
    $e = $config['etab_ou'] ?? "";
    switch ($file_type) {
        case 'gpei':
            read_gpei_xml($config, $csv_file, $data, $e);
            break;
        case 'json':
            read_ent_json($config, $csv_file, $data, $manu);
            break;
        case "asm_teacher":
        case "asm_student":
            $contents = iconv($enc, "UTF-8", $contents);
            $contents = strtr($contents, ",", ";");
        case "kosmos":
        case "csv_pronote":
            $contents = iconv($enc, "UTF-8", $contents);
        default:
            // $contents = substr($contents, 3);
            $contents = explode("\n", $contents);
            $keys = [];
            foreach ($contents as $num => $line) {
                $line = rtrim($line);
                if ($num == 0) {
                    $keys = preg_replace("/\"/", "", explode(";", $line, 80));
                    $keys = map_ent_fields($keys, $file_type);
                } elseif (! empty($line)) {
                    $index = $num - 1;
                    $val = preg_replace("/\"/", "", explode(";", $line, 80));
                    if (count($val) > 0) {
                        foreach ($keys as $ind => $key) {
                            if (! empty($key)) {
                                $val[$ind] = trim($val[$ind]);
                                switch ($key) {
                                    case "allClasses":
                                    case "functions":
                                    case "allGroups":
                                    case "allRights":
                                        $data[$index][$key] = map_classe(explode(",", $val[$ind]), $file_type);
                                        break;
                                    case "allMatieres":
                                        $data[$index][$key] = map_matiere($config, explode(",", $val[$ind]), $file_type);
                                        break;
                                    case "structures":
                                        $data[$index][$key] = array_map('trim', (explode(",", $val[$ind])));
                                        $data[$index]['structure'] = trim($data[$index][$key][0] ?? 0);
                                        break;
                                    case "type":
                                        $data[$index][$key] = map_profil($val[$ind], $file_type);
                                        break;
                                    case "firstName":
                                        $data[$index][$key] = $val[$ind];
                                        $data[$index][$key . "_norm"] = simplify_firstname($val[$ind]);
                                        break;
                                    case "lastName":
                                        $data[$index][$key] = $val[$ind];
                                        $data[$index][$key . "_norm"] = simplify_name($val[$ind]);
                                        break;
                                    case "birthDate":
                                        $data[$index][$key] = map_birthdate($val[$ind], $file_type);
                                        break;
                                    case "sex":
                                        $data[$index][$key] = map_sex($val[$ind]);
                                        break;
                                    case "email":
                                        if ($file_type == "csv_oZe") {
                                            $data[$index]['login'] = explode("@", $val[$ind])[0];
                                            $data[$index][$key] = $val[$ind];
                                        }
                                        break;
                                    default:
                                        if (! empty($val[$ind])) {
                                            $data[$index][$key] = $val[$ind];
                                        }
                                }
                            }
                        }
                        if ($file_type == "csv_pronote_eleve") {
                            $data[$index]['type'] = "Student";
                        } elseif ($file_type == "asm_teacher") {
                            $data[$index]['type'] = "Teacher";
                        } elseif ($file_type == "asm_student") {
                            $data[$index]['type'] = "Student";
                        }
                    }
                }
            }
            if (isset($data[0]['allClasses'])) {
                usort($data, "cmp_classe");
            }
            break;
    }
    $n = count($data);
    if ($n == 0) {
        $n = "aucun";
    }
    $html .= "Fichier " . $csv_file . " format " . $file_type . " encodé en $enc : " . $n . " utilisateurs analysés.<br>\n";

    unlink($csv_file);
    // $html .= "ent contient " . $n . " enregistrements<br>";
    // $html .= "fichier : <pre>" . print_r($data, true) . "</pre>";
    return $html;
}

function stats_users($config, $csv_out = false)
{
    $stats = apcu_fetch('stats_activation');

    if ($stats === false) {

        $users = search_ad($config, "*", "user", "all", [
            'dn',
            'memberof',
            'pwdlastset'
        ], "subtree", true, true);
        foreach ($users as $u) {
            $m = [];
            if (! isset($stats[$u['etab']])) {
                $stats[$u['etab']] = [
                    'activated' => [
                        'Administratifs' => 0,
                        'Profs' => 0,
                        'Eleves' => 0
                    ],
                    'total' => [
                        'Administratifs' => 0,
                        'Profs' => 0,
                        'Eleves' => 0
                    ]
                ];
            }
            if (isset($u['memberof']) && is_array($u['memberof'])) {
                $activated = $u['pwdlastset'] != 0;
                foreach ($u['memberof'] as $groupe) {
                    if (preg_match("/CN=(Eleves|Profs|Administratifs),OU=Groups.*/", $groupe, $m)) {
                        $type = $m[1];
                        if ($activated)
                            $stats[$u['etab']]['activated'][$type] ++;
                        $stats[$u['etab']]['total'][$type] ++;
                        break;
                    }
                }
            }
        }
        $taa = $ta = $tpa = $tp = $tea = $te = 0;
        foreach ($stats as $e => $stat) {
            if ($e != 0) {
                if ($stat['total']['Administratifs'] > 0) {
                    $stats[$e]['ratio']['Administratifs'] = $stat['activated']['Administratifs'] / $stat['total']['Administratifs'];
                } else {
                    $stats[$e]['ratio']['Administratifs'] = 0;
                }
                if ($stat['total']['Profs'] > 0) {
                    $stats[$e]['ratio']['Profs'] = $stat['activated']['Profs'] / $stat['total']['Profs'];
                } else {
                    $stats[$e]['ratio']['Profs'] = 0;
                }
                if ($stat['total']['Eleves'] > 0) {
                    $stats[$e]['ratio']['Eleves'] = $stat['activated']['Eleves'] / $stat['total']['Eleves'];
                } else {
                    $stats[$e]['ratio']['Eleves'] = 0;
                }
                if ($stat['total']['Eleves'] + $stat['total']['Profs'] + $stat['total']['Administratifs'] > 0) {
                    $stats[$e]['ratio']['total'] = ($stat['activated']['Eleves'] + $stat['activated']['Profs'] + $stat['activated']['Administratifs']) / ($stat['total']['Eleves'] + $stat['total']['Profs'] + $stat['total']['Administratifs']);
                } else {
                    $stats[$e]['ratio']['total'] = 0;
                }

                $taa += $stat['activated']['Administratifs'] ?? 0;
                $ta += $stat['total']['Administratifs'] ?? 0;
                $tpa += $stat['activated']['Profs'] ?? 0;
                $tp += $stat['total']['Profs'] ?? 0;
                $tea += $stat['activated']['Eleves'] ?? 0;
                $te += $stat['total']['Eleves'] ?? 0;
                $stats[$e]['activated']['total'] = $stat['activated']['Administratifs'] + $stat['activated']['Profs'] + $stat['activated']['Eleves'];
                $stats[$e]['total']['total'] = $stat['total']['Administratifs'] + $stat['total']['Profs'] + $stat['total']['Eleves'];
            }
        }
        $stats['total'] = [
            'activated' => [
                'Administratifs' => $taa,
                'Profs' => $tpa,
                'Eleves' => $tea
            ],
            'total' => [
                'Administratifs' => $ta,
                'Profs' => $tp,
                'Eleves' => $te
            ],
            'ratio' => [
                'Administratifs' => $taa / $ta,
                'Profs' => $tpa / $tp,
                'Eleves' => $tea / $te,
                'total' => ($tpa + $taa + $tea) / ($ta + $tp + $te)
            ]
        ];
        apcu_store('stats_activation', $stats, 3600);
    }
    // return $stats;
    $csv[0] = "UAI;Administratifs actifs;Administratifs total;Administratifs ratio;Profs actifs;Profs total;Profs ratio;Eleves actifs;Eleves total;Eleves ratio;ratio total";
    foreach ($stats as $e => $stat) {
        if ($e != 0) {
            $csv[$e] = $e . ";" . $stat['activated']['Administratifs'] . ";" . $stat['total']['Administratifs'] . ";" . number_format($stats[$e]['ratio']['Administratifs'], 2, ",") . ";";
            $csv[$e] .= $stat['activated']['Profs'] . ";" . $stat['total']['Profs'] . ";" . number_format($stats[$e]['ratio']['Profs'], 2, ",") . ";";
            $csv[$e] .= $stat['activated']['Eleves'] . ";" . $stat['total']['Eleves'] . ";" . number_format($stats[$e]['ratio']['Eleves'], 2, ",") . ";" . number_format($stats[$e]['ratio']['total'], 2, ",");
        }
    }
    if ($csv_out) {
        return implode("\n", $csv);
    } else {
        return $stats;
    }
}

function ldap_users($config)
{
    $users = search_ad($config, "*", "user", "all", [], "subtree", false, true); // , $config['dn']['people']);
    foreach ($users as $k => $user) {
        if (isset($user['prenom'])) {
            $users[$k]["prenom_norm"] = simplify_firstname($user['prenom']);
        }
        if (isset($user['nom'])) {
            $users[$k]["nom_norm"] = simplify_name($user['nom']);
        }
    }
    return $users;
}

/**
 * construit un tableau avec les utilisateurs classés par catégorie en classe
 * pour l'affichage des listes de selection
 *
 * @param unknown $config
 * @param unknown $users
 * @return array|unknown
 */
function ldap_users_group($config, $users)
{
    // return $users;
    $classes = array(
        'eleve' => array(),
        'prof' => array(),
        'administratif' => array()
    );
    $config['etab_ou'] = $config['etab_ou'] ?? "0";
    $g = false;
    foreach ($users as $user) {
        if ($config['etab_ou'] == 0 || $user['etab'] == $config['etab_ou']) {
            if (strcasecmp(ldap_dn2oudn($user['dn']), $config['dn']['trash']) == 0) {
                $classes['trash'][] = $user;
            } elseif (isset($user['memberof'])) {
                $i = 0;
                if (preg_match("/.*(admin|exam|invite|test|api-).*/i", $user['cn'])) {
                    $user['protected'] = true;
                }
                foreach ($user['memberof'] as $groupe) {
                    if (preg_match("/CN=no_trash_users.*/i", $groupe)) {
                        $user['protected'] = true;
                        break;
                    }
                }
                foreach ($user['memberof'] as $groupe) {
                    $m = array();
                    if (preg_match("/CN=(Classe|Equipe|Cours)_(.*),OU=(Classes|Equipes|Cours).*/i", $groupe, $m)) {
                        if ($m[1] == "Classe") {
                            $i ++;
                            $classes['eleve'][$m[2]][] = $user;
                        } elseif ($m[1] == "Cours") {
                            $i ++;
                            $classes['eleve'][$m[2]][] = $user;
                        } elseif ($m[1] == "Equipe" && $i ++ == 0) {
                            $classes['prof'][] = $user;
                        }
                        $g = true;
                    }
                }
                if ($i == 0) {
                    foreach ($user['memberof'] as $groupe) {
                        $m = array();
                        if (preg_match("/CN=(Eleves|Profs|Administratifs),OU=Groups.*/", $groupe, $m)) {
                            if ($m[1] == "Eleves") {
                                $classes['eleve']['sans'][] = $user;
                            } elseif ($m[1] == "Profs") {
                                $classes['prof'][] = $user;
                            } elseif ($m[1] == "Administratifs") {
                                $classes['administratif'][] = $user;
                            }
                            $g = true;
                        }
                    }
                }
                if (! $g && ! $user['protected']) {
                    $classes['sans'][] = $user;
                }
            }
        }
    }

    return $classes;
}

function normalize_name($name, $prenom = false)
{
    if ($prenom) {
        $string = explode(" ", $name)[0];
    } else {
        $string = $name;
    }
    $table = [
        "_" => "",
        "-" => "",
        "'" => "",
        " " => ""
    ];
    return strtolower(remplace_accents(strtr($string, $table)));
}

function compare_name($ent_string = "", $ad_string = "", $prenom = false, $distance = 1)
{
    if (empty($ent_string) || empty($ad_string))
        return false;
    // $ent_string = normalize_name($ent_string, $prenom);
    // $ad_string = normalize_name($ad_string, $prenom);
    $res = levenshtein($ent_string, $ad_string);
    return ($res < $distance);
}

function match_id($config, $ent_user, $ldap_user)
{
    if (! $id = match_gpei($config, $ent_user, $ldap_user)) {
        if (! $id = match_siecle($config, $ent_user, $ldap_user)) {
            if (! $id = match_openent_id($config, $ent_user, $ldap_user)) {
                // if ($id = match_openent_ext($config, $ent_user, $ldap_user)) {
                // return "openent_ext : " . $id;
                // } else {
                return false;
                // }
                // if (! match_pronote($config, $ent_user, $ldap_user)) {
                // return match_asm($config, $ent_user, $ldap_user);
                // }
                // }
            } else {
                return "openent_id : " . $id;
            }
        } else {
            return "siecle : " . $id;
        }
    } else {
        return "gpei : " . $id;
    }
}

function test_id($ldap_user)
{
    return (empty($ldap_user['externalId']) && empty($ldap_user['id']) && empty($ldap_user['Id GPEI']) && empty($ldap_user['Id Siecle']));
}

function match_openent_ext($config, $ent_user, $ldap_user)
{
    if (empty($ent_user['externalId']) || empty($ldap_user['externalId'])) {
        return false;
    } else {
        if ($ent_user['externalId'] != $ldap_user['externalId']) {
            return $ent_user['externalId'];
            // transition
            // return ($ent_user['externalId'] == $ldap_user['id']);
        } else {
            return false;
        }
    }
}

function match_openent_id($config, $ent_user, $ldap_user)
{
    if (empty($ent_user['id']) || empty($ldap_user['id'])) {
        return false;
    } else {
        if ($ent_user['id'] == $ldap_user['id']) {
            return $ent_user['id'];
        } else {
            return false;
        }
    }
}

function match_asm($config, $ent_user, $ldap_user)
{
    if (empty($ent_user['Id ASM']) || empty($ldap_user['Id ASM'])) {
        return false;
    } else {
        return ($ent_user['Id ASM'] == $ldap_user['Id ASM']);
    }
}

function match_gpei($config, $ent_user, $ldap_user)
{
    if (empty($ent_user['Id GPEI']) || empty($ldap_user['Id GPEI'])) {
        return false;
    } else {
        if ($ent_user['Id GPEI'] == $ldap_user['Id GPEI']) {
            return $ent_user['Id GPEI'];
        } else {
            return false;
        }
    }
}

function match_siecle($config, $ent_user, $ldap_user)
{
    if (empty($ent_user['Id Siecle']) || empty($ldap_user['Id Siecle'])) {
        return false;
    } else {
        if ($ent_user['Id Siecle'] == $ldap_user['Id Siecle']) {
            return $ent_user['Id Siecle'];
        } else {
            return false;
        }
    }
}

function match_pronote($config, $ent_user, $ldap_user)
{
    if (empty($ent_user['Id Pronote']) || empty($ldap_user['Id Pronote'])) {
        return false;
    } else {
        return ($ent_user['Id Pronote'] == $ldap_user['Id Pronote']);
    }
}

function match_etab($config, $ent_user, $ldap_user)
{
    $uai = $ldap_user['etab'];
    if (! isset($ent_user['structure'])) {
        return false;
    } else {
        return (strcasecmp($ent_user['structure'], $uai) == 0);
    }
}

function match_birthdate($config, $ent_user, $ldap_user)
{
    $ret = false;
    if (isset($ent_user['birthDate']) && ! empty($ldap_user['hash'])) {
        $ret = (hash('SHA256', $ent_user['birthDate']) == $ldap_user['hash']);
        $ret |= ($ent_user['birthDate'] == decode_birthdate($ldap_user['hash']));
    }
    return $ret;
}

function match_user($config, $ent_user, $ldap_user, $distance = 1)
{
    if (isset($ent_user['login']) && $ent_user['login'] == $ldap_user['cn']) {
        return true;
    } elseif (isset($ent_user['login']) && strtr($ent_user['login'], "-", "_") == $ldap_user['cn']) {
        return true;
    } elseif (isset($ent_user["originalLogin"]) && $ent_user["originalLogin"] == @$ldap_user["userprincipalname"] . "@" . $config['domain']) {
        return true;
    } elseif (isset($ent_user["test_login"]) && $ent_user["test_login"] == $ldap_user["cn"]) {
        return true;
    } elseif (isset($ent_user["firstName_norm"]) && isset($ent_user["lastName_norm"]) && isset($ldap_user["nom_norm"]) && compare_name($ent_user["lastName_norm"], $ldap_user["nom_norm"], false, $distance) && compare_name($ent_user["firstName_norm"], $ldap_user["prenom_norm"], true, $distance)) {
        return true;
    } else {
        return false;
    }
}

function match_login($config, $ent_user, $ldap_user, $distance = 1)
{
    if (isset($ent_user['login']) && $ent_user['login'] == $ldap_user['cn']) {
        return true;
    } elseif (isset($ent_user['login']) && strtr($ent_user['login'], "-", "_") == $ldap_user['cn']) {
        return true;
    } elseif (isset($ent_user["originalLogin"]) && $ent_user["originalLogin"] == @$ldap_user["userprincipalname"] . "@" . $config['domain']) {
        return true;
    } elseif (isset($ent_user["test_login"]) && $ent_user["test_login"] == $ldap_user["cn"]) {
        return true;
    } elseif (levenshtein($ent_user["test_login"], $ldap_user["cn"]) < $distance) {
        return true;
    } else {
        return false;
    }
}

/**
 * recherche des correspondances entre ENT et AD
 *
 * @param unknown $config
 * @param unknown $ent_user
 * @param unknown $ad_users
 * @param array $match
 *            : tableau des correspondances
 * @param boolean $auto
 *            : uniqueement les correspondances strictes si oui. Obligatoire en production, si la source d'alimentation es unique. Attention aucune correspondance ne sera faite avec les comptes sam
 * @param boolean $asm
 *            : mode spécial pour recollage ASM
 * @param boolean $manu
 *            : mise en correspondance forcée via l'interface
 * @param string $html
 * @return number
 */
function match_ad($config, $ent_user, &$ad_users, &$match = [], $auto = false, $asm = false, $manu = false, &$html = "")
{
    $new = cache_fetch('new');
    if (! is_array($new)) {
        $new = [];
    }
    $l = $k = 0;
    $found = $almost_found = false;
    $i = 0;
    $rename = 0;
    $n = 0;
    // tolérance sur la différence de nom
    $distance = 1;
    $msg_almost_found = "";
    $url = $config['se4_url'] ?? "http://" . $config['se4fs_name'];
    switch ($ent_user['type']) {
        case "Student":
        case "Personnel":
        case "Guest":
        case "Teacher":
        case "trash":
        case "attributs":
            // $html .= "Cherche : Compte " . $ent_user['firstName_norm'] . " " . $ent_user['lastName_norm'] . "</b>";

            $ent_user['test_login'] = mk_cn($config, $ent_user["lastName"], $ent_user["firstName"]);
            if (empty($ent_user['login'])) {
                $ent_user['login'] = $ent_user['test_login'];
            }
            $ent_user['birthDate'] = map_birthdate($ent_user['birthDate'] ?? "");

            $ad_user = $match_ids = [];
            // correspondance exactes avec l'id unique. En production seule cette correspondance devrait être faite lors des synchros automatiques.
            foreach ($ad_users as $l => $ad_user) {
                $id = match_id($config, $ent_user, $ad_user);
                if ($id !== false) {
                    $ad_user['match_message'] = $id;
                    if (preg_match("/" . $config['dn']['trash'] . "/i", $ad_user['dn'])) {
                        $trash_ids[$l] = $ad_user;
                    } else {
                        $match_ids[$l] = $ad_user;
                    }
                    // $msg_almost_found .= "correspondance Id : <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") avec <b>" . $ad_user["cn"] . " " . $ad_user["prenom"] . " " . $ad_user["nom"] . "</b><br>";
                    $found = true;
                }
            }
            if ($found && (count($match_ids) == 0)) {
                // on va récuperer les poubelles
                $match_ids = $trash_ids;
                $html .= "correspondance " . $trash_ids[0]["match_massage"] . " avec un utilisateur supprimé : <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") avec <b> " . $trash_ids[0]["cn"] . "</b><br>";
            }
            if ($found && (count($match_ids) == 1)) {
                $k = array_key_first($match_ids);
                $m = [
                    'ent' => $ent_user,
                    'ad' => $match_ids[$k]
                ];
            } elseif ($found && (count($match_ids) > 1)) {
                $test_users = $match_ids;
                $distance = 3;
                $found = false;
                // $html .= "<pre>" . print_r($match_ids, true) . "</pre><br>";
            } else {
                $test_users = $ad_users;
            }

            # Si l'utilisateur a pas été trouvé avec un des ID, on cherche
            if (! $found || $distance > 1) {
                // génération d'un tableau des homonymes, sur lequel on teste d'autres critères
                $match_ad = [];
                foreach ($test_users as $l => $ad_user) {
                    if (match_user($config, $ent_user, $ad_user)) {
                        $msg_almost_found .= "Correspondance potentielle du compte ENT <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") avec <b>" . $ad_user["prenom"] . " " . $ad_user["nom"] . "</b> (" . $ad_user['cn'] . ") sur l'AD. <a target=\"_blank\" href=\"mod_user_entry.php?cn=" . $ad_user['cn'] . "\">Modifier ce compte</a>.<br>\n";
                        $match_ad[$l] = $ad_user;
                    }
                }
                // vérification login
                if (count($match_ad) > 0) {
                    foreach ($match_ad as $l => $ad_user) {
                        if (match_login($config, $ent_user, $ad_user, 2)) {
                            $am = [
                                'ent' => $ent_user,
                                'ad' => $ad_user
                            ];
                            $ak = $l;
                            // test si les id AAF; ENT, GPEI et Siecle du compte LDAP sont vides
                            if (test_id($ad_user)) {
                                $html .= "Correspondance suffisante (match_user + match_login + test_id) entre le compte ENT <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") et le compte AD <b>" . $ad_user["prenom"] . " " . $ad_user["nom"] . "</b> (" . $ad_user['cn'] . ").<br>\n";
                                $found = true;
                                $k = $ak;
                                $m = $am;
                            } else {
                                $msg_almost_found .= "Le login du compte ENT <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") correspond pour le compte AD <b>" . $ad_user["prenom"] . " " . $ad_user["nom"] . "</b> (" . $ad_user['cn'] . ").<br>\n";
                                $almost_found = true;
                            }
                        } else {
                            // on enlève le compte
                            unset($match_ad[$l]);
                        }
                    }
                }

                // vérification de la date de naissance
                if (isset($match_ad) && count($match_ad) > 0) {
                    foreach ($match_ad as $l => $ad_user) {
                        if (match_birthdate($config, $ent_user, $ad_user)) {
                            $am = [
                                'ent' => $ent_user,
                                'ad' => $ad_user
                            ];
                            $ak = $l;
                            // test si les id AAF; ENT, GPEI et Siecle du compte LDAP sont vides
                            if (test_id($ad_user)) {
                                $html .= "Correspondance suffisante (match_user + match_birthdate + test_id) entre le compte ENT <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") et le compte AD <b>" . $ad_user["prenom"] . " " . $ad_user["nom"] . "</b> (" . $ad_user['cn'] . ").<br>\n";
                                $found = true;
                                $k = $ak;
                                $m = $am;
                            } else {
                                $msg_almost_found .= "La date de naissance du compte ENT <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") correspond pour le compte AD <b>" . $ad_user["prenom"] . " " . $ad_user["nom"] . "</b> (" . $ad_user['cn'] . ").<br>\n";
                                $almost_found = true;
                            }
                        }
                    }
                }

                // asm : même établissement
                if (! $found && ! $almost_found && isset($match_ad) && count($match_ad) > 0) {
                    foreach ($match_ad as $l => $ad_user) {
                        if (! $asm || match_etab($config, $ent_user, $ad_user)) {
                            $am = [
                                'ent' => $ent_user,
                                'ad' => $ad_user
                            ];
                            $ak = $l;
                            // test si les id AAF; ENT, GPEI et Siecle du compte LDAP sont vides
                            if (test_id($ad_user)) {
                                $html .= "Correspondance suffisante (match_user + match_etab + test_id) entre le compte ENT <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") et le compte AD <b>" . $ad_user["prenom"] . " " . $ad_user["nom"] . "</b> (" . $ad_user['cn'] . ").<br>\n";
                                $found = true;
                                $k = $ak;
                                $m = $am;
                            } else {
                                $msg_almost_found .= "L'établissement du compte ENT <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") correspond pour le compte AD <b>" . $ad_user["prenom"] . " " . $ad_user["nom"] . "</b> (" . $ad_user['cn'] . ").<br>\n";
                                $almost_found = true;
                            }
                            break;
                        }
                    }
                }
                if ($distance > 1) {
                    // on prend le premier de la liste par défaut si rien n'a été trouvé
                    $html .= $msg_almost_found;
                    if (empty($am)) {
                        $html .= "Id identiques, mais pas de correspondance  entre le compte ENT <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $ent_user["login"] . ") et le compte AD <b>" . $test_users[$k]["prenom"] . " " . $test_users[$k]["nom"] . "</b> (" . $test_users[$k]['cn'] . ").<br>\n";

                        // $am = [
                        // 'ent' => $ent_user,
                        // 'ad' => $test_users[$k]
                        // ];
                        // $ak = $k;
                    } else {
                        $html .= "Correspondance entre le compte ENT <b>" . $am['ent']["firstName"] . " " . $am['ent']["lastName"] . "</b> (" . $am['ent']["login"] . ") et le compte AD <b>" . $am['ad']["prenom"] . " " . $am['ad']["nom"] . "</b> (" . $am['ad']['cn'] . ").<br>\n";
                        $k = $ak;
                        $m = $am;
                        $found = true;
                        unset($test_users[$k]);
                    }
                    foreach ($test_users as $l => $ad_user) {
                        if (! preg_match("/" . $config['dn']['trash'] . "/i", $ad_user['dn'])) {
                            $ad_user['trash'] = true;
                            $t = [
                                $l => [
                                    'ent' => [],
                                    'ad' => $ad_user
                                ]
                            ];
                            $html .= "Compte ad en doublon : <b> " . $ad_user['cn'] . "</b> à supprimer<br>";
                        } else {
                            $html .= "Compte ad en doublon : <b> " . $ad_user['cn'] . "</b> <br>";
                        }
                        unset($ad_users[$l]);
                    }
                }
                // on passe en correspondance approximative si $auto est faux
                if (! $auto && ! $found && $almost_found) {
                    $html .= "Correspondance partielle considérée comme suffisante entre le compte ENT <b>" . $am['ent']["firstName"] . " " . $am['ent']["lastName"] . "</b> (" . $am['ent']["login"] . ") et le compte AD <b>" . $am['ad']["prenom"] . " " . $am['ad']["nom"] . "</b> (" . $am['ad']['cn'] . ").<br>\n";
                    $k = $ak;
                    $m = $am;
                    $found = true;
                }
            }
            if ($found && ! $manu) {
                // Association en cas de correspondance
                if ($asm || $ent_user['type'] == "attributs") {
                    $html .= "Trouvé : Compte Apple <b> " . $ent_user['firstName'] . " " . $ent_user['lastName'] . "</b> (AD " . $m['ad']['cn'] . ") <br>";
                } else {
                    if (@$config['ent_autorename'] == "1") {
                        // if ((($ent_user['originalLogin'] == $ent_user['login'] || $config['force_ad_alias']) && ($ent_user['login'] != $m['ad']['cn'])) || strlen($m['ad']['cn']) > 19) { // || mk_cn($config, $ent_user['lastName'], $ent_user['firstName'], true) != $m['ad']['cn']) {
                        $alias = false;
                        $msg = "";
                        if (strlen($m['ad']['cn']) > 19) {
                            // $alias = mk_cn($config, $ent_user['lastName'], $ent_user['firstName']);
                            $alias = $ent_user['test_login'];
                            $msg = "  <b>" . $alias . "</b> car le le login est trop long.";
                        } elseif (! empty($ent_user['login']) && ! empty($ent_user['originalLogin']) && $ent_user['login'] != $ent_user['originalLogin'] && $ent_user['login'] != $m['ad']['cn']) {
                            // un alias est défini, on le garde
                            $alias = $ent_user['login'];
                            $msg = ", on tente de garder : <b>" . $alias . "</b>.";
                        } elseif (! empty($ent_user['login']) && ! empty($ent_user['originalLogin']) && $ent_user['login'] == $ent_user['originalLogin'] && $ent_user['originalLogin'] != $m['ad']['cn']) {
                            // $alias = mk_cn($config, $ent_user['lastName'], $ent_user['firstName']);
                            $alias = $ent_user['test_login'];
                            if ($alias == $m['ad']['cn']) {
                                $msg = ", on tente de garder l'AD : <b>" . $alias . "</b>.";
                            } else {
                                $msg = ", on tente de simplifier en : <b>" . $alias . "</b>.";
                            }
                        } elseif (empty($ent_user['originalLogin']) && levenshtein($ent_user['test_login'], $m['ad']['cn']) > 6) {
                            if (! empty($ent_user['email']) && $ent_user['type'] != "Student" && levenshtein(strtolower(explode("@", $ent_user['email'])[0]), $m['ad']['cn']) < 4) {
                                // conservation du login si pas eleve et mail ressemblant
                                $msg = ", le login ad " . $m['ad']['cn'] . " correspond au mail, " . $ent_user['email'] . " on le garde ";
                            } else {
                                $alias = $ent_user['test_login'];
                                $msg = ", le login ad " . $m['ad']['cn'] . " ne correspond pas du tout au nom, on renomme : <b>" . $alias . "</b>.";
                            }
                        } else {
                            $alias = false;
                        }
                        // if ((! empty($ent_user['login']) && $ent_user['login'] != $m['ad']['cn'] && ! empty($ent_user['originalLogin']) && $ent_user['originalLogin'] != $m['ad']['cn']) || strlen($m['ad']['cn']) > 19) { // || mk_cn($config, $ent_user['lastName'], $ent_user['firstName'], true) != $m['ad']['cn']) {
                        if ($alias) {
                            $m['rename'] = $alias;
                            $rename ++;
                        }
                        if (! empty($msg)) {
                            $html .= "Compte ENT <b> " . $ent_user['firstName'] . " " . $ent_user['lastName'] . "</b> (ENT " . $ent_user['originalLogin'] . "/" . $ent_user['login'] . ", AD " . $m['ad']['cn'] . ") : ";
                            $html .= " à renommer" . $msg . "<br>\n";
                        }
                    } elseif ($ent_user['type'] == "trash") {
                        $m['ad']['trash'] = true;
                        $m = [
                            'ent' => $ent_user,
                            'ad' => $m['ad']
                        ];
                        $html .= "Compte ENT <b> " . $m['ad']['cn'] . " à supprimer</b>";
                    }
                    // par défaut on pne chage pas l'association
                }
                $i ++;
            } else {
                // nouvel utilisateur
                // $html .= "add : <pre>" . print_r($ent_user, true) . "</pre>";
                if (! empty($ent_user['lastName'])) {
                    if (empty($ent_user['firstName'])) {
                        $ent_user['firstName'] = "";
                    }

                    // homonymes : on teste le login sur les correspondances déjà faites et sur l'AD
                    $login = $ent_user['test_login'];
                    $nl = 0;
                    while ((($k1 = array_search($ent_user['login'], array_column(array_column($match, 'ent'), 'login'))) !== false || ($k2 = array_search($ent_user['login'], array_column($ad_users, 'cn'))) !== false) && $nl < 50) {
                        $ent_user['login'] = mk_cn($config, $ent_user["lastName"], $ent_user["firstName"], NULL, $nl ++);
                    }
                    if ($distance > 1) {
                        $html .= $msg_almost_found;
                    } elseif ($almost_found) {
                        $id = $ent_user["id"] ?? $ent_user["Id GPEI"];
                        $html .= $msg_almost_found;
                        $html .= "Le compte <b>" . $ent_user["firstName"] . " " . $ent_user["lastName"] . "</b> (" . $login . " / " . $ent_user['login'] . ") n'a pas pu être mis en correspondance malgré les similitudes,
un nouveau sera donc créé sur l'AD. Vérifier éventuellement qu'il ne s'agit pas d'un doublon.
Il est possible de forcer la correspondance en modifiant la valeur de l'<b>Id ENT</b>
de l'un des comptes similaires par <b>" . $id . "</b>
dans la partie annuaire de l'interface SambaÉdu. Dans ce cas, relancer l'importation après modification.<br>\n";
                    }
                    // if (! isset($ent_user['login']) || strcasecmp($ent_user['login'], mk_cn($config, $ent_user['login'], "")) != 0) {
                    // $ent_user['login'] = $ent_user['test_login'];
                    // }
                    // if (((! empty($ent_user['allClasses'][0]['name']) || ! empty($ent_user['functions'][0]['etab'])) && ! empty($ent_user['type']) && $auto) || $manu) {
                    if ((! empty($ent_user['allClasses'][0]['name']) || isset($ent_user['functions'][0]['etab']) || ! empty($ent_user['fonction'])) && ! empty($ent_user['type']) && $auto) {
                        $m = [
                            'ent' => $ent_user,
                            'ad' => array()
                        ];
                        $n ++;
                        $html .= "Compte ENT <b>" . $ent_user['firstName'] . " " . $ent_user['lastName'] . "</b> (" . $login . ") : ";
                        $html .= "nouvel utilisateur, login demandé : <b>" . $ent_user['login'] . "</b>.<br>\n";
                    } else {
                        $new[] = $ent_user;
                        // $html .= " pas de classe : non traité.<br>";
                    }
                }
            }
            if ($found) {
                unset($ad_users[$k]);
            }
            $found = false;
            break;
        case "Etab":
            if (! empty($ent_user['UAI'])) {
                if (etab_uai($config, $ent_user['Id GPEI']) == 0) {
                    $m = [
                        'ent' => $ent_user,
                        'ad' => array()
                    ];
                    $html .= " Nouvel établissement : " . etab_uai($config, $ent_user['Id GPEI']) . " " . $ent_user['name'] . " " . $ent_user['UAI'] . ".<br>\n";
                    $n ++;
                }
            } else {
                if (etab_uai($config, $ent_user['Id GPEI']) != 0) {
                    $m = [
                        'ent' => $ent_user,
                        'ad' => array()
                    ];
                    $html .= " Etablissement à supprimer : " . etab_uai($config, $ent_user['id']) . ".<br>\n";
                }
            }

        // $html .= " compte parent, non traité.<br>";
    }
    if (isset($m)) {
        $match[] = $m;
    }
    if (isset($t) && is_array($t)) {
        $match = array_merge($match, $t);
    }

    cache_store('new', $new, $config['ent_timeout']);
    apcu_store('rename', $rename, $config['ent_timeout']);
    // $html .= "<pre>" . print_r($match, true) . "</pre>";
    return $n;
}

function match_ad_from_ent($config, $duree, $auto = false, $asm = false, $manu = false)
{
    apcu_store("match_lock", true);
    $html_1 = "";
    $ad_users = cache_fetch('ad_users');
    if (empty($ad_users)) {
        $ad_users = ldap_users($config);
        $html_1 = "<b>Récupération des comptes AD :</b> " . count($ad_users) . " utilisateurs<br>\n";
    }
    $ent = cache_fetch('ent');
    if (! $match = cache_fetch('match')) {
        $match = array();
    }
    if (! $n = apcu_fetch('n_new')) {
        $n = 0;
    }
    $total = apcu_fetch('total');
    if (! $total && is_array($ent)) {
        $total = count($ent);
        apcu_store('total', $total);
    }
    if (! $html = apcu_fetch('ent_html')) {
        $html = "<b>Début de la mise en correspondance</b><br>\n";
    }
    $html .= $html_1;
    if ($ad_users === false) {
        $html .= "ERREUR problème d'accès à l'AD<br>\n";
        unset($ent);
    }
    if (is_array($ent)) {
        $res = count($ent);
        $time = time();
        while ($res -- > 0 && (time() - $time < $duree)) {
            $ent_user = array_shift($ent);
            $n += match_ad($config, $ent_user, $ad_users, $match, $auto, $asm, $manu, $html);
        }
        cache_store('match', $match, $config['ent_timeout']);
        apcu_store('n_new', $n);
        cache_store('ad_users', $ad_users, $config['ent_timeout']);

        if ($res > 0) {
            $end = false;
            cache_store('ent', $ent, $config['ent_timeout']);
            $html .= "Mise en correspondance :<br> " . ($total - $res - $n) . " utilisateurs sur " . $total . " ont été mis en correspondance automatiquement.<br>\n";
            $html .= "Nouveaux utilisateurs : " . $n . " reste AD : " . count($ad_users) . "<br>\n";
        } else {
            $end = true;
            cache_delete('ent');
            if (! empty($config['etab_ou'])) {
                $html .= "<b>Mise en correspondance terminée pour " . $config['etab_ou'] . " !</b><br>\n";
            } else {
                $html .= "<b>Mise en correspondance terminée !</b><br>\n";
            }
            // on dégage les utilisateurs des autres etabs de la liste des corresp manuelles et suppr
            foreach ($ad_users as $k => $user) {
                if (! empty($config['etab_ou']) && $user['etab'] != $config['etab_ou']) {
                    unset($ad_users[$k]);
                }
            }
            cache_store('ad_users', $ad_users, $config['ent_timeout']);
            $classes = ldap_users_group($config, $ad_users);
            $e = 0;
            foreach ($classes['eleve'] as $classe) {
                $e += count($classe);
            }
            $net = 0;
            foreach (array_column($match, "ent") as $t) {
                if (isset($t['type']) && $t['type'] == "Etab") {
                    $net ++;
                }
            }
            if ($net > 0) {
                $html .= "$net établissements ont été mis en correspondance automatiquement.\n";
            }
            if (isset($classes['sans'])) {
                $s = count($classes['sans']);
            } else {
                $s = "aucun";
            }
            $html .= ($total - $n - $net) . " utilisateurs ont été mis en correspondance automatiquement.\n";
            $html .= "<ul>\n<li>Nouveaux utilisateurs : " . $n . "</li>\n";
            $html .= "<li>Comptes à renommer : " . apcu_fetch('rename') . "</li>\n";
            $html .= "<li>Comptes AD sambaEdu sans correspondance ENT : \n<ul>\n<li>Profs : " . count($classes['prof']) . "</li>\n<li>Eleves : " . $e . "</li>\n<li>Administratifs : " . count($classes['administratif']) . "</li>
<li>Affectation non définie : " . $s . "</li>\n</ul>\n</li>\n";
            if (! $new = cache_fetch("new")) {
                $new = [];
            }
            $html .= "<li>Comptes ENT sans correspondance dans AD SambaEdu, à traiter manuellement : " . (count($new)) . "</li>\n</ul>\n";
            apcu_delete("match_lock");
        }
    } else {
        $html .= "ERREUR pas de données à importer<br>\n";
        apcu_delete("match_lock");
        $end = true;
    }
    // $html .= "<pre>" . print_r($new, true) . "</pre>";

    // $html .= "<pre>" . print_r($match, true) . "</pre>";

    apcu_store('ent_html', $html, $config['ent_timeout']);
    return $end;
}

// POUR RÉFÉRENCE, NON UTILISÉ
// function merge_pronote($config)
// {
// $m = 0;
// $html = match_ad($config, true);
// $ent_match = cache_fetch('ent');
// $ent_new = cache_fetch('new');
// // $html .= match_ad($config, true, true);
// $pronote_match = cache_fetch('ent');
// $pronote_new = cache_fetch('new');
// foreach ($ent_match as $i => $ent_user) {
// if ($ent_user['ent']['type'] == "Teacher") {
// foreach ($pronote_match as $pronote_user) {
// if ($pronote_user['ent']['type'] == "Teacher" && $pronote_user['ent']['login'] == $ent_user['ent']['login']) {
// $ent_match[$i]['ent']['allClasses'] = $pronote_user['ent']['allClasses'];
// $m ++;
// break;
// }
// }
// }
// }
// foreach ($ent_new as $j => $ent_user) {
// if ($ent_user['type'] == "Teacher") {
// foreach ($pronote_new as $pronote_user) {
// if ($pronote_user['type'] == "Teacher" && $pronote_user['login'] == $ent_user['login']) {
// $ent_user['allClasses'] = $pronote_user['allClasses'];
// $ent_match[] = [
// 'ent' => $ent_user,
// 'ad' => array()
// ];
// $m ++;
// unset($ent_new[$j]);
// break;
// }
// }
// }
// }

// $html .= "Fusion Pronote : " . $m . " comptes profs ont été fusionnés.<br>\n";
// // $html .= "Comptes ENT sans correspondance dans AD SambaEdu, à traiter manuellement : " . count($new) . "<br>";
// cache_store('new', $ent_new, $config['ent_timeout']);
// cache_store('match', $ent_match, $config['ent_timeout']);
// return $html;
// }
function match_ent_users($config, $ad_cns)
{
    $match = cache_fetch('match');
    $ent = cache_fetch('new');
    $ad_users = cache_fetch('ad_users');
    $n = $m = 0;
    $html = count($ad_cns) . "correspondances manuelles : <br>\n";
    foreach ($ad_cns as $login => $ad_cn) {
        // $html .= "$login => $ad_cn<br>";
        if (! empty($ad_cn) && $ad_cn != "admin") {
            if ($ad_cn == "#new#") {
                // nouveau
                $ad_user = array();
                $n ++;
            } else {
                foreach ($ad_users as $k => $ad_user) {
                    if ($ad_user['cn'] == $ad_cn) {
                        $html .= "on retire $k => " . $ad_user['cn'] . "<br>\n";
                        unset($ad_users[$k]);
                        $m ++;
                        break;
                    }
                }
            }
            foreach ($ent as $index => $ent_user) {
                if ($ent_user['login'] == $login) {
                    $match[] = [
                        'ent' => $ent_user,
                        'ad' => $ad_user
                    ];
                    unset($ent[$index]);
                    break;
                }
            }
        }
    }
    cache_store('ad_users', $ad_users, $config['ent_timeout']);
    file_put_contents("/tmp/ad_users", serialize($ad_users));
    cache_store('ent', $ent, $config['ent_timeout']);
    cache_store('match', $match, $config['ent_timeout']);
    $count = count($match) ?? 0;
    $html .= "<br>Mise en correspondance des comptes :<br><br>\n";
    $html .= "$m comptes mis en correspondance et $n nouveaux comptes à créer.<br>\n";
    $html .= "Il reste " . $count . " correspondances à traiter.<br>\n";
    return $html;
}

function rename_ent_users($config, $ad_cns)
{
    $match = cache_fetch('match');
    $m = 0;
    $html = count($ad_cns) . "renommage de :";
    foreach ($ad_cns as $login => $ad_cn) {
        foreach ($match as $index => $user) {
            if ($user['ent']['login'] == $login) {
                $match[$index]['rename'] = $login;
                $html .= " " . $login . ",";
                $m ++;
                break;
            }
        }
    }
    cache_store('match', $match, $config['ent_timeout']);

    $html .= "<br>$m compte(s) à renommer.<br>\n";
    return $html;
}

function trash_ad_users($config, $ad_cns)
{
    $ad_users = cache_fetch('ad_users');
    $match = cache_fetch('match');
    $m = 0;
    $html = "Mise à la corbeille de : ";
    foreach ($ad_cns as $login => $ad_cn) {
        foreach ($ad_users as $l => $user) {
            if ($user['cn'] == $login) {
                $ad_users[$l]['trash'] = true;
                $trash = [
                    'ad' => $ad_users[$l],
                    'ent' => []
                ];
                if (is_array($match) && ! in_array($trash, $match)) {
                    array_unshift($match, $trash);
                    $html .= " " . $login . ",";
                    $m ++;
                }
                break;
            }
        }
    }
    cache_store('ad_users', $ad_users, $config['ent_timeout']);
    $match = cache_store('match', $match, $config['ent_timeout']);
    $html .= "<br>$m compte(s) vont être mis à la corbeille.<br>\n";
    return $html;
}

function create_ent_user($config, &$user, &$html = "", &$listing = [])
{
    $uai = 0;
    $result = true;
    $rules = get_password_rule($config);
    $length = $rules['min-pwd-length'] ?? 5;
    if ($rules['complexity'] == "on") {
        $complexity = true;
    } else {
        $complexity = false;
    }
    $random = create_random_password($length, $complexity);
    if (@$config['no_passwd_change'] == "1" || ! empty($user['ent']['force'])) {
        $change = false;
    } else {
        $user['ad']['pwdlastset'] = 0;
        $change = true;
    }
    if (! empty($user['ent']['code'])) {
        // on fixe le password initial à la valeur du code ent, et on doit le changer...
        if ($complexity && empty($user['ent']['force'])) {
            $password = $user['ent']['code'] . "-#ENT#";
        } else {
            $password = $user['ent']['code'];
        }
        $mode = "ent";
    } else {
        $mode = "sambaedu";
        $config['pwdPolicy'] = $config['pwdPolicy'] ?? 0;
        switch ($config['pwdPolicy']) {
            case "0":
            case "1":
                if (! empty($user['ent']['birthDate'])) {
                    if ($complexity) {
                        $password = implode(array_reverse(explode("-", $user['ent']['birthDate']))) . "-#ENT#";
                    } else {
                        $password = implode(array_reverse(explode("-", $user['ent']['birthDate'])));
                    }
                } else {
                    $password = $config['activation_code'] ?? $random;
                }
                break;
            case "2":
                $password = $random;
                break;
            default:
                $password = $config['activation_code'] ?? $random;
                break;
        }
    }
    if (! empty($user['ent']['birthDate'])) {
        $user['ad']["physicaldeliveryofficename"] = encode_birthdate($user['ent']['birthDate']);
    }
    $categorie = "";
    switch ($user['ent']['type']) {
        case "Student":
            $categorie = "Eleves";
            break;
        case "Teacher":
            $categorie = "Profs";
            break;
        case "Personnel":
        case "Guest":
            $categorie = "Administratifs";
            break;
    }
    if (empty($user['ent']['firstName'])) {
        $user['ent']['firstName'] = "x";
    }
    $s = $user['ent']['Id Siecle'] ?? "";
    $g = $user['ent']['Id GPEI'] ?? "";
    $a = $user['ent']['Id ASM'] ?? "";
    $p = $user['ent']['Id Pronote'] ?? "";
    $user['ad']['employeenumber'] = $s . "," . $g . "," . $a . "," . $p;
    $i = $user['ent']['id'] ?? "";
    $x = $user['ent']['externalId'] ?? "";
    $user['ad']['title'] = $i . "," . $x;

    $config['ent_email'] = $config['ent_email'] ?? 0;
    $config['ent_email_domain'] = $config['ent_email_domain'] ?? 0;
    if ($config['ent_email'] == "1" && ! empty($user['ent']['email'])) {
        if ($config['ent_email_domain'] != "0" && ! empty($user['ent']['originalLogin'])) {
            $email = $user['ent']['originalLogin'] . "@" . $config['ent_email_domain'];
        } else {
            $email = $user['ent']['email'] ?? $user['ent']['login'] . "@" . $config['domain'];
        }
    } else {
        $email = $user['ent']['login'] . "@" . $config['domain'];
    }
    if (! empty($categorie) && ! empty($user['ent']['login'])) {

        // rangement de l'utilisateur dans sont étab de rattachement

        if (isset($user['ent']['structure'])) {
            // $html .= $user['ent']['structure'];
            $uai = etab_uai($config, $user['ent']['structure']);
        } else {
            // on cree l'utilisateur dans l'etablisseemnt qui correspond à la console en cours d'utilisation
            $uai = $config['etab_ou'] ?? 0;
            $html .= "Attention l'utilisateur  " . $user['ent']['login'] . " n'a pas de structure attribuée.<br>\n";
        }
        if (! is_array($uai) && $uai != 0) {
            $config = get_config($config, true, true);
            $config['people_rdn'] = "OU=" . $uai . "," . $config['people_rdn'];
            ouadd($config, $config['people_rdn'] . "," . $config['ldap_base_dn']);
            ouadd($config, "OU=" . $categorie . "," . $config['people_rdn'] . "," . $config['ldap_base_dn']);
            if (! empty($user['ent']['fonction'])) {
                $result = ouadd($config, "OU=" . $user['ent']['fonction'] . ",OU=" . $categorie . "," . $config['people_rdn'] . "," . $config['ldap_base_dn']);
                $racine = "OU=" . $user['ent']['fonction'] . ",OU=" . $categorie;
            } else {
                $racine = "OU=" . $categorie;
            }
        } else {
            if (! empty($user['ent']['fonction'])) {
                $result = ouadd($config, "OU=" . $user['ent']['fonction'] . ",OU=" . $categorie . "," . $config['people_rdn'] . "," . $config['ldap_base_dn']);
                $racine = "OU=" . $user['ent']['fonction'] . ",OU=" . $categorie;
            } else {
                $racine = "OU=" . $categorie;
            }
        }
        $user['ad']['dn'] = "cn=" . $user['ent']['login'] . "," . $racine . "," . $config['people_rdn'] . "," . $config['ldap_base_dn'];
        $user['ad']['cn'] = $user['ent']['login'];
        $user['ad']['sn'] = $user['ent']['lastName'];
        $user['ad']['givenname'] = $user['ent']['firstName'];
        $user['ad']['mail'] = $email;
        $user['ad']['displayname'] = $user['ent']['firstName'] . " " . $user['ent']['lastName'];
        $user['ad']['useraccountcontrol'] = 512;
        $user['ad']['unicodepwd'] = iconv("UTF-8", "UTF-16LE", "\"" . $password . "\"");
        // $password = "\"" . $password . "\"";
        // $len = strlen(password);
        // for ($i = 0; $i < $len; $i ++)
        // $pass .= "{$password[$i]}\000";
        // $user['ad']["unicodePwd"] = $pass;

        $result &= create_ad_user($config, $user['ad']);
        // $html .= "<pre>" . $command . "</pre><br>";
        if ($result) {
            $user['ad'] = search_user($config, $user['ent']['login'], "all", true);
            system("sudo mkhomedir_helper " . $user['ad']['cn'] . " 007 /etc/skel/user");
            system("sudo chgrp -R www-admin /home/" . $user['ad']['cn']);

            // on desactive la complexité et on met le code ENT.
            /*
             * if ($complexity && $mode == "ent") {
             * set_password_activation_rule($config);
             * $password = $user['ent']['code'];
             * groupaddmember($config, $user['ad'], "ent_activation");
             * usersetpassword($config, $user['ad']['cn'], $password, $change);
             * groupdelmember($config, $user['ad']['cn'], "ent_activation");
             * }
             */
            require_once "cloud.inc.php";
            // pas sûr que cela fonctionne car le mdp doit être changé avant l'accès au cloud ?
            if (empty($config['no_cloud'])) {
                configure_user_cloud($config, $user['ad']['cn'], $password, $html, true);
            }
            // Ajout a un groupe principal
            $result = groupaddmember($config, $user['ad'], $categorie);
            if (! empty($uai)) {
                $result &= groupaddmember($config, $user['ad'], $uai);
            }
            if (isset($user['ent']['etabs']) && count($user['ent']['etabs']) > 0) {
                foreach ($user['ent']['etabs'] as $id) {
                    if (etab_uai($config, $id) != $uai) {
                        $result &= groupaddmember($config, $user['ad'], etab_uai($config, $id));
                    }
                }
            }
            if (! empty($user['ent']['fonction'])) {
                $result &= groupaddmember($config, $user['ad'], $user['ent']['fonction']);
            }
            $message_etab = "";
            if ($etab = etab_to_name($config, $uai)) {
                $message_etab = " pour " . $etab;
            }
            if ($result) {
                $html .= "Utilisateur ajouté au groupe " . $categorie . $message_etab . ".<br>\n";
            } else {
                $html .= "ERREUR Utilisateur pas ajouté à " . $categorie . $message_etab . ".<br>\n";
            }
            $user['ad'] = search_user($config, $user['ent']['login'], "all", true);
            if (isset($user['ad']['cn'])) {
                $html .= "Utilisateur créé : <b>" . $user['ad']['cn'] . "</b>, type : " . $categorie . ", mot de passe initial : <b>" . $password . "</b> .<br>\n";
                $user['ent']['code'] = $password;
                $listing[$user['ad']['dn']] = [
                    'nom' => $user['ad']['nom'],
                    'pre' => $user['ad']['prenom'],
                    'cla' => $categorie,
                    'structure' => $user['ad']['etab'],
                    'cn' => $user['ad']['cn'],
                    'change' => $change,
                    'mode' => $mode,
                    'pwd' => $password
                ];
                $result = true;
            }
        } else {
            userdel($config, $user['ent']['login']);
            $html .= "ATTENTION l'utilisateur type : " . $categorie . " : " . $user['ent']['login'] . " n'a pas été créé : <br><pre> " . print_r($user['ad'], true) . "</pre><br>\n";
            $result = true;
        }
    } else {
        $html .= "ERREUR l'utilisateur  " . $user['ent']['login'] . " n'a pas été créé car il n'est pas conforme.<br>\n";
        $result = false;
    }
    return $result;
}

function create_temp_user($config, &$html = "", &$listing = [], $type = "invite", $duree = 0, $classe = "")
{
    $rules = get_password_rule($config);
    $length = $rules['min-pwd-length'] ?? 8;
    $cat = [
        'invite' => "invite",
        'Administratifs' => "invite",
        'Eleves' => "eleve",
        'Profs' => "prof"
    ];
    $users = search_ad($config, $cat[$type] . "." . "*" . $config['suffix'], "user");
    // on recherche un n° de compte libre. Les comptes expirent à minuit donc pas besoin de tester l'expiration
    if (is_array($users) && count($users) > 0) {
        $n = 1;
        $ns = [];
        foreach ($users as $user) {
            $u = [];
            if (preg_match("/^" . $cat[$type] . "\.([0-9]+)" . $config['suffix'] . "$/", $user['cn'], $u)) {
                $ns[] = $u[1];
            }
        }
        while (in_array($n, $ns)) {
            $n ++;
        }
    } else {
        $n = 1;
    }
    unset($user);
    // date d'expiration à minuit aujourd'hui par défaut, La duréee ajoute un nombre de jour supplémentaires
    $end = strtotime(date("Y-m-d", strtotime("tomorrow +" . $duree . "day +11hour")));
    // $html .= "fin : " . $end . "<br>";
    // création user
    if (empty($config['no_temp_profiles'])) {
        $login = $cat[$type] . "." . $n . $config['suffix'];
        if ($cat[$type] == 'invite') {
            $categorie = 'Administratifs';
        } else {
            $categorie = $type;
        }
    } else {
        $login = "invite." . $n . $config['suffix'];
        $categorie = "Eleves";
        $cat[$type] = "invite";
        $type = "invite";
    }
    $racine = $config['dn'][$categorie];
    $password = create_random_password($length - 1, false);
    $user['dn'] = "cn=" . $login . "," . $racine;
    $user['cn'] = $login;
    $user['sn'] = $n;
    $user['givenname'] = $cat[$type];
    $user['displayname'] = $cat[$type] . " " . "temporaire " . $n;
    $user['useraccountcontrol'] = 512;
    $user['accountexpires'] = unixTimeToLdapTime($end);
    $user['unicodepwd'] = iconv("UTF-8", "UTF-16LE", "\"" . $password . "\"");
    $result = create_ad_user($config, $user);
    // recherche de la classe
    if ($result) {
        $user = search_user($config, $login, "all", true);
        system("sudo mkhomedir_helper " . $login . " 007 /etc/skel/user");
        system("sudo chgrp -R www-admin /home/" . $login);

        // Ajout a un groupe principal
        $result = groupaddmember($config, $user, $categorie);

        if (! empty($config['cloud_uri']) && empty($config['no_cloud'])) {
            require_once "cloud.inc.php";
            // pas sûr que cela fonctionne tout de suite, alors on le met en attente.
            configure_user_cloud($config, $login, $password, $html, true);
        }
        if ($type != 'invite' && ! empty($classe)) {
            if (! in_array($classe, list_classes_etab_fast($config, $type))) {
                if (! preg_match("/^Temp_/", $classe)) {
                    $classe = "Temp_" . $classe;
                }
                $result &= create_group($config, $classe, "Classe temporaire", "classe");
            }
            $result &= add_user_group($config, $classe, $user, true, false, "classe");
        }
        $uai = $user['etab'];
        if (! empty($uai)) {
            $result &= groupaddmember($config, $user, $uai);
        }
        $user = search_user($config, $login, "all", true);
        // $html .= "<pre>" . print_r($user, true) . "</pre>";
        if (isset($user['cn'])) {
            $html .= "Utilisateur temporaire créé : <b>" . $user['cn'] . "</b>, type : " . $cat[$type] . ", mot de passe initial : <b>" . $password . "</b> .<br>\n";
            $user['ent']['code'] = $password;
            $listing[$user['dn']] = [
                'nom' => $user['nom'],
                'pre' => $user['prenom'],
                'cla' => $cat[$type],
                'structure' => $user['etab'],
                'cn' => $user['cn'],
                'change' => false,
                'mode' => "expire le " . date("d/m/Y H\\h", ldapTimeToUnixTime($user['accountexpires'])),
                'pwd' => $password
            ];
        }
    }
    if (! $result) {
        $html .= "ATTENTION l'utilisateur temporaire type : " . $cat[$type] . " : " . $login . " n'a pas été créé : <br><pre> " . print_r($user, true) . "</pre><br>\n";
    }
    return $result;
}

function delete_temp_user($config, &$html = "", $all = false)
{
    $cat = [
        // 'invite' => "invite",
        'Administratifs' => "invite",
        'Eleves' => "eleve",
        'Profs' => "prof"
    ];
    $users = [];
    $result = true;
    foreach ($cat as $nom) {
        $us = search_ad($config, $nom . "." . "*" . $config['suffix'], "user");
        if (is_array($us) && count($us) > 0) {
            $users = array_merge($users, $us);
        }
    }
    foreach ($users as $user) {
        if (preg_match("/^(invite|eleve|prof)\.([0-9]+)" . $config['suffix'] . "$/", $user['cn']) && ($all || ldapTimeToUnixTime($user['accountexpires']) < time())) {
            if (! empty($config['cloud_uri']) && empty($config['no_cloud'])) {
                require_once "cloud.inc.php";
                delete_nc_user($config, $user['cn']);
            }
            $r = delete_ad($config, $user);
            if ($r) {
                system("sudo rm -fr /home/" . $user['cn']);
                system("sudo rm -fr /home/profiles/" . $user['cn'] . "V*");
                $html .= "Utilisateur temporaire <b>" . $user['cn'] . "</b> effacé.<br>\n";
            } else {
                $html .= "ERREUR Utilisateur temporaire <b>" . $user['cn'] . "</b> non effacé.<br>\n";
            }
        }
    }

    return $result;
}

function map_ldap_modify_attrib(array $tab)
{
    $ret = [];
    $map = array(
        "employeenumber" => "id Siecle/GPEI/ASM/Pronote/NC",
        "userprincipalname" => "login",
        "displayname" => "nom affiché",
        "givenname" => "prénom",
        "password" => "mot de passe",
        "sn" => "nom",
        "title" => "id ENT/AAF"
    );
    foreach ($tab as $elem) {
        if (isset($map[$elem])) {
            $ret[] = $map[$elem];
        } else {
            $ret[] = $elem;
        }
    }
    return $ret;
}

function modify_ad_user($config, $user, &$html, $asm = false, $invalidate_cache = false)
{
    $result = false;
    $modif = [];
    if ($asm) {
        $m = false;
        foreach ($user['ent'] as $ent_attr => $ent_val) {
            switch ($ent_attr) {
                case 'Id ASM':
                    $s = $user['ent']['Id Siecle'] ?? $user['ad']['Id Siecle'] ?? "";
                    $g = $user['ent']['Id GPEI'] ?? $user['ad']['Id GPEI'] ?? "";
                    $a = $user['ent']['Id ASM'] ?? $user['ad']['Id ASM'] ?? "";
                    $p = $user['ent']['Id Pronote'] ?? $user['ad']['Id Pronote'] ?? "";
                    $n = $user['ent']['Id NC'] ?? $user['ad']['Id NC'] ?? "";
                    if (! $m) {
                        if (! isset($user['ad']['employeenumber'])) {
                            $modif[] = [
                                "attrib" => "employeenumber",
                                "modtype" => LDAP_MODIFY_BATCH_ADD,
                                "values" => [
                                    $s . "," . $g . "," . $a . "," . $p . "," . $n
                                ]
                            ];
                            $m = true;
                        } elseif ($user['ad']['Id ASM'] != $a) {
                            $modif[] = [
                                "attrib" => "employeenumber",
                                "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                                "values" => [
                                    $s . "," . $g . "," . $a . "," . $p . "," . $n
                                ]
                            ];
                            $m = true;
                        }
                    }
                    break;
            }
        }
    } else {
        $config['ent_email'] = $config['ent_email'] ?? 0;
        $config['ent_email_domain'] = $config['ent_email_domain'] ?? 0;
        $m = $mi = false;
        foreach ($user['ent'] as $ent_attr => $ent_val) {
            switch ($ent_attr) {
                case '':
                case 'id':
                case 'externalId':
                    $i = $user['ent']['id'] ?? $user['ad']['id'] ?? "";
                    $x = $user['ent']['externalId'] ?? $user['ad']['externalId'] ?? "";
                    if (! $mi) {
                        if (! isset($user['ad']['title'])) {
                            $modif[] = [
                                "attrib" => "title",
                                "modtype" => LDAP_MODIFY_BATCH_ADD,
                                "values" => [
                                    $i . "," . $x
                                ]
                            ];
                            $mi = true;
                        } elseif ($user['ad']['id'] != $i || $user['ad']['externalId'] != $x) {
                            $modif[] = [
                                "attrib" => "title",
                                "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                                "values" => [
                                    $i . "," . $x
                                ]
                            ];
                            $mi = true;
                        }
                    }
                    break;
                case 'Id Siecle':
                case 'Id GPEI':
                case 'Id Pronote':
                case 'Id NC':
                    $user['ad']['Id Siecle'] = $user['ad']['Id Siecle'] ?? "";
                    $user['ad']['Id GPEI'] = $user['ad']['Id GPEI'] ?? "";
                    $user['ad']['Id ASM'] = $user['ad']['Id ASM'] ?? "";
                    $user['ad']['Id Pronote'] = $user['ad']['Id Pronote'] ?? "";
                    $user['ad']['Id NC'] = $user['ad']['Id NC'] ?? "";

                    $s = $user['ent']['Id Siecle'] ?? $user['ad']['Id Siecle'];
                    $g = $user['ent']['Id GPEI'] ?? $user['ad']['Id GPEI'];
                    $a = $user['ent']['Id ASM'] ?? $user['ad']['Id ASM'];
                    $p = $user['ent']['Id Pronote'] ?? $user['ad']['Id Pronote'];
                    $n = $user['ent']['Id NC'] ?? $user['ad']['Id NC'];
                    if (! $m) {
                        if (! isset($user['ad']['employeenumber'])) {
                            $modif[] = [
                                "attrib" => "employeenumber",
                                "modtype" => LDAP_MODIFY_BATCH_ADD,
                                "values" => [
                                    $s . "," . $g . "," . $a . "," . $p . "," . $n
                                ]
                            ];
                            $m = true;
                        } elseif ($user['ad']['Id Siecle'] != $s || $user['ad']['Id GPEI'] != $g || $user['ad']['Id ASM'] != $a || $user['ad']['Id Pronote'] != $p || $user['ad']['Id NC'] != $n) {
                            $modif[] = [
                                "attrib" => "employeenumber",
                                "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                                "values" => [
                                    $s . "," . $g . "," . $a . "," . $p . "," . $n
                                ]
                            ];
                            $m = true;
                        }
                    }
                    break;
                case 'originalLogin':
                    if (! isset($user['ad']['userprincipalname'])) {
                        $modif[] = [
                            "attrib" => "userprincipalname",
                            "modtype" => LDAP_MODIFY_BATCH_ADD,
                            "values" => [
                                $ent_val . "@" . $config['domain']
                            ]
                        ];
                    } elseif ($user['ad']['userprincipalname'] != $ent_val . "@" . $config['domain']) {
                        $modif[] = [
                            "attrib" => "userprincipalname",
                            "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                            "values" => [
                                $ent_val . "@" . $config['domain']
                            ]
                        ];
                    }
                    break;
                case 'firstName':
                    if (! isset($user['ad']['prenom'])) {
                        $modif[] = [
                            "attrib" => "givenname",
                            "modtype" => LDAP_MODIFY_BATCH_ADD,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    } elseif ($user['ad']['prenom'] != $ent_val) {
                        $modif[] = [
                            "attrib" => "givenname",
                            "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    }
                    break;
                case 'email':
                    if (! empty($config['ent_email'])) {
                        if (! empty($config['ent_email_domain'])) {
                            $ent_val = $user['ent']['originalLogin'] . "@" . $config['ent_email_domain'];
                        } elseif (empty($ent_val)) {
                            $ent_val = $user['ent']['login'] . "@" . $config['domain'];
                        }
                    } else {
                        $ent_val = $user['ent']['login'] . "@" . $config['domain'];
                    }

                    if (! isset($user['ad']['email'])) {
                        $modif[] = [
                            "attrib" => "mail",
                            "modtype" => LDAP_MODIFY_BATCH_ADD,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    } elseif ($user['ad']['email'] != $ent_val) {
                        $modif[] = [
                            "attrib" => "mail",
                            "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    }
                    break;
                case 'lastName':
                    if (! isset($user['ad']['nom'])) {
                        $modif[] = [
                            "attrib" => "sn",
                            "modtype" => LDAP_MODIFY_BATCH_ADD,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    } elseif ($user['ad']['nom'] != $ent_val) {
                        $modif[] = [
                            "attrib" => "sn",
                            "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    }
                    break;
                case 'birthDate':
                    if (isset($user['ad']['hash'])) {
                        if ($ent_val != decode_birthdate($user['ad']['hash'])) {
                            $encrypt = encode_birthdate($ent_val);
                            $modif[] = [
                                "attrib" => "physicaldeliveryofficename",
                                "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                                "values" => [
                                    $encrypt
                                ]
                            ];
                        }
                    } else {
                        $encrypt = encode_birthdate($ent_val);
                        $modif[] = [
                            "attrib" => "physicaldeliveryofficename",
                            "modtype" => LDAP_MODIFY_BATCH_ADD,
                            "values" => [
                                $encrypt
                            ]
                        ];
                    }
                    break;
                case 'displayName':
                    if (! isset($user['ad']['fullname'])) {
                        $modif[] = [
                            "attrib" => "displayname",
                            "modtype" => LDAP_MODIFY_BATCH_ADD,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    } elseif ($user['ad']['fullname'] != $ent_val) {
                        $modif[] = [
                            "attrib" => "displayname",
                            "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                            "values" => [
                                $ent_val
                            ]
                        ];
                    }
                    break;
                case 'password':
                    $modif[] = [
                        "attrib" => "unicodepwd",
                        "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                        "values" => [
                            $ent_val = iconv("UTF-8", "UTF-16LE", "\"" . $ent_val . "\"")
                        ]
                    ];
                    break;
                // case 'login':
                // if (strcasecmp($user['ad']['samaccountname'], $ent_val) != 0) {
                // $modif[] = [
                // "attrib" => "samaccountname",
                // "modtype" => LDAP_MODIFY_BATCH_REPLACE,
                // "values" => [
                // $ent_val
                // ]
                // ];
                // }
                // break;
                // pour le moment on s'arrête là, on pourrait completer d'autres attributs...
            }
        }
    }
    if (count($modif) == 0) {
        $result = true;
    } else {
        $dn = $user['ad']['dn'];
        if ($config['bind']) {
            $result = ldap_modify_batch($config['bind'], $dn, $modif);
        }
        $m = implode(", ", map_ldap_modify_attrib(array_column($modif, 'attrib')));
        if ($result) {
            if ($invalidate_cache) {
                apcu_store("ldap_cache_invalid", true, 60);
            }
            $html .= "Utilisateur " . $user['ad']['cn'] . " mis à jour : " . $m . ".<br>\n";
        } else {
            $html .= "ERREUR : Utilisateur " . $user['ad']['cn'] . " / " . $dn . " pas mis à jour <br>\n";
            $html .= ldap_error($config['bind']) . "<br>";
            $html .= "<pre>" . print_r($modif, true) . print_r($user['ad'], true) . "</pre><br>\n";
        }
    }

    return $result;
}

function move_ad_user($config, &$user, &$html)
{
    // deplace l'utilisateur dans ou=UAI
    // le met dans le groupe cn=UAI
    // ne fait rien si pas d'uai défini
    $result = true;
    if (! isset($user['ent']['update']) || $user['ent']['update'] !== false) {
        $move = false;
        $new_uai = 0;
        if (isset($user['ent']['structure'])) {
            if (etab_uai($config, $user['ent']['structure']) != 0) {
                $new_uai = etab_uai($config, $user['ent']['structure']);
            } else {
                $new_uai = 0;
            }
        } else {
            $html .= "ERREUR l'utilisateur  " . $user['ent']['login'] . " n'a pas de structure attribuée.<br>\n";
            return false;
        }
        // definition de la branche destination
        $old_uai = $user['ad']['etab'];
        if ($new_uai != 0) {
            // if ($new_uai != $etab_uai) {
            // if ($etab_uai == 0) {
            // $config['people_rdn'] = "OU=" . $new_uai . "," . $config['people_rdn'];
            // } else {
            // $config['people_rdn'] = preg_replace("/" . $etab_uai . "/", $new_uai, $config['people_rdn']);
            // }
            // $move = true;
            // } else
            if ($new_uai != $old_uai) {
                if ($old_uai == 0) {
                    $config['people_rdn'] = "OU=" . $new_uai . "," . $config['people_rdn'];
                } else {
                    $config['people_rdn'] = preg_replace("/" . $old_uai . "/", $new_uai, $config['people_rdn']);
                }
                $move = true;
            }
        } else {
            // retour sur central
            // $config = get_config($config, true, true);
            // return false;
        }
        // verif categorie

        $cat = ldap_dn2cn_case(ldap_dn2oudn($user['ad']['dn']));
        if (preg_match("/Direction|Secretariat|Documentaliste|Medical|VieScol|Agent|AED|Tech|AESH|Autres/", $cat)) {
            $fonc = $cat;
            $cat = ldap_dn2cn_case(ldap_dn2oudn(ldap_dn2oudn($user['ad']['dn'])));
        } else {
            $fonc = "";
        }
        switch ($user['ent']['type']) {
            case "Student":
                $categorie = "Eleves";
                break;
            case "Teacher":
                $categorie = "Profs";
                break;
            default:
                $categorie = "Administratifs";
                break;
        }
        if ($cat != $categorie) {
            $move = true;
        }
        // verif fonction
        if (! empty($user['ent']['fonction']) && (($categorie == "Administratifs" && ($user['ent']['fonction'] != "Documentaliste" && $user['ent']['fonction'] != "AESH")) || (($user['ent']['fonction'] == "Documentaliste" || $user['ent']['fonction'] == "AESH") && $categorie == "Profs"))) {
            ouadd($config, "OU=" . $user['ent']['fonction'] . ",OU=" . $categorie . "," . $config['people_rdn'] . "," . $config['ldap_base_dn']);
            $racine = "OU=" . $user['ent']['fonction'] . ",OU=" . $categorie;
        } else {
            $racine = "OU=" . $categorie;
            $user['ent']['fonction'] = "";
        }
        if (isset($user['ent']['fonction']) && $fonc != $user['ent']['fonction']) {
            $move = true;
        }
        // deplacement de l'utilisateur dans l'arborescence

        if ($move) {
            // $config = check_ad($config, $new_uai);
            $config = get_config($config, true, $new_uai);
            $new_dn = "CN=" . $user['ad']['cn'] . "," . $racine . "," . $config['people_rdn'] . "," . $config['ldap_base_dn'];
            $result &= move_ad($config, $user['ad']['cn'], $new_dn, "user");
            if ($result) {
                $user['ad']['dn'] = $new_dn;
                $html .= "Utilisateur " . $user['ad']['cn'] . " déplacé dans " . $new_dn . " depuis " . $old_uai . "<br>\n";
            } else {
                $html .= "ATTENTION uai : " . $new_uai . " : Utilisateur " . $user['ad']['cn'] . " non déplacé vers " . $new_dn . " depuis " . $old_uai . "<br>\n";
                $result = true; // hack temporaine pour ne pas bloquer
            }
        }

        // suppression dans les groupes etablissements, categorie et fonction
        $old_etabs = [];
        $gc = $gf = false;
        if (isset($user['ad']['memberof'])) {
            foreach ($user['ad']['memberof'] as $k => $g) {
                $m = [];
                if (preg_match("/CN=([0-9]{7}[a-z])," . $config['dn']['etablissements'] . "/i", $g, $m)) {
                    // anciens étabs
                    $old_etabs[$k] = $m[1];
                } elseif (preg_match("/CN=(Direction|Secretariat|Documentaliste|Medical|VieScol|Agent|AED|Tech|AESH|Autres)/", $g, $m)) {
                    // on retire l'utilisateur des anciennes fonctions
                    if (isset($user['ent']['fonction']) && $m[1] != $user['ent']['fonction']) {
                        $result |= groupdelmemberbydn($config, $user['ad']['dn'], $g);
                        unset($user['ad']['memberof'][$k]);
                    } else {
                        $gf = true;
                    }
                } elseif (preg_match("/CN=(Profs|Eleves|Administratifs)/", $g, $m)) {
                    // on retire l'utilisateur des anciennes catégories
                    if ($m[1] != $categorie) {
                        $result |= groupdelmemberbydn($config, $user['ad']['dn'], $g);
                        unset($user['ad']['memberof'][$k]);
                    } else {
                        $gc = true;
                    }
                }
            }
        }
        // gestion etabs
        if (! empty($user['ent']['etabs'])) {
            $new_etabs = [];
            foreach ($user['ent']['etabs'] as $id) {
                $new_etabs[] = etab_uai($config, $id);
            }
            $add = array_diff($new_etabs, $old_etabs);
            $del = array_diff($old_etabs, $new_etabs);
        } elseif ($old_uai != $new_uai) {
            $add = [
                $new_uai
            ];
            $del = [
                $old_uai
            ];
        } else {
            $add = $del = [];
        }
        foreach ($add as $a) {
            $result &= groupaddmemberbydn($config, $user['ad']['dn'], "CN=" . $a . "," . $config['dn']['etablissements']);
            $user['ad']['memberof'][] = "CN=" . $a . "," . $config['dn']['etablissements'];
        }
        foreach ($del as $k => $d) {
            // if (is_array($user['ad']['memberof']) && $k = array_search("CN=" . $d . "," . $config['dn']['etablissements'], $user['ad']['memberof'])) {
            $result |= groupdelmemberbydn($config, $user['ad']['dn'], "CN=" . $d . "," . $config['dn']['etablissements']);
            unset($user['ad']['memberof'][$k]); // }
        }
        // ajout du groupe categorie
        if ($cat != $categorie || ! $gc) {
            $c = "CN=" . $categorie . "," . get_config($config, true, true)['dn']['groups'];
            groupaddmemberbydn($config, $user['ad']['dn'], $c);
            $user['ad']['memberof'][] = $c;
        }
        // ajout du groupe fonction
        if (! empty($user['ent']['fonction']) && (! $gf || $fonc != $user['ent']['fonction'])) {
            $f = "CN=" . $user['ent']['fonction'] . "," . get_config($config, true, true)['dn']['groups'];
            groupaddmemberbydn($config, $user['ad']['dn'], $f);
            $user['ad']['memberof'][] = $f;
        }
    } else {
        $html .= "Utilisateur " . $user['ad']['cn'] . " protégé : pas de déplacement.<br>\n";
    }
    // apcu_store("ldap_cache_invalid", true, 60);
    return $result;
}

function modify_ad_login($config, $user, &$html)
{
    $old = $user['ad']['cn'];
    $result = true;
    if (count(search_user($config, $user['ent']['login'])) > 0) {
        // $html .= "ERREUR : Utilisateur " . $user['ent']['login'] . " est déjà en doublon dans l'AD, veuillez l'effacer manuellement ou choisir un login ENT différent<br>";
        $result = userdel($config, $user['ent']['login']);
        if ($result) {
            $html .= "Utilisateur " . $user['ent']['login'] . " est en doublon dans l'AD, on le supprime<br>\n";
        } else {
            $html .= "ERREUR : Utilisateur " . $user['ent']['login'] . " est déjà en doublon dans l'AD, et n'a pas pu être supprimé. Veuillez l'effacer manuellement ou choisir un login ENT différent<br>\n";
            return false;
        }
    }
    $result &= move_ad($config, $user['ad']['cn'], "cn=" . $user['ent']['login'] . "," . ldap_dn2oudn($user['ad']['dn']), "user");
    if ($result) {
        $attrs = array();
        $attrs['samaccountname'] = $user['ent']['login'];
        $result = modify_ad($config, $user['ent']['login'], "user", $attrs);
        system("sudo mv /home/" . $user['ad']['cn'] . " /home/" . $user['ent']['login']);
        system("sudo mv /home/profiles/" . $user['ad']['cn'] . ".V6 /home/profiles/" . $user['ent']['login'] . ".V6");
        if (is_eleve($config, $user['ent']['login'])) {
            $old_rep = invert_login($config, $user['ad']['cn']);
            $new_rep = invert_login($config, $user['ent']['login']);
            foreach (list_classes($config, $user) as $classe) {
                system("sudo mv '/var/sambaedu/Classes/Classe_" . $classe . "/" . $old_rep . "' '/var/sambaedu/Classes/Classe_" . $classe . "/" . $new_rep . "'");
            }
        }
        $html .= "Utilisateur " . $old . " renommé en " . $user['ent']['login'] . " dans l'AD et dans les dossiers du FS.<br>\n";
    } else {
        $html .= "ERREUR : Utilisateur " . $user['ad']['cn'] . " non renommé <br>\n";
    }
    return $result;
}

/**
 * crée ou met à jour un utilisateur à partir de du tableau des correspondances ent.
 * utilise le tableau 'match' en cache apcu, et enleve l'utilisateur en cas de succès.
 *
 * @param unknown $config
 * @return string
 */
function update_user_ent($config, &$match, &$group, &$html, $remove = false, $manu = false, $asm = false, $mode = "all", $listing_name = "listing")
{
    $client = NULL;
    $result = true;
    $del = false;
    $ad_users = cache_fetch('ad_users');
    if (! $listing = apcu_fetch($listing_name)) {
        $listing = [];
    }
    // $codes = json_decode(file_get_contents("/etc/sambaedu/codes.json"), true);
    if (is_array($match) && count($match) > 0) {
        $user = array_shift($match);

        // $html .= "Utilisateur " . $user['ent']['login'] . "<br>";
        // $html .= "<pre>" . print_r($user, true) . "</pre>";
        $user['ent']['type'] = $user['ent']['type'] ?? "";
        switch ($user['ent']['type']) {
            case "Etab":
                if (! isset($user['ad']['cn'])) {
                    $result = add_etab($config, $user['ent']);
                    if ($result) {
                        $html .= "Nouvel Etablissement " . $user['ent']['UAI'] . "<br>\n";
                    } else {
                        $html .= "Erreur impossible d'ajouter l'etablissement " . $user['ent']['UAI'] . "<br>\n";
                    }
                } else {
                    $html .= "Erreur impossible de supprimer l'etablissement " . $user['ent']['UAI'] . ". Non implémenté<br>\n";
                    $result = false;
                }
                if ($result) {
                    cache_store('match', $match, $config['ent_timeout']);
                    return count($match);
                } else {
                    return false;
                }
                break;
            default:
                if ($mode != "group") {
                    $rename = $user['rename'] ?? false;
                    if ($rename && ! empty($config['ent_remove_alias'])) {
                        $full_login = $user['ent']['originalLogin'];
                    } else {
                        $full_login = $user['ent']['login'] ?? $user['ent']['test_login'] ?? "";
                    }
                    if (empty($full_login) && $remove) {
                        if (isset($user['ad']['cn'])) {
                            if (trash_user($config, $user['ad'])) {
                                $html .= "Déplacement dans la corbeille de l'utilisateur " . $user['ad']['cn'] . "<br>\n";
                                $result = true;
                                $del = true;
                            } else {
                                $html .= "ERREUR : Utilisateur " . $user['ent']['login'] . " n'a pas été mis à la corbeille<br>\n";
                                $result = false;
                            }
                        } else {
                            $html .= "ATTENTION : Utilisateur inconnu <br>\n";
                            $html .= "<pre>" . print_r($user, true) . "</pre>\n";
                            $result = false;
                        }
                    } elseif (! isset($user['ad']['cn'])) {
                        // nouvel utilisateur, on met en place les attributs de base, les autres seront synchronisés automatiquement
                        $user['ad'] = search_user($config, $user['ent']['login'], true);
                        if (is_array($user['ad']) && count($user['ad']) > 0) {
                            // userdel($config, $user['ent']['login']);
                            $result = true;
                            $html .= "ATTENTION : Utilisateur " . $user['ent']['login'] . " est en doublon dans L'ENT, on ne l'importe pas deux fois.<br>\n";
                        } else {
                            $html .= "Nouvel utilisateur : " . $user['ent']['login'] . ".<br>\n";
                            change_ent_alias($config, $client, $ad_users, $user, $html, $manu);
                            $result = create_ent_user($config, $user, $html, $listing);
                            if ($result) {
                                $user['ad'] = search_user($config, $user['ent']['login']);
                            }
                            if (count($user['ad']) == 0) {
                                $result = true;
                                $html .= "ERREUR : Utilisateur " . $user['ent']['login'] . " non créé, on continue.<br>";
                            } else {
                                if ($full_login == $user['ent']['login']) {
                                    $html .= "Utilisateur " . $user['ad']['cn'] . " créé.<br>\n";
                                } else {
                                    $html .= "Alias de login modifié pour l'utilisateur " . $full_login . " => " . $user['ad']['cn'] . ".<br>\n";
                                }
                            }
                        }
                    } else {
                        if (strcasecmp(ldap_dn2oudn($user['ad']['dn']), $config['dn']['trash']) == 0) {
                            $categorie = "";
                            switch ($user['ent']['type']) {
                                case "Student":
                                    $categorie = "Eleves";
                                    break;
                                case "Teacher":
                                    $categorie = "Profs";
                                    break;
                                case "Personnel":
                                case "Guest":
                                    $categorie = "Administratifs";
                                    break;
                            }
                            $result = recup_user(get_config($config, true, etab_uai($config, $user['ent']['structure'])), $user['ad']['cn'], $categorie);
                            if ($result) {
                                $u = search_user($config, $user['ad']['cn'], "all", true);
                                if (isset($u['etab'])) {
                                    $user['ad'] = $u;
                                    $html .= "Utilisateur " . $user['ad']['cn'] . " récupéré et remis dans " . $u['etab'] . ". <br>\n";
                                } else {
                                    $html .= "Utilisateur " . $user['ad']['cn'] . " récupéré. <br>\n";
                                }
                            } else {
                                $html .= "ERREUR : Utilisateur " . $user['ad']['cn'] . " non récupéré. <br>\n";
                            }
                        }
                        if ($rename) { // && $user['ent']['login'] != $user['ad']['cn']) {
                            $html .= "Renommage de " . $user['ad']['cn'] . " :<br>\n";
                            if (! change_ent_alias($config, $client, $ad_users, $user, $html)) {
                                // modif ent impossible, on change AD
                                $result = modify_ad_login($config, $user, $html);
                                if ($result) {
                                    $html .= "Changement d'alias ENT impossible, changement juste dans l'AD " . $full_login . " => " . $user['ad']['cn'] . ".<br>\n";
                                }
                            } elseif ($user['ad']['cn'] != $user['ent']['login']) {
                                // modif ent ok, mais resultat different, on change AD
                                $result = modify_ad_login($config, $user, $html);
                                if ($result) {
                                    $html .= "Alias ENT différent, changement dans l'AD " . $user['ad']['cn'] . " => " . $user['ent']['login'] . ".<br>\n";
                                }
                            }
                            $u = search_user($config, $user['ent']['login'], $config['dn']['people'], true);
                            if (isset($u['dn'])) {
                                $user['ad'] = $u;
                            }
                        }
                        if (empty($user['ent']['displayName'])) {
                            $user['ent']['firstName'] = $user['ent']['firstName'] ?? "";
                            $user['ent']['lastName'] = $user['ent']['lastName'] ?? "";
                            $user['ent']['displayName'] = ucfirst($user['ent']['firstName']) . " " . ucfirst($user['ent']['lastName']);
                        }
                        $result = modify_ad_user($config, $user, $html, $asm);
                        if (! $result) {
                            $html .= "Erreur lors de la mise à jour de l'utilisateur, on continue<br>\n";
                        }
                        if (! $asm) {
                            $result = move_ad_user($config, $user, $html);
                            if (! $result) {
                                $html .= "Erreur lors du déplacement de l'utilisateur, on continue<br>\n";
                                $result = true;
                            }
                        }
                    }
                }
                if ($mode == "group" || $result) {
                    if (! $asm && ! $del) {
                        // gestion des classes et groupes : on ajoute l'utilisateur une fois créé
                        $group[] = $user;
                        apcu_store($listing_name, $listing, $config['ent_timeout']);
                    }
                    return count($match);
                } else {
                    return false;
                }
        }

        /*
         * foreach ($listing as $l){
         * if ($l['change']){
         * $codes[$l['cn']] = $l['pwd'];
         * } else {
         * unset ($codes[$l['cn']]);
         * }
         * }
         * file_put_contents("/etc/sambaedu/codes.json", json_encode($codes));
         */
    } else {
        return false;
    }
}

/**
 * crée ou met à jour le tableau members qui va contenir le dn de tous les mebres des groupes classe, matiere et rights issi de l'en, pour comparaison ultérieure avec les groupes AD
 * ce tableau indexé permet de traiter de façon globale les groupes et d'éviter de devoir faire plusieurs passes
 *
 * @param unknown $config
 * @return string
 */
function list_members_ent($config, &$match, &$members, &$html)
{
    if (count($match) > 0) {
        $user = array_shift($match);
        // recherche des groupes d'appartenance de l'utilisateur
        if (isset($user['ent']['allClasses']) && count($user['ent']['allClasses']) > 0) {
            // $classes = array_column($user['ent']['allClasses'], 'name');
            foreach ($user['ent']['allClasses'] as $ent_groups) {
                // $ent_classes[] = $ent_groups['name'];
                if ((empty($ent_groups['etab']))) {
                    $ent_groups['etab'] = 0;
                }
                switch ($user['ent']['type']) {
                    case "Teacher":
                        $type = 'Equipe';
                        break;
                    case "Student":
                        $type = "Classe";
                        break;
                    default:
                        $type = "";
                }
                if (! empty($type)) {
                    // tableau indexé inverse
                    add_array_unique($user['ad']['dn'], $members['groupes'][etab_uai($config, $ent_groups['etab'])][$ent_groups['name']][$type]['ent']);
                    $members['groupes'][etab_uai($config, $ent_groups['etab'])][$ent_groups['name']]['description'] = $ent_groups['description'] ?? $ent_groups['name'];
                }
            }
        }
        // groupes
        if (isset($user['ent']['allGroups']) && count($user['ent']['allGroups']) > 0) {
            foreach ($user['ent']['allGroups'] as $ent_groups) {
                // $ent_cours[] = $ent_groups['name'];
                if ((empty($ent_groups['etab']))) {
                    $ent_groups['etab'] = 0;
                }
                switch ($user['ent']['type']) {
                    case "Teacher":
                        $type = 'Equipe';
                        break;
                    case "Student":
                        $type = "Cours";
                        break;
                    default:
                        $type = "";
                }
                // on ne prend pas les (.)P[123] sauf si ent_group_part = "1"
                if (! empty($type)) {
                    // $g = [];
                    // if (! empty($config['ent_groups_part']) || ! (preg_match("/^(.*)([-_ ]?)(((G|P|groupe)[-_. ]?[1-9])|([0-9]\/[0-9][-_. ]?[1-9]?))$/iU", $ent_groups['name'], $g) && in_array($g[1], $classes))) {
                    // tableau indexé inverse
                    add_array_unique($user['ad']['dn'], $members['groupes'][etab_uai($config, $ent_groups['etab'])][$ent_groups['name']][$type]['ent']);
                    $members['groupes'][etab_uai($config, $ent_groups['etab'])][$ent_groups['name']]['description'] = $ent_groups['description'] ?? $ent_groups['name'];
                    // }
                }
            }
        }
        // matieres on prend allclassesSuject qui est plus restrictif en n'inclut pas la discipline d'enseignement du prof
        // if (isset($user['ent']['allClassesSubject']) && count($user['ent']['allClassesSubject']) > 0) {
        // $user['ent']['allMatieres'] = $user['ent']['allClassesSubject'];
        // }
        if (isset($user['ent']['allMatieres']) && count($user['ent']['allMatieres']) > 0) {
            if ($user['ent']['type'] == "Teacher") {
                foreach ($user['ent']['allMatieres'] as $matieres) {
                    // tableau indexé inverse des dn
                    add_array_unique($user['ad']['dn'], $members['matieres'][$matieres['name']]['member']['ent']);
                    $members['matieres'][$matieres['name']]['id'] = $matieres['id'] ?? "";
                }
            } /*
               * elseif ($user['ent']['type'] == "Student") {
               * // ajout des dn des classes et cours de l'élève. Ceci permet de restreindre le choix éventuel pour le prof des matières à activer pour les partage. Tous les annuaires ne
               * // fournissent pas l'appartenance des élèves aux matières. Si elles sont présentes on ajoute au groupe matière le groupe Classe_|Cours_ Cela permet ensuite
               * // de proposer pour une classe l'intersection avec les matières du prof
               * foreach ($user['ent']['allMatieres'] as $matieres) {
               * if (isset($user['ent']['allClasses']) && count($user['ent']['allClasses']) > 0) {
               * foreach ($user['ent']['allClasses'] as $ent_groups) {
               * // $ent_classes[] = $ent_groups['name'];
               * $local_config = get_config($config, true, etab_uai($config, $ent_groups['etab']));
               * add_array_unique("CN=Classe_" . $ent_groups['name'] . "," . $local_config['dn']['classes'], $members['matieres'][$matieres['name']]['member']['ent']);
               * // if (! isset($members['matieres'][$matieres['name']]['member']['ent'])) {
               * // $members['matieres'][$matieres['name']]['member']['ent'][] = "CN=Classe_" . $ent_groups['name'] . "," . $local_config['dn']['classes'];
               * // } else {
               * // $members['matieres'][$matieres['name']]['member']['ent'] = array_unique(array_merge($members['matieres'][$matieres['name']]['member']['ent'], [
               * // "CN=Classe_" . $ent_groups['name'] . "," . $local_config['dn']['classes']
               * // ]));
               * // }
               * }
               * }
               * if (isset($user['ent']['allGroups']) && count($user['ent']['allGroups']) > 0) {
               * foreach ($user['ent']['allGroups'] as $ent_groups) {
               * // $ent_classes[] = $ent_groups['name'];
               * $local_config = get_config($config, true, etab_uai($config, $ent_groups['etab']));
               * add_array_unique("CN=Cours_" . $ent_groups['name'] . "," . $local_config['dn']['cours'], $members['matieres'][$matieres['name']]['member']['ent']);
               * // if (! isset($members['matieres'][$matieres['name']]['member']['ent'])) {
               * // $members['matieres'][$matieres['name']]['member']['ent'][] = "CN=Cours_" . $ent_groups['name'] . "," . $local_config['dn']['cours'];
               * // } else {
               * // $members['matieres'][$matieres['name']]['member']['ent'] = array_unique(array_merge($members['matieres'][$matieres['name']]['member']['ent'], [
               * // "CN=Cours_" . $ent_groups['name'] . "," . $local_config['dn']['cours']
               * // ]));
               * // }
               * }
               * }
               * }
               * }
               */
        }
        // Discipline. Pas exploitées pour le moment.
        if (isset($user['ent']['aafFunctions']) && isset($user['ent']['aafFunctions']) && count($user['ent']['aafFunctions']) > 0) {
            foreach ($user['ent']['aafFunctions'] as $matieres) {
                add_array_unique($user['ad']['dn'], $members['disciplines'][$matieres]['member']['ent']);
            }
        }
        // Droits
        if (isset($user['ent']['allRights']) && count($user['ent']['allRights']) > 0) {
            foreach ($user['ent']['allRights'] as $right) {
                add_array_unique($user['ad']['dn'], $members['rights'][$right['name']]['member']['ent']);
                $members['rights'][$right['name']]['id'] = $right['id'] ?? "";
            }
        }
        // ajout AD
        if (isset($user['ad']['memberof']) && count($user['ad']['memberof']) > 0) {
            foreach ($user['ad']['memberof'] as $ad_group) {
                $m = array();
                if (preg_match("/^CN=(Classe|Equipe|Cours)_(.*),/iU", $ad_group, $m)) {
                    // uai de l'etab extrait de la branche AD
                    $etab = ldap_dn2uai($ad_group);
                    add_array_unique($user['ad']['dn'], $members['groupes'][$etab][$m[2]][$m[1]]['ad']);
                } elseif (preg_match("/.*," . $config['dn']['matieres'] . "/i", $ad_group)) {
                    add_array_unique($user['ad']['dn'], $members['matieres'][ldap_dn2cn($ad_group)]['member']['ad']);
                } elseif (preg_match("/.*," . $config['dn']['rights'] . "/i", $ad_group)) {
                    $members['rights'][ldap_dn2cn($ad_group)]['member']['ad'][] = $user['ad']['dn'];
                }
            }
        }
    }
    return count($match);
}

/**
 * crée ou met à jour le tableau members qui va contenir le dn de tous les mebres des groupes classe, matiere et rights issi de l'en, pour comparaison ultérieure avec les groupes AD
 * ce tableau indexé permet de traiter de façon globale les groupes et d'éviter de devoir faire plusieurs passes
 *
 * @param unknown $config
 * @return bool
 */
function diff_members_ad($config, &$members, &$html)
{
    $add = 0;
    $suppr = 0;
    if (isset($members['groupes'])) {
        foreach ($members['groupes'] as $etab => $groupe) {
            $classes = [];
            foreach ($groupe as $nom => $g) {
                $del = true;
                // $del = false;
                unset($t);
                foreach ($g as $type => $membres) {
                    if (isset($membres['ent'])) {
                        if ($type == "Classe" || $type == "Cours") {
                            $t = $type;
                        }
                        if ($type != "Equipe" && count($membres['ent']) > 0) {
                            // le groupe contient des élèves
                            $del = false;
                        }
                        if ($type == "Classe" && count($membres['ent']) > 0) {
                            // classe non vide
                            add_array_unique($nom, $classes);
                        }
                        // diff pour membres
                        $membres['ent'] = $membres['ent'] ?? [];
                        $membres['ad'] = $membres['ad'] ?? [];
                        $members['groupes'][$etab][$nom][$type]['add'] = array_diff($membres['ent'], $membres['ad']);
                        $add += count($members['groupes'][$etab][$nom][$type]['add']);
                        $members['groupes'][$etab][$nom][$type]['del'] = array_diff($membres['ad'], $membres['ent']);
                        $suppr += count($members['groupes'][$etab][$nom][$type]['del']);
                    } elseif (isset($membres['ad'])) {
                        $members['groupes'][$etab][$nom][$type]['del'] = $membres['ad'];
                        $suppr += count($members['groupes'][$etab][$nom][$type]['del']);
                    }
                }
                if (isset($t) && $del && $config['etab_ou'] == $etab) {
                    // si le groupe n'a pas d'élève on supprime aussi l'équipe car is s'agit d'un groupe bidon dans l'emploi du temps
                    // en multi étab le prof peut être mis dans une équipe d'un autre bahut, et dans ce clas la classe sera vide car
                    // les élèves ne seront pas mis pour l'étab en cours de traitement.
                    // dans ce cas on garde l'équipe. Le nettoyage en fin d'import n'y touchera pas non plus car pas dans l'étab.

                    if (isset($members['groupes'][$etab][$nom]['Equipe']['del']) && isset($members['groupes'][$etab][$nom]['Equipe']['ent']) && isset($members['groupes'][$etab][$nom]['Equipe']['ad']) && count($members['groupes'][$etab][$nom]['Equipe']['ad']) > 0) {
                        $members['groupes'][$etab][$nom]['Equipe']['del'] = $members['groupes'][$etab][$nom]['Equipe']['ent'];
                        $suppr -= count($members['groupes'][$etab][$nom]['Equipe']['del']) - count($members['groupes'][$etab][$nom]['Equipe']['ent']);
                    } else {
                        $members['groupes'][$etab][$nom]['Equipe']['del'] = [];
                        $suppr -= count($members['groupes'][$etab][$nom]['Equipe']['del']);
                    }
                    $c = $members['groupes'][$etab][$nom]['Equipe']['add'] ?? [];
                    $add -= count($c);
                    $c = $members['groupes'][$etab][$nom][$t]['add'] ?? [];
                    $add -= count($c);
                    $members['groupes'][$etab][$nom]['Equipe']['add'] = [];
                    $members['groupes'][$etab][$nom][$t]['add'] = [];
                }
            }
            // deuxième passe : on filtre autant que possible les groupes de dédoublement. Cela suppose d'avoir construit le tableau des classes completes avant
            // on recherche des motifs coincidant avec le nom de classes. Attention, peut poser des soucis avec les groupes de niveau en college
            foreach ($groupe as $nom => $g) {
                $del = false;
                $m = [];
                if (isset($g['Cours']) && empty($config['ent_groups_part']) && preg_match("/^(.*)([-_ ]?)(((G|P|groupe)[-_. ]?[1-9])|([0-9]\/[0-9][-_. ]?[1-9]?))$/iU", $nom, $m) && in_array($m[1], $classes)) {
                    $del = true;
                }

                if ($del && isset($members['groupes'][$etab][$nom]['Equipe']['del']) && count($members['groupes'][$etab][$nom]['Cours']['del']) && count($members['groupes'][$etab][$nom]['Equipe']['add']) && count($members['groupes'][$etab][$nom]['Cours']['add'])) {
                    $suppr -= count($members['groupes'][$etab][$nom]['Equipe']['del']);
                    $members['groupes'][$etab][$nom]['Equipe']['del'] = $members['groupes'][$etab][$nom]['Equipe']['ad'] ?? [];
                    $c = $members['groupes'][$etab][$nom]['Equipe']['ad'] ?? [];
                    $suppr += count($c);
                    $suppr -= count($members['groupes'][$etab][$nom]['Cours']['del']);
                    $members['groupes'][$etab][$nom]['Cours']['del'] = $members['groupes'][$etab][$nom]['Cours']['ad'] ?? [];
                    $c = $members['groupes'][$etab][$nom]['Cours']['ad'] ?? [];
                    $suppr += count($c);
                    $add -= count($members['groupes'][$etab][$nom]['Equipe']['add']);
                    $add -= count($members['groupes'][$etab][$nom]['Cours']['add']);
                    $members['groupes'][$etab][$nom]['Equipe']['add'] = [];
                    $members['groupes'][$etab][$nom]['Cours']['add'] = [];
                }
            }
        }
    }
    foreach ($members['matieres'] as $nom => $membres) {
        if (isset($membres['member']) && is_array($membres['member'])) {
            $membres['member']['ent'] = $membres['member']['ent'] ?? [];
            $membres['member']['ad'] = $membres['member']['ad'] ?? [];
            $members['matieres'][$nom]['member']['add'] = array_diff($membres['member']['ent'], $membres['member']['ad']);
            $add += count($members['matieres'][$nom]['member']['add']);
            $members['matieres'][$nom]['member']['del'] = array_diff($membres['member']['ad'], $membres['member']['ent']);
            $suppr += count($members['matieres'][$nom]['member']['del']);
        }
    }
    if (isset($members['rights'])) {
        foreach ($members['rights'] as $nom => $membres) {
            if (isset($membres['member']) && is_array($membres['member'])) {
                $membres['member']['ent'] = $membres['member']['ent'] ?? [];
                $membres['member']['ad'] = $membres['member']['ad'] ?? [];
                $members['rights'][$nom]['member']['add'] = array_diff($membres['member']['ent'], $membres['member']['ad']);
                $add += count($members['rights'][$nom]['member']['add']);
                if (count($membres['member']['ent']) > 0) {
                    // on ne prend en compte que si les droits sont dans l'import
                    $members['rights'][$nom]['member']['del'] = array_diff($membres['member']['ad'], $membres['member']['ent']);
                    $suppr += count($members['rights'][$nom]['member']['del']);
                }
            }
        }
    }
    $members['add'] = $add;
    $members['suppr'] = $suppr;
    return true;
}

/**
 * crée ou met à jour ls groupes un utilisateur à partir de du tableau des correspondances ent.
 * utilise le tableau 'group' en cache apcu, et enleve l'utilisateur en cas de succès.
 * les groupes et classes sont suffixés avec -xxxx de l'uai si besoin.
 *
 * @param unknown $config
 * @return string
 */
function update_group_ent($config, &$members, &$html, $remove = false, $manu = false, $listing_name = "listing")
{
    $time = time();
    $result = false;
    $end = true;
    if (! $listing = apcu_fetch($listing_name)) {
        $listing = [];
    }
    // Exécution dans le contexte etab de la classe à traiter
    // on supprime d'abord puis on ajoute
    $type_dn = [
        "Equipe" => "equipes",
        "Classe" => "classes",
        "Cours" => "cours"
    ];
    $type_msg = [
        "Equipe" => "de l'équipe",
        "Classe" => "à la classe",
        "Cours" => "au cours"
    ];
    foreach ($members['groupes'] as $etab => $groupe) {
        $message_etab = "";
        if ($etab_name = etab_to_name($config, $etab)) {
            $message_etab = " pour " . $etab_name;
        }
        foreach ($groupe as $nom => $g) {
            foreach ($g as $type => $membres) {
                // si le groupe n'a pas d'élève on supprime aussi l'équipe car is s'agit d'un groupe bidon dans l'emploi du temps
                if (isset($type_dn[$type])) {
                    $groupdn = "CN=" . $type . "_" . $nom . "," . get_config($config, true, $etab)['dn'][$type_dn[$type]];
                    if (($manu && $remove) || ($remove && ! empty($config['ent_autogroups']))) {
                        if (isset($membres['del'])) {
                            foreach ($membres['del'] as $k => $m) {
                                $result = groupdelmemberbydn($config, $m, $groupdn);
                                if ($result) {
                                    $html .= ldap_dn2cn($m) . " enlevé " . $type_msg[$type] . " " . $nom . $message_etab . ", <br>\n";
                                    unset($members['groupes'][$etab][$nom][$type]['del'][$k]);
                                    $members['suppr'] --;
                                } else {
                                    $html .= "ERREUR " . ldap_dn2cn($m) . " n'a pas été enlevé de " . $type_msg[$type] . " " . $nom . $message_etab . ". <br>\n";
                                    $members['suppr'] --;
                                }
                            }
                        }
                    }
                    if (isset($membres['add']) && count($membres['add']) > 0) {
                        if (empty($membres['ad'])) {
                            $result = groupaddbydn($config, $groupdn, $type . " " . $g['description']);
                        }
                        foreach ($membres['add'] as $k => $m) {
                            $result = groupaddmemberbydn($config, $m, $groupdn);
                            if ($result) {
                                $html .= ldap_dn2cn($m) . " ajouté " . $type_msg[$type] . " " . $nom . $message_etab . ", <br>\n";
                                if (isset($listing[$m])) {
                                    $listing[$m]['cla'] = $nom;
                                }
                                unset($members['groupes'][$etab][$nom][$type]['add'][$k]);
                                $members['add'] --;
                            } else {
                                $html .= "ERREUR " . ldap_dn2cn($m) . " n'a pas été ajouté " . $type_msg[$type] . " " . $nom . $message_etab . ". <br>\n";
                                $members['add'] --;
                            }
                        }
                    }
                }
                if ((time() - $time) > 50) {
                    $end = false;
                    break;
                }
            }
            if ((time() - $time) > 50) {
                $end = false;
                break;
            }
        }
        if ((time() - $time) > 50) {
            $end = false;
            break;
        }
    }
    // matieres
    if ($end && isset($members['matieres'])) {
        foreach ($members['matieres'] as $nom => $g) {
            if (isset($g['member']) && is_array($g['member'])) {
                $groupdn = "CN=" . $nom . "," . $config['dn']['matieres'];
                // if ($manu && $remove) {
                if (isset($g['member']['del'])) {
                    foreach ($g['member']['del'] as $k => $m) {
                        $result = groupdelmemberbydn($config, $m, $groupdn);
                        if ($result) {
                            $html .= ldap_dn2cn($m) . " enlevé de la matière " . $nom . ", <br>\n";
                            unset($members['matieres'][$nom]['member']['del'][$k]);
                            $members['suppr'] --;
                        } else {
                            $html .= "ERREUR " . ldap_dn2cn($m) . " n'a pas été enlevé de la matière " . $nom . " <br>\n";
                            $members['suppr'] --;
                        }
                    }
                }
                // }
                if (isset($g['member']['add'])) {
                    if (empty($g['member']['ad']) && count($g['member']['add']) > 0) {
                        $description = $g['description'] ?? $nom;
                        $id = $g['id'] ?? "";
                        $result = groupaddbydn($config, $groupdn, $description, $id);
                    }
                    foreach ($g['member']['add'] as $k => $m) {
                        $result = groupaddmemberbydn($config, $m, $groupdn);
                        if ($result) {
                            $html .= ldap_dn2cn($m) . " ajouté à la matière " . $nom . ", <br>\n";
                            unset($members['matieres'][$nom]['member']['add'][$k]);
                            $members['add'] --;
                        } else {
                            $html .= "ERREUR " . ldap_dn2cn($m) . " n'a pas été ajouté à la matière " . $nom . " <br>\n";
                            $members['add'] --;
                        }
                    }
                }
                if ((time() - $time) > 50) {
                    $end = false;
                    break;
                }
            }
        }
    }
    // rights
    if ($end && isset($members['rights'])) {
        foreach ($members['rights'] as $nom => $g) {
            if (isset($g['member']) && is_array($g['member'])) {
                $groupdn = "CN=" . $nom . "," . $config['dn']['rights'];
                if ($manu && $remove) {
                    if (isset($g['member']['del'])) {
                        foreach ($g['member']['del'] as $k => $m) {
                            $result = groupdelmemberbydn($config, $m, $groupdn);
                            if ($result) {
                                $html .= ldap_dn2cn($m) . " enlevé du droit " . $nom . ", <br>\n";
                                unset($members['rights'][$nom]['member']['del'][$k]);
                                $members['suppr'] --;
                            } else {
                                $html .= "ERREUR " . ldap_dn2cn($m) . " n'a pas été enlevé du droit " . $nom . " <br>\n";
                            }
                        }
                    }
                }
                if (isset($g['member']['add'])) {
                    foreach ($g['member']['add'] as $k => $m) {
                        $result = groupaddmemberbydn($config, $m, $groupdn);
                        if ($result) {
                            $html .= ldap_dn2cn($m) . " ajouté au droit " . $nom . ", <br>\n";
                            unset($members['rights'][$nom]['member']['add'][$k]);
                            $members['add'] --;
                        } else {
                            $html .= "ERREUR " . ldap_dn2cn($m) . " n'a pas été ajouté au droit " . $nom . " <br>\n";
                        }
                    }
                }
                if ((time() - $time) > 50) {
                    $end = false;
                    break;
                }
            }
        }
    }

    if (! $end) {
        $html .= "Attention, " . $nom . " traitement trop long interrompu.<br>\n";
        // il faut rafraichir le cache pour la passe suivante qui va concerner le même compte
        apcu_store("ldap_cache_invalid", true, 60);
    }
    apcu_store($listing_name, $listing);
    return $members['add'] + $members['suppr'];
}

/**
 * applique les changements dans l'ad.
 * Peut être long...
 *
 * @todo rendre la fonction asynchrone, avec rappel de la page jusqu'à épuisement du le la liste des modifications
 *       executer le traitement par lots de 100 utilisateurs.
 *       on limite l'execution à 20s pour eviter les timeouts dans l'interface
 * @param unknown $config
 * @return string
 */
function update_ad_from_ent($config, $duree, $remove = false, $asm = false, $manu = false, $mode = "all")
{
    if (! $group = cache_fetch('group')) {
        $group = [];
    }

    // $html .= "<pre>" . print_r($match, true) . "</pre>";
    $res = $size = $nb = 0;
    $match = cache_fetch('match');
    if (is_array($match)) {
        $res = $size = count($match);
    }
    $members = cache_fetch('members');
    if (! is_array($members)) {
        $members['matieres'] = [];
    }
    // on l'efface avant ?
    $html = apcu_fetch('ent_html');
    $html_copie = "";
    if (! empty($html) && ! mb_strpos($html, "but de la mise")) {
        $html .= "<b>Début de la mise à jour de la base : </b><br>\n";
        $html_copie = $html;
    }
    $time = time();
    while ($res > 0 && $nb ++ < $size && (time() - $time < $duree)) {
        $res = update_user_ent($config, $match, $group, $html, $remove, $manu, $asm, $mode);
    }
    $type = "utilisateurs";
    cache_store('match', $match, $config['ent_timeout']);
    cache_store('group', $group, $config['ent_timeout']);
    if (is_array($match) && is_array($group) && $res == 0 && count($match) == 0 && count($group) > 0) {
        $nb = 0;
        $res = $size = count($group);
        while ($res > 0 && $nb ++ < $size && (time() - $time < $duree)) {
            // $res = update_group_ent($config, $group, $html, $remove, $manu);
            // constitution du tableau indexé des groupes ent
            $res = list_members_ent($config, $group, $members, $html);
        }
        if ($res == 0) {
            diff_members_ad($config, $members, $html);
            $html .= $members['add'] . " ajout(s) et " . $members['suppr'] . " suppression(s)  de groupes restants<br>\n";
        }
        // $html .= "<pre>" . print_r($members, true) . "</pre>, <br>";
        // apcu_store('ent_html', $html, $config['ent_timeout']);
        // return true;
        cache_store('group', $group, $config['ent_timeout']);
        cache_store('members', $members, $config['ent_timeout']);
    }
    if ($res == 0 && is_array($match) && count($match) == 0 && is_array($members) && count($members) > 0) {
        $res = $members['add'] + $members['suppr'];
        while ($res > 0 && (time() - $time < $duree)) {
            $res = update_group_ent($config, $members, $html, $remove, $manu);
        }
        if ($res < 0) {
            $res = 0;
        }
        // $html .= "<pre>" . print_r($members, true) . "</pre>, <br>";
        $type = "groupes";
        cache_store('group', $group, $config['ent_timeout']);
        cache_store('members', $members, $config['ent_timeout']);
    }
    if ($res > 0) {
        $html .= $res . " $type restants<br>\n";
        $end = false;
    } else {
        $end = true;
        if ($html == $html_copie) {
            $html .= "Pas de modification à effectuer.<br>\n";
        }
        clean_groups($config, $html);
        $html .= "<b>Import terminé !</b><br>\n";
    }
    apcu_store('ent_count', $res, $config['ent_timeout']);
    apcu_store('ent_html', $html, $config['ent_timeout']);
    apcu_store('ent_type', $type, $config['ent_timeout']);

    return $end;
}

// //////////////////////////////////////////////////////////////////////////////////////////
function form_config_ent($config, $ent, $file = false)
{
    // liste du parametrage par defaut des ent à compléter
    $ent_list = array(
        'openent' => array(
            'description' => "ENT openent",
            'mode' => "openent",
            'auth' => 1,
            'uri' => array(
                'Ile de France' => "https://ent.iledefrance.fr/",
                'Paris' => "https://www.parisclassenumerique.fr/",
                'Essonne' => "https://www.moncollege-ent.essonne.fr/",
                'Seine et Marne' => "https://ent77.seine-et-marne.fr/",
                'Nouvelle Aquitaine' => "https://mon.lyceeconnecte.fr/",
                'Normandie' => "https://ent.l-educdenormandie.fr/",
                'Guadeloupe' => "https://neoconnect.opendigitaleducation.com/",
                'Guyane' => "https://wilapa-guyane.com/",
                'Yvelines' => "https://ent.ecollege78.fr/",
                'Corse' => "https://ent.leia.corsica/"
            ),
            'user' => "",
            'user_password' => "",
            'oauth2_id' => "sambaedu",
            'oauth2_pass' => "",
            'oauth2_redirect_uri' => "https://se4." . $config['domain'] . "/oauth2/callback.php",
            'oauth2_auth' => "auth/oauth2/auth",
            'oauth2_token' => "auth/oauth2/token",
            'oauth2_userinfo' => "auth/oauth2/userinfo",
            'cron' => 0
        ),
        'json' => array(
            'description' => "Fichier export OpenENT json",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'kosmos' => array(
            'description' => "Fichier export ENT kosmos",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv_oZe' => array(
            'description' => "Fichier export ENT oZe",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv_openent' => array(
            'description' => "Fichier export csv OpenENT",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv_pronote' => array(
            'description' => "Fichier export CSV Index Education EDT / Pronote (élèves ou professeurs)",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv_pronote_CAS' => array(
            'description' => "Fichier export CSV Pronote pour identifiants CAS uniquement",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv_groups' => array(
            'description' => "Groupes uniquement (CSV)",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        ),
        'csv' => array(
            'description' => "Aucun (CSV)",
            'mode' => "csv",
            'auth' => 0,
            'cron' => 0
        )
        /*
     * 'pronote' => array(
     * 'description' => "Pronote.net (abonnement)",
     * 'mode' => "pronote",
     * 'auth' => 0,
     * 'api_url' => "https://pronote-api.sambaedu.org",
     * 'api_user' => "synchro",
     * 'api_passwd' => "",
     * 'server_url' => "https://pronote.diderot.org/",
     * 'cas' => [
     * "ac-besancon",
     * "ac-caen",
     * "ac-clermont",
     * "ac-grenoble",
     * "ac-lille",
     * "ac-lyon",
     * "ac-montpellier",
     * "ac-nancy-metz",
     * "ac-nantes",
     * "ac-orleans-tours",
     * "ac-reims",
     * "ac-rouen",
     * "ac-strasbourg",
     * "ac-toulouse",
     * "arsene76",
     * "aten",
     * "hdf",
     * "iledefrance",
     * "kdecole-wayf",
     * "none",
     * "seineetmarne",
     * "toutatice",
     * "viescolaire"
     * ],
     * 'cours_unit' => "55",
     * 'cron' => 0
     * )
     */
    );
    // menu de choix de l'ent à configurer
    if (empty($ent)) {
        $ent['ent_type'] = "csv";
    }
    $html = "Choix de l'ENT ";
    $html .= "<select name=\"ent[ent_type]\" onchange=submit()>";
    $html .= "<option value=\"\">Choisir...</option>";
    foreach ($ent_list as $type => $params) {
        if ($file && $params['mode'] == "csv" || ! $file) {
            $html .= "<option value=\"$type\"";
            if ($type == $ent['ent_type']) {
                $html .= " selected>";
            } else {
                $html .= ">";
            }
            $html .= $params['description'] . "</option>";
        }
    }
    $html .= "</select><br>";
    if (! $file) {
        if (! isset($ent['pronote_cron'])) {
            $ent['pronote_cron'] = "0";
        }
        if (! isset($ent['openent_cron'])) {
            $ent['openent_cron'] = "0";
        }
        if (! isset($ent['ent_autorename'])) {
            $ent['ent_autorename'] = "0";
        }
        if (! isset($ent['ent_autogroups'])) {
            $ent['ent_autogroups'] = "0";
        }
        if (! isset($ent['ent_auth'])) {
            $ent['ent_auth'] = "0";
        }
        if (! isset($ent['ent_sync_cours'])) {
            $ent['ent_sync_cours'] = "0";
        }
        if (! isset($ent['ent_email'])) {
            $ent['ent_email'] = "0";
        }
        if (! isset($ent['ent_update_mode'])) {
            $ent['ent_update_mode'] = "all";
        }
        if (! isset($ent['openent_mfa'])) {
            $ent['openent_mfa'] = "0";
        }
        if (! isset($ent['openent_kc_auth'])) {
            $ent['openent_kc_auth'] = "0";
        }

        foreach ($ent as $key => $val) {
            if ((isset($val) && isset($config[$key]) && $val != $config[$key]) || isset($val)) {
                set_config($config, $key, $val, "ent");
            }
        }
        $config = get_config($config, true);
    } else {
        $config['ent_type'] = $ent['ent_type'];
    }
    if (isset($config['ent_type'])) {

        foreach ($ent_list[$config['ent_type']] as $key => $val) {
            $prefix = $ent_list[$config['ent_type']]['mode'] . "_";
            $ent[$prefix . $key] = $config[$prefix . $key] ?? $val;
        }
        switch ($ent_list[$ent['ent_type']]['mode']) {
            case "pronote":
                // if (! isset($ent['pronote_autorename']))
                // $ent['pronote_autorename'] = "0";
                $html .= "<div>URL du serveur passerelle vers Pronote";
                $html .= " <input type=\"text\" name=\"ent[pronote_api_url]\" value=" . $ent['pronote_api_url'] . "><br>\n";
                $html .= "Login du compte vie scolaire de l'établissement utilisé pour la synchro";
                $html .= " <input type=\"text\" name=\"ent[pronote_api_user]\" value=" . $ent['pronote_api_user'] . "><br>\n";
                $html .= "Mot de passe de l'utilisateur de l'etablissement ";
                $html .= " <input type=\"password\" name=\"ent[pronote_api_passwd]\" value=" . $ent['pronote_api_passwd'] . "><br>\n";
                $html .= "URL du serveur Pronote";
                $html .= " <input type=\"text\" name=\"ent[pronote_server_url]\" value=" . $ent['pronote_server_url'] . "><br>\n";
                $html .= "Type de CAS pour l'authentification Pronote";
                $html .= "<select name=\"ent[pronote_cas]\">";
                $html .= "<option value=\"\">Choisir...</option>";
                foreach ($ent_list['pronote']['cas'] as $key => $cas) {
                    $html .= "<option value=\"$cas\"";
                    if ($cas == $ent['pronote_cas']) {
                        $html .= " selected>";
                    } else {
                        $html .= ">";
                    }
                    $html .= $cas . "</option>";
                }
                $html .= "</select><br>";
                $html .= "Durée de la séquence de cours";
                $html .= " <input type=\"text\" name=\"ent[pronote_cours_unit]\" value=" . $ent['pronote_cours_unit'] . "><br>\n";

                $html .= "Mise à jour automatique ";
                if (isset($config['pronote_cron']) && $config['pronote_cron'] == 1)
                    $html .= " <input type=\"checkbox\" name=\"ent[pronote_cron]\" value=\"1\" checked><br>\n";
                else
                    $html .= " <input type=\"checkbox\" name=\"ent[pronote_cron]\" value=\"1\"><br>\n";
                // On ne connait pas le login pronote ?
                // $html .= "Renommage automatique des logins pour unicité avec l'ent ";
                // if ($config['pronote_autorename'] == 1)
                // $html .= " <input type=\"checkbox\" name=\"ent[pronote_autorename]\" value=\"1\" checked><br>\n";
                // else
                // $html .= " <input type=\"checkbox\" name=\"ent[pronote_autorename]\" value=\"1\"><br>\n";
                $html .= "</div>";
                break;
            case "csv":
            case "csv_siecle":
            case "csv_sts":
            case "xml":
            case "json":
                $html .= "Format du fichier d'importation des comptes : " . $ent_list[$ent['ent_type']]['mode'] . "<br>";
                $html .= "</div>";
                break;
            case "openent":
                $html .= "<div>URI de l'ENT ";
                $html .= "<select name=\"ent[openent_uri]\">";
                $html .= "<option value=\"\">Choisir...</option>";
                foreach ($ent_list['openent']['uri'] as $key => $uri) {
                    $html .= "<option value=\"$uri\"";
                    if ($uri == $ent['openent_uri']) {
                        $html .= " selected>";
                    } else {
                        $html .= ">";
                    }
                    $html .= $key . " : " . $uri . "</option>";
                }
                $html .= "</select></div><br/><div>";
                $html .= "<b>Import de ENT :</b><br/>\n";
                $html .= "Login d'un compte administrateur local de ENT de l'établissement : ";
                $html .= "<input type=\"text\" name=\"ent[openent_user]\" value=" . $ent['openent_user'] . "><br/>\n";
                $html .= "Mot de passe de ce compte : ";
                $html .= "<input type=\"password\" name=\"ent[openent_user_password]\" value=" . $ent['openent_user_password'] . "><br/>\n";
                $html .= "Double authentification du compte via l'envoi d'un SMS. ";
                if (isset($config['openent_mfa']) && $config['openent_mfa'] == 1) {
                    $html .= "<input type=\"checkbox\" name=\"ent[openent_mfa]\" value=\"1\" checked><br/>\n";
                } else {
                    $html .= "<input type=\"checkbox\" name=\"ent[openent_mfa]\" value=\"1\"><br/>\n";
                }
                $html .= "<br>Authentification OpenID via Keycloak. ";
                if (isset($config['openent_kc_auth']) && $config['openent_kc_auth'] == 1) {
                    $html .= "<input type=\"checkbox\" name=\"ent[openent_kc_auth]\" value=\"1\" checked><br/>\n";
                } else {
                    $html .= "<input type=\"checkbox\" name=\"ent[openent_kc_auth]\" value=\"1\"><br/>\n";
                }
                $html .= "<br>Mise à jour automatique : chaque nuit les comptes seront synchronisés avec l'ENT. ";
                if (isset($config['openent_cron']) && $config['openent_cron'] == 1) {
                    $html .= "<input type=\"checkbox\" name=\"ent[openent_cron]\" value=\"1\" checked><br/>\n";
                } else {
                    $html .= "<input type=\"checkbox\" name=\"ent[openent_cron]\" value=\"1\"><br/>\n";
                }
                $html .= "<br>Renommage automatique des logins pour unicité avec l'ENT : si besoin un alias de login sera créé sur l'ENT pour respecter les contraintes du login SambaÉdu. De même, si l'utilisateur modifie manuellement son login sur l'ENT, la modification sera faite sur l'AD SambaÉdu. ";
                if (isset($config['ent_autorename']) && $config['ent_autorename'] == 1) {
                    $html .= "<input type=\"checkbox\" name=\"ent[ent_autorename]\" value=\"1\" checked><br/>\n";
                } else {
                    $html .= "<input type=\"checkbox\" name=\"ent[ent_autorename]\" value=\"1\"><br/>\n";
                }
                $html .= "<br/>";
                $html .= "Suppression automatique des anciennes affectations des membres des équipes et des groupes lors de l'import. À n'activer qu'une fois que les équipes et les groupes ont bien été remontés sur l'ENT.
En début d'année, si l'import des équipes et groupes a été faite à l'aide des fichier CSV Pronote, ne pas cocher la case ! ";
                if (isset($config['ent_autogroups']) && $config['ent_autogroups'] == 1) {
                    $html .= "<input type=\"checkbox\" name=\"ent[ent_autogroups]\" value=\"1\" checked><br/>\n";
                } else {
                    $html .= "<input type=\"checkbox\" name=\"ent[ent_autogroups]\" value=\"1\"><br/>\n";
                }
                $html .= "<br/>";
                $html .= "Importer les groupes d'enseignements (en plus des classes). ";
                if (isset($config['ent_sync_cours']) && $config['ent_sync_cours'] == 1) {
                    $html .= "<input type=\"checkbox\" name=\"ent[ent_sync_cours]\" value=\"1\" checked><br/>\n";
                } else {
                    $html .= "<input type=\"checkbox\" name=\"ent[ent_sync_cours]\" value=\"1\"><br/>\n";
                }
                $html .= "<br/>";
                /*
                 * $html .= "<br/>";
                 * $html .= "Importer les adresses mails. ";
                 * if ($config['ent_email'] == 1)
                 * $html .= "<input type=\"checkbox\" name=\"ent[ent_email]\" value=\"1\" checked><br/>\n";
                 * else
                 * $html .= "<input type=\"checkbox\" name=\"ent[ent_email]\" value=\"1\"><br/>\n";
                 */

                $html .= "</div><br/><br/><div><b>Connecteur permettant l'authentification via l'ENT :</b><br/>\n";
                $html .= "Champs spécifiques OAuth2 :<br/>\n";
                $html .= "&emsp;Client ID (identifiant OAuth2) : ";
                $html .= "<input type=\"text\" name=\"ent[openent_oauth2_id]\" value=" . $ent['openent_oauth2_id'] . "> (conseil : mettre l'UAI à la fin, pour que cet ID soit unique au niveau de l'ENT, par exemple sambaedu-0752310D)<br/>\n";
                $html .= "&emsp;Scope : <input type=\"text\" value=\"userinfo\" readonly> (non modifiable)<br/>\n";
                $html .= "&emsp;Code secret Oauth2 : ";
                $html .= "<input type=\"password\" name=\"ent[openent_oauth2_pass]\" value=" . $ent['openent_oauth2_pass'] . "><br/>\n";
                $html .= "&emsp;URL de retour vers l'adresse externe du SE4 après l'authentification OAuth2 : ";
                $html .= "<input type=\"text\" size=\"50\" name=\"ent[openent_oauth2_redirect_uri]\" value=" . $ent['openent_oauth2_redirect_uri'] . "> (finie par /oauth2/callback.php)<br>\n";
                $html .= "<details>\n";
                $html .= "    <summary>Paramètres avancés :</summary>\n";
                $html .= "    &emsp;Chemin API auth Oauth2 : ";
                $html .= "<input type=\"text\" name=\"ent[openent_oauth2_auth]\" value=" . $ent['openent_oauth2_auth'] . "><br/>\n";
                $html .= "    &emsp;Chemin API token Oauth2 : ";
                $html .= "<input type=\"text\" name=\"ent[openent_oauth2_token]\" value=" . $ent['openent_oauth2_token'] . "><br/>\n";
                $html .= "    &emsp;Chemin API userinfo Oauth2 : ";
                $html .= "<input type=\"text\" name=\"ent[openent_oauth2_userinfo]\" value=" . $ent['openent_oauth2_userinfo'] . "><br/>\n";
                $html .= "</details>\n";
                $html .= "</div>\n";

                break;
            default:
                $html .= "Pas de parametrage possible.<br/>";
        }
        // $html .= "<input name=\"ent[ent_type]\" value= \"" . $ent_list[$ent['ent_type']]['mode'] . "\" type=\"hidden\">";
        if (! $file) {
            $html .= "<br/><div><b>Configuration SSO (connexion à l'interface utilisateur via l'ENT) :</b><br/>Activation de l'authentification OAuth2 via l'ENT : ";
            if ($config['ent_auth'] == 1)
                $html .= "<input type=\"checkbox\" name=\"ent[ent_auth]\" value=\"1\" checked><br></div>\n";
            else
                $html .= "<input type=\"checkbox\" name=\"ent[ent_auth]\" value=\"1\"><br></div>\n";
        }
        if (str_contains($_SERVER['REQUEST_URI'], "annu/config_ent.php")) {
            $html .= "<br/><input type=\"submit\" value=\"Valider\">\n";
        }
    }

    return $html;
}

/**
 * importe un fichier ENT, format csv ou xml.
 * Une archive zip est accepée, dans ce cas tous les ficihiers seront importés
 *
 * @param unknown $config
 * @return string
 */
function upload_ent($config, $files, $file_type, $manu = false)
{
    if (apcu_fetch("match_lock")) {
        return "Une importation est déjà en cours !<br>";
    }
    $html = "<br>";
    if (isset($files['csv_ent'])) {
        if (empty($files['csv_ent']['name'])) {
            return "<p style='color:red;'><b>ERREUR:</b> Aucun fichier n'a été fourni !</p>\n";
        }
        $html .= "Le fichier " . $files['csv_ent']['name'] . " a été chargé<br>";

        $tmp_ent_file = $files['csv_ent']['tmp_name'];
        $ent_file = $files['csv_ent']['name'];
        $size_ent_file = $files['csv_ent']['size'] / 1024;

        if (empty($tmp_ent_file)) {
            $html .= "<p>L'upload du fichier <span style='color:red;'>$ent_file</span> a semble-t-il &eacute;chou&eacute;.</p>";

            $upload_max_filesize = ini_get('upload_max_filesize');
            $post_max_size = ini_get('post_max_size');

            $html .= "<p>Il se peut que le fichier fourni ait &eacute;t&eacute; trop volumineux.<br />PHP est actuellement param&eacute;tr&eacute; avec:<br />\n";
            $html .= "</p>\n";
            $html .= "<blockquote>\n";
            $html .= "<span style='color:blue;'>upload_max_filesize</span>=<span style='color:green;'>" . $upload_max_filesize . "</span><br />\n";
            $html .= "<span style='color:blue;'>post_max_size</span>=<span style='color:green;'>" . $post_max_size . "</span><br />\n";
            $html .= "</blockquote>\n";
            $html .= "<p>\n";
            $html .= "Si ces valeurs sont insuffisantes pour vos fichiers XML, il est possible de modifier les valeurs limites dans <span style='color:green;'>/etc/php/" . substr(phpversion(), 0, 3) . "/fpm/pool.d/sambaedu.conf</span>\n";
            $html .= "</p>\n";
            return $html;
        }
        $dossier_tmp_import_comptes = sys_get_temp_dir();
        // mkdir($dossier_tmp_import_comptes);
        $dest_file = $dossier_tmp_import_comptes . "/" . $ent_file;
        if (file_exists($dest_file)) {
            unlink($dest_file);
        }
        if ($files['csv_ent']['name'] == "siecle_eleves.csv" || $files['csv_ent']['name'] == "sts_emp.csv") {
            file_put_contents($dest_file, file_get_contents($tmp_ent_file));
        } else {
            if (! is_uploaded_file($tmp_ent_file)) {
                return "<p style='color:red'>Impossible d'importer le fichier.</p><br>\n";
            }
            // $source_file = stripslashes("$tmp_ent_file");
            if (! move_uploaded_file("$tmp_ent_file", "$dest_file")) {
                return "<p style='color:red'>Impossible de copier le fichier.</p>\n";
            }
        }
        $html .= "Fichier de " . $size_ent_file . " ko correctement importé.<br>\n";
        // Si jamais un fichier non dézippé est fourni
        $extension_fichier_emis = strtolower(strrchr($ent_file, "."));

        if (($extension_fichier_emis == ".zip") || ($files['csv_ent']['type'] == "application/zip")) {
            if (! extension_loaded('zip')) {
                return "Erreur, le module zip manque.";
            }
            $dossier_tmp_import_comptes = $dossier_tmp_import_comptes . "/" . hash("sha256", rand());
            mkdir($dossier_tmp_import_comptes);
            $zip = new ZipArchive();
            if ($zip->open($dest_file) === TRUE) {
                $zip->extractTo($dossier_tmp_import_comptes);
            } else {
                return "<p style='color:red'>Echec de l'extraction de l'archive ZIP.</p>\n";
            }
            $zip->close();
            $files = scandir($dossier_tmp_import_comptes);
            $import_files = array();
            foreach ($files as $file) {
                if (preg_match("/.*\.(csv|xml|json)$/", $file) == 1) {
                    $import_files[] = $dossier_tmp_import_comptes . "/" . $file;
                } elseif (preg_match("/[0-9]{8}|[0-9]{4}-[0-9]{2}-[0-9]{2}/", $file) == 1) {
                    $fx = scandir($dossier_tmp_import_comptes . "/" . $file);
                    foreach ($fx as $f) {
                        if (preg_match("/.*\.xml$/", $f) == 1) {
                            if (preg_match("/_Etab/", $f)) {
                                array_unshift($import_files, $dossier_tmp_import_comptes . "/" . $file . "/" . $f);
                            } else {
                                array_push($import_files, $dossier_tmp_import_comptes . "/" . $file . "/" . $f);
                            }
                            $html .= $file . "/" . $f . "<br>";
                        } elseif (preg_match("/.*\.csv$/", $f) == 1) {
                            if (preg_match("/staff/", $f)) {
                                array_unshift($import_files, $dossier_tmp_import_comptes . "/" . $file . "/" . $f);
                            } else {
                                array_push($import_files, $dossier_tmp_import_comptes . "/" . $file . "/" . $f);
                            }
                            $html .= $file . "/" . $f . "<br>";
                        }
                    }
                }
            }
        } else {
            $import_files[] = $dest_file;
        }
        if (! $ent = cache_fetch('ent')) {
            $ent = [];
        }
        foreach ($import_files as $file) {
            if ($file_type == "csv_machines") {
                $html .= import_computer_list($config, $file, $file_type, $ent);
            } elseif ($file_type == "csv_sites") {
                $html .= import_site_list($config, $file, $file_type, $ent);
            } else {
                if ($file_type == "asm") {
                    if (basename($file, ".csv") == "students") {
                        $new_file_type = "asm_student";
                        $html .= import_ent($config, $file, $new_file_type, $ent);
                    } elseif (basename($file, ".csv") == "staff") {
                        $new_file_type = "asm_teacher";
                        $html .= import_ent($config, $file, $new_file_type, $ent);
                    }
                } else {
                    $html .= import_ent($config, $file, $file_type, $ent, $manu);
                }
            }
        }
        sort($ent);
        cache_store('ent', $ent, $config['ent_timeout']);
    } else {
        $html .= "Fichier d'importation des comptes : format $file_type <br>\n";
        switch ($file_type) {
            case "asm":
                $html .= 'Les fichiers doivent être des fichier ASM, eleve et profs. Il est possible de fournir directement l\'archive zip contenant tous les fichiers.<p>';
                break;
            case "json":
                $html .= 'Les fichiers doivent être des fichier json, issus de l\'export OpenENT https://adresseENT/directory/ ou https://adresseENT/directory/export/users (pour la partie utilisateurs) depuis un compte ADML. Il est possible de fournir directement l\'archive zip contenant tous les fichiers.<p>';
                break;
            case "gpei":
                $html .= 'Les fichiers doivent être des fichier GPEI, complets ou delta. Il est possible de fournir directement l\'archive zip contenant tous les fichiers<p>';
                break;
            case "csv_pronote":
                $html .= '<p>Fichier CSV contenant les informations des services des professeurs.</p>
<p>À récupérer <b>obligatoirement depuis le logiciel EDT d\'Index Education en mode SPR ou proviseur</b>, menu <i>Imports/Exports</i>, <i>TXT/CSV/XML/SQL</i>,
 <i>Exporter un fichier texte</i>, choisir Professeurs comme <i>Type de données à exporter</i>, et tout laisser par défaut (voir captures ci-dessous).</p>
<p>Ce fichier permet d\'associer correctement les profs à leurs classes et groupes sans devoir attendre la remontée STS ou ENT.</p><p></p>
<p>Il est également possible d\'importer les élèves de cette façon, si par exemple vous ne disposez pas de fichier SIECLE.</p>
<p>ATTENTION cet export doit plus être pensé comme complémentaire à une synchro ENT, en début d\'année par exemple,
 en attendant que les classes et groupes aient été remontées dans l\'ENT, que comme le moyen principal de renseigner la
 base profs. En effet il ne contient pas les logins, et donc ne gérera pas les doublons à l\'échelle de l\'académie ou de la région comme l\'ENT.</p>
  <li>  première ligne avec les champs :
  <li>  NOM;PRENOM;FONCTION;CIVILITE;CLASSES;GROUPES
  <li>  + NOM : nom en majuscule => TEST
  <li>  + PRENOM : prénom => Elève
  <li>  + FONCTION : ENS
  <li>  EMAIL : test@ac-truc.fr
  <li>  + CLASSES : classe(s), séparées par des virgules et CLASSE pour les élèves
  <li>  GROUPES : groupe(s), séparés par des virgules
  <li>  CIVILITE : sexe M/Mme

  <p>seuls les champs avec + sont obligatoires</p>

<pre>
NOM;PRENOM;FONCTION;CIVILITE;CLASSES;GROUPES;EMAIL
TERIEUR;Alex;ENS;M;3_E,5_1;3XX,5yy,test@ac-truc.fr
</pre><p>
';
                $html .= "<p>Étapes d'export :</p>";
                $html .= "<img src=\"images/edt1.png\"><img src=\"images/edt2.png\"><p>";
                break;
            case "csv_machines":
                $html .= 'Fichier csv contenant les informations des machines
<li> Facultatif :  première ligne avec les champs :
  <li>  name;ip;mac;type;ou
  <li>   + name : nom de la machine, compatible windows (< 15, pas de . et de _)
  <li>   ip : adresse ip (facultative, en cas d\'absence la machine aura une ip dynamique)
  <li>  + mac : adresse mac aa:bb:cc:dd:ee:ff
  <li>  type : computer ou equipement (par défaut computer)
 <li>   ou : salle dans laquelle doit être mise la machine. Par défaut OU=Computers

  <p>seuls les champs avec + sont obligatoires<br>
<pre>
name;ip;mac;type;ou
poste1;;aa:bb:cc:dd:ee:ff;;
poste2;172.20.100.100;aa:bb:cc:dd:ee:00;computer;Salle_info
imprimante1;172.20.100.101;aa:bb:cc:dd:ee:01;equipement;
</pre><p>
';
                break;
            case "csv_sites":
                $html .= 'Fichier csv contenant les informations de l\'adressage IP des sites
<li>première ligne avec les champs :
  <li>  UAI;Perdir;Pedagogie;Imprimantes;Serveurs;PMX
<pre>
UAI;Perdir;Pedagogie;Imprimantes;Serveurs;PMX
0990000X;172.16.1.0;172.17.0;172.18.0;10.0.0.0</pre><p>
';
                break;
            default:
                $html .= 'Le fichier doit être au format csv, <b>IMPERATIVEMENT encodé en UTF-8</b>, et il peut être zippé et doit obligatoirement contenir la ligne avec les intitulés des colonnes.
les champs nom, prénom, profil et classe sont obligatoires. D\'autres champs sont possibles, voir le code<p>
  Le csv format utilisé correspond à celui utilisé par l\'ENT Openent :
 <li> première ligne avec les champs :
  <li>  "";"Id Siecle";"type";"lastName";"firstName";"originalLogin";"login","code";"functions";"structures";"allClasses";"childrens";"parents";"birthDate";"sex";"allGroups"
 <li>   id : identifiant unique ENT
 <li>   Id Siecle : numero eleve siecle (eleves uniquement)
 <li>   + lastName : nom en majuscule => TEST
  <li>  + firstName : prénom => Elève
  <li>  + type : Student|Teacher|Personnel|Relative|Guest
 <li>   originalLogin : login d\'origine, prenom.nom(x) x étant un nombre pour les doublons.
 <li>   login : login simplifié chois par l\'utilisateur => repris en priorité par Sambaedu
 <li>   code : mot de passe initial => repris par sambaedu si il existe
 <li>   functions : Personnel administratif
 <li>   structures : nom de(s) l\'établissemnt(s) en majuscules ,séparés par des virgules
 <li>   + allClasses : classe(s), séparées par des virgules
 <li>   allGroups : groupe(s), séparés par des virgules
 <li>   birthDate : date de naissance AAAAMMJJ
  <li>  sex : sexe M/F

  <p>seuls les champs avec + sont obligatoires<br>

<pre>
"lastName";"firstName";"type";"birthDate";"sex";"allClasses"
"TERIEUR";"Alex";"Student";"20000120";"M";"3_E"
"TERIEUR";"Alain";"Teacher";"19690528";"M";"3_E,4_G,5_A,6_F"
"BONNOT";"Jean";"Personnel";"19600131";"M";"3_E"
</pre><p>
';
        }
        $html .= "<input title=\"chemin du fichier\" size=\"100\" name=\"csv_ent\" type=\"file\"></input><p>\n";
    }
    return $html;
}

/**
 * importe un fichier zip, format csv ou xml.
 * gpei
 * Une archive zip est accepée, dans ce cas tous les ficihiers seront importés
 *
 * @param unknown $config
 * @return string
 */
function import_archive($config, $file, $file_type)
{
    $html = "";
    $dossier_tmp_import_comptes = sys_get_temp_dir();
    if (! extension_loaded('zip')) {
        return "Erreur, le module zip manque.";
    }
    $dossier_tmp_import_comptes = $dossier_tmp_import_comptes . "/" . hash("sha256", rand());
    mkdir($dossier_tmp_import_comptes);
    $zip = new ZipArchive();
    if ($zip->open($file) === TRUE) {
        $zip->extractTo($dossier_tmp_import_comptes);
    } else {
        return "<p style='color:red'>Echec de l'extraction de l'archive ZIP.</p>\n";
    }
    $zip->close();
    $files = scandir($dossier_tmp_import_comptes);
    $import_files = array();

    foreach ($files as $file) {
        if (preg_match("/.*\.[csv|xml]$/", $file) == 1) {
            $import_files[] = $dossier_tmp_import_comptes . "/" . $file;
        } elseif (preg_match("/[0-9]{8}|[0-9]{4}-[0-9]{2}-[0-9]{2}/", $file) == 1) {
            $fx = scandir($dossier_tmp_import_comptes . "/" . $file);
            foreach ($fx as $f) {
                if (preg_match("/.*\.xml$/", $f) == 1) {
                    if (preg_match("/_Etab/", $f)) {
                        array_unshift($import_files, $dossier_tmp_import_comptes . "/" . $file . "/" . $f);
                    } else {
                        array_push($import_files, $dossier_tmp_import_comptes . "/" . $file . "/" . $f);
                    }
                    $html .= $file . "/" . $f . "<br>";
                } elseif (preg_match("/.*\.csv$/", $f) == 1) {
                    if (preg_match("/staff/", $f)) {
                        array_unshift($import_files, $dossier_tmp_import_comptes . "/" . $file . "/" . $f);
                    } else {
                        array_push($import_files, $dossier_tmp_import_comptes . "/" . $file . "/" . $f);
                    }
                    $html .= $file . "/" . $f . "<br>";
                }
            }
        }
    }

    if (! $ent = cache_fetch('ent')) {
        $ent = [];
    }
    foreach ($import_files as $file) {
        if ($file_type == "csv_machines") {
            $html .= import_computer_list($config, $file, $file_type, $ent);
        } else {
            if ($file_type == "asm") {
                if (basename($file, ".csv") == "students") {
                    $new_file_type = "asm_student";
                    $html .= import_ent($config, $file, $new_file_type, $ent);
                } elseif (basename($file, ".csv") == "staff") {
                    $new_file_type = "asm_teacher";
                    $html .= import_ent($config, $file, $new_file_type, $ent);
                }
            } else {
                $html .= import_ent($config, $file, $file_type, $ent);
            }
        }
    }
    sort($ent);
    cache_store('ent', $ent, $config['ent_timeout']);
    return $html;
}

function form_map_users($config, $global = false)
{
    $ent = cache_fetch('new');
    $ad_users_group = ldap_users_group($config, cache_fetch('ad_users'));

    $html = "<table class='table-bordered' width=\"80%\">\n";
    $html .= "<th colspan=4 class=menuheader style=\"height: 30\">Mise en correspondance des utilisateurs</th>\n";

    $html .= "<tr><td class='menuheader' align=\"center\">Type</td>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant ENT</td>";
    $html .= "<td class='menuheader' align=\"center\">Classe(s)</td>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant AD</td>";
    $html .= "</tr>\n";

    foreach ($ent as $index => $user) {
        if ($user['type'] == "Teacher") {
            $user['type'] = "prof";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($ad_users_group['prof'] as $prof) {
                $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ")</option>";
            }
            if ($global) {
                foreach ($ad_users_group['prof'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", sans classe)</option>";
                }
                foreach ($ad_users_group['trash'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Corbeille)</option>";
                }
                foreach ($ad_users_group['administratif'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Administratif)</option>";
                }
            }

            $html .= "</select></td>";
        } elseif ($user['type'] == "Student") {
            $user['type'] = "eleve";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($user['allClasses'] as $classe) {
                foreach ($ad_users_group['eleve'][$classe['name']] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ")</option>";
                }
            }
            if ($global) {
                foreach ($ad_users_group['eleve']['sans'] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ", sans classe)</option>";
                }
                foreach ($ad_users_group['trash'] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ", corbeille)</option>";
                }
            }

            $html .= "</select></td>";
        }
        if ($user['type'] == "Personnel" || $user['type'] == "Guest") {
            $user['type'] = "administratif";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($ad_users_group['administratif'] as $prof) {
                $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ")</option>";
            }
            if ($global) {
                foreach ($ad_users_group['prof'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Administratif)</option>";
                }
                foreach ($ad_users_group['trash'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", corbeille)</option>";
                }
            }

            $html .= "</select></td>";
        } else {
            $type = "";
        }
        $html .= "</tr>";
        $html .= "<input type=\"hidden\" name=\"form\" value=\"match\">";
    }

    $html .= "</table>";
    return $html;
}

function form_rename_users($config, $global = false)
{
    $match = cache_fetch('match');

    $html = "<table class='table-bordered' width=\"80%\">\n";
    $html .= "<th colspan=4 class=menuheader style=\"height: 30\">Renommage des utilisateurs</th>\n";

    $html .= "<tr>";
    $html .= "<td class='menuheader' align=\"center\">type</td>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant commun</td>";
    $html .= "<td class='menuheader' align=\"center\">Classe(s)</td>";
    $html .= "<td class='menuheader' align=\"center\">Login proposé</td>";
    $html .= "<td class='menuheader' align=\"center\">Renommer</td>";
    $html .= "</tr>\n";

    foreach ($match as $user) {
        if (($user['ent']['login'] != $user['ad']['cn']) || ($user['ent']['login'] != mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName']) && $user['ent']['login'] != $user['ent']['originalLogin'])) {
            if ($user['ent']['type'] == "Teacher") {
                $user['ent']['type'] = "prof";
            } elseif ($user['ent']['type'] == "Student") {
                $user['ent']['type'] = "eleve";
            } elseif ($user['ent']['type'] == "Personnel") {
                $user['ent']['type'] = "administratif";
            }

            $html .= "<tr><td>" . $user['ent']['type'] . "</td>";
            $html .= "<td>" . $user['ent']['firstName'] . " " . $user['ent']['lastName'] . " (" . $user['ent']['login'] . ")</td>";
            if (count($user['ent']['allClasses']) > 0 && ! empty($user['ent']['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['ent']['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>" . mk_cn($config, $user['ent']['lastName'], $user['ent']['firstName']) . "</td>\n";
            $html .= "<td>";
            $html .= "<input type=\"checkbox\" name=\"ad[" . $user['ent']['login'] . "]\" ";
            if ($global) {
                $html .= "checked";
            }
            $html .= "></td>";
            $html .= "</tr>";
        }
    }
    $html .= "<input type=\"hidden\" name=\"form\" value=\"rename\">";
    $html .= "</table>";
    return $html;
}

function display_trash_user(&$html, $user, $classe, $type, $selected)
{
    if (! $user['protected'] && $type != "trash") {
        $html .= "<tr><td>" . $user['cn'] . "</td>";
        $html .= "<td>" . $user['prenom'] . " " . $user['nom'] . " </td>";
        $html .= "<td>" . $type . "</td>";
        $html .= "<td>" . $classe . "</td>\n";
        $html .= "<td>";
        $html .= "<input type=\"checkbox\" name=\"ad[" . $user['cn'] . "]\" ";
        if ($selected || $user['trash']) {
            $html .= "checked";
        }
        $html .= "></td>";
        $html .= "</tr>";
    }
}

function form_trash_users($config, $select = "none")
{
    $ad_users_group = ldap_users_group($config, cache_fetch('ad_users'));

    $html = "<table class='table-bordered' width=\"80%\">\n";
    $html .= "<th colspan=5 class=menuheader style=\"height: 30\">Mise à la corbeille des utilisateurs</th>\n";

    $html .= "<tr>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant AD</td>";
    $html .= "<td class='menuheader' align=\"center\">Nom</td>";
    $html .= "<td class='menuheader' align=\"center\">type</td>";
    $html .= "<td class='menuheader' align=\"center\">Classe(s)</td>";
    $html .= "<td class='menuheader' align=\"center\">Supprimer</td>";
    $html .= "</tr>\n";

    foreach ($ad_users_group as $type => $groupes) {
        if ($type == "eleve") {
            foreach ($groupes as $classe => $users) {
                foreach ($users as $user) {
                    $global = ($select == "all" || $select == $type || ($select == "inactive" && $user['pwdlastset'] == 0));
                    display_trash_user($html, $user, $classe, $type, $global);
                }
            }
        } else {
            foreach ($groupes as $user) {
                $global = ($select == "all" || $select == $type || ($select == "inactive" && $user['pwdlastset'] == 0));
                display_trash_user($html, $user, "", $type, $global);
            }
        }
    }

    $html .= "<input type=\"hidden\" name=\"form\" value=\"trash\">";
    $html .= "</table>";
    return $html;
}

// @TODO
function form_merge_cours($config, $etab, $global)
{
    $ent = cache_fetch('ent');
    $ad_users_group = ldap_users_group($config, cache_fetch('ad_users'));

    $html = "<table class='table-bordered' width=\"80%\">\n";
    $html .= "<th colspan=4 class=menuheader style=\"height: 30\">Fusion des groupes cours pour le partage de fichiers</th>\n";

    $html .= "<tr><td class='menuheader' align=\"center\">Nom</td>";
    $html .= "<td class='menuheader' align=\"center\">Cours Fusionnés</td>";
    $html .= "<td class='menuheader' align=\"center\">cours fusionnés</td>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant AD</td>";
    $html .= "</tr>\n";

    foreach ($ent as $user) {
        if ($user['type'] == "Teacher") {
            $user['type'] = "prof";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($ad_users_group['prof'] as $prof) {
                $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ")</option>";
            }
            if ($global) {
                foreach ($ad_users_group['prof'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", sans classe)</option>";
                }
                foreach ($ad_users_group['trash'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Corbeille)</option>";
                }
                foreach ($ad_users_group['administratif'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Administratif)</option>";
                }
            }

            $html .= "</select></td>";
        } elseif ($user['type'] == "Student") {
            $user['type'] = "eleve";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($user['allClasses'] as $classe) {
                foreach ($ad_users_group['eleve'][$classe['name']] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ")</option>";
                }
            }
            if ($global) {
                foreach ($ad_users_group['eleve']['sans'] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ", sans classe)</option>";
                }
                foreach ($ad_users_group['trash'] as $eleve) {
                    $html .= "<option value=\"" . $eleve['cn'] . "\">" . $eleve['prenom'] . " " . $eleve['nom'] . " (" . $eleve['cn'] . ", corbeille)</option>";
                }
            }

            $html .= "</select></td>";
        }
        if ($user['type'] == "Personnel" || $user['type'] == "Guest") {
            $user['type'] = "administratif";
            $html .= "<tr><td>" . $user['type'] . "</td>";
            $html .= "<td>" . $user['firstName'] . " " . $user['lastName'] . " " . $user['login'] . "</td>";
            if (count($user['allClasses']) > 0 && ! empty($user['allClasses'][0]['name'])) {
                $html .= "<td>";
                foreach ($user['allClasses'] as $classe) {
                    $html .= $classe['name'] . " ";
                }
            } else {
                $html .= "<td>pas de classe !</td>";
            }

            $html .= "</td>";
            $html .= "<td>";
            $html .= "<select name=\"ad[" . $user['login'] . "]\">";
            $html .= "<option value=\"\">Choisir...</option>";
            $html .= "<option value=\"#new#\">Créer...</option>";
            foreach ($ad_users_group['administratif'] as $prof) {
                $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ")</option>";
            }
            if ($global) {
                foreach ($ad_users_group['prof'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", Administratif)</option>";
                }
                foreach ($ad_users_group['trash'] as $prof) {
                    $html .= "<option value=\"" . $prof['cn'] . "\">" . $prof['prenom'] . " " . $prof['nom'] . " (" . $prof['cn'] . ", corbeille)</option>";
                }
            }

            $html .= "</select></td>";
        } else {
            $type = "";
        }
        $html .= "</tr>";
        $html .= "<input type=\"hidden\" name=\"form\" value=\"match\">";
    }

    $html .= "</table>";
    return $html;
}

function form_reinit_password($config, $selected = false)
{
    $match = cache_fetch('match');
    $html = "La table affiche les utilisateurs dont l'ENT est en mode \"code d'activation\". Par défaut lors de l'importation si le compte SambaEdu comporte déjà un mot de passe, celui-ci n'est pas changé. Si vous souhaitez qu'il soit également réinitialisé cochez la case de l'utilisateur";
    $html .= "<table class='table-bordered' width=\"80%\">\n";
    $html .= "<th colspan=5 class=menuheader style=\"height: 30\">Réinitialisation des mots de passes</th>\n";

    $html .= "<tr>";
    $html .= "<td class='menuheader' align=\"center\">Identifiant AD</td>";
    $html .= "<td class='menuheader' align=\"center\">Nom</td>";
    $html .= "<td class='menuheader' align=\"center\">type</td>";
    $html .= "<td class='menuheader' align=\"center\">Code</td>";
    $html .= "<td class='menuheader' align=\"center\">reinitialiser</td>";
    $html .= "</tr>\n";

    foreach ($match as $user) {
        if (! empty($user['ent']['code'])) {
            $html .= "<tr><td>" . $user['ad']['cn'] . "</td>";
            $html .= "<td>" . $user['ad']['prenom'] . " " . $user['ad']['nom'] . " </td>";
            $html .= "<td>" . $user['ent']['type'] . "</td>";
            $html .= "<td>" . $user['ent']['code'] . "</td>\n";
            $html .= "<td>";
            $html .= "<input type=\"checkbox\" name=\"ad[" . $user['ad']['cn'] . "]\" ";
            if ($selected || $user['ad']['pwdlastset'] != 0) {
                $html .= "checked";
            }
            $html .= "></td>";
            $html .= "</tr>";
        }
    }

    $html .= "<input type=\"hidden\" name=\"form\" value=\"password\">";
    $html .= "</table>";
    return $html;
}

function form_ent_next($type = "apply")
{
    $types = array(
        "nouveau" => "1 - Récupération des données",
        "nouveau_force" => "1 - Forcer la récupération",
        "read_file" => "1 - Analyse du fichier des données",
        "sync_groups" => "1 - Analyse des groupes",
        "match" => "2 - Mise en correspondance automatique",
        // "merge_pronote" => "1 - Fusion des données prof de Pronote",
        "match_groups" => "3 - Correspondances par groupe",
        "match_global" => "3 - Correspondances globales",
        "rename" => "4 - Renommer des utilisateurs",
        "rename_global" => "4 - Renommer tous les utilisateurs",
        "password" => "5 - Réinitialiser le mot de passe des nouveaux utilisateurs",
        "trash_none" => "6 - Choisir les utilisateurs sans correspondance à mettre à la corbeille",
        "trash_all" => "6 - Sélectionner tous les utilisateurs sans correspondance pour mettre à la corbeille",
        "trash_eleve" => "6 - Sélectionner tous les élèves sans correspondance pour mettre à la corbeille",
        "trash_prof" => "6 - Sélectionner tous les profs sans correspondance pour mettre à la corbeille",
        "trash_administratif" => "6 - Sélectionner tous les personnels sans correspondance pour mettre à la corbeille",
        "trash_sans" => "6 - Sélectionner tous utilisateurs non affectés pour les mettre à la corbeille",
        "trash_inactive" => "6 - Sélectionner tous les utilisateurs non activés et sans correspondance pour mettre à la corbeille",
        "apply" => "7 - Valider",
        "commit" => "8 - Appliquer les changements"
    );
    $html = "Sélectionner l'étape suivante :  <SELECT NAME=\"action\"\">";
    foreach ($types as $t => $txt) {
        $html .= "<option value=\"$t\"";
        if ($type == $t) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "<input type=\"submit\" value=\"Valider\">";
    $html .= "</SELECT><br>";
    return $html;
}

function generate_listing($name = "listing", $wait = true)
{
    // Lien pour la récupération du mailing
    $listing = apcu_fetch($name);
    if (is_array($listing) && count($listing, COUNT_RECURSIVE) > 1) {
        $html = ("<form id='postlisting' action='../annu/listing.php' method='post'>");
        $html .= ("<table><tr><td><img src='../elements/images/pdffile.png'></td><td>Il n'est peut-être pas très prudent de conserver inutilement ces données sur le serveur</td><td>");
        $html .= ("<input type='submit' name='action' value='télécharger pdf' />");
        $html .= ("<input type='submit' name='action' value='télécharger csv' /></td></tr>");
        $html .= ("<input type='hidden' name='name' value='" . $name . "' /></td></tr>");
        if ($wait) {
            $html .= ("<tr><td>Attention : </td><td>la réinitialisation ne sera effective que quelques minutes après avoir cliqué ici : </td><td><input type='submit' name='action' value='réinitialiser les mots de passe' /></td></tr>");
        } else {
            $html .= ("<tr><td>Info  : </td><td>La réinitialisation peut prendre quelques minutes ! </td><td></td></tr>");
        }
        $html .= ("</table></form>");
        // apcu_delete('listing');
        return $html;
    }
}

// Tri du listing par classe/nom/prénom
function trieleve($a, $b)
{
    $res = strcmp(@$a["structure"], @$b["structure"]);
    if ($res == 0) {
        // tri par classe
        $res = strcmp($a["cla"], $b["cla"]);
        if ($res == 0) {
            $res = strcmp($a["nom"], $b["nom"]);
            if ($res == 0) {
                $res = strcmp($a["pre"], $b["pre"]);
            }
        }
    }

    return $res;
}

function apply_user_passwords($config, $tmax = 30)
{
    lock('passwords');
    $passwords = apcu_fetch("passwords");
    if (! is_array($passwords)) {
        $passwords = [];
    } else {
        apcu_delete('passwords');
    }
    unlock('passwords');

    $t = time();
    while (count($passwords) > 0 && (time() - $t) < $tmax) {
        $dn = array_key_first($passwords);
        $c = array_shift($passwords);
        usersetpassword($config, $dn, $c['newpassword'], $c['change'], $c['oldpassword']);
    }
    lock('passwords');
    if (is_array($p = apcu_fetch("passwords"))) {
        $passwords = array_merge($passwords, $p);
    }
    $count = count($passwords);
    apcu_store('passwords', $passwords);
    unlock('passwords');
    return $count;
}

function reinit_listing_passwords($config, $listing)
{
    $res = true;
    $n = 0;
    if (count($listing) > 0) {
        foreach ($listing as $dn => $user) {
            $res = usersetpassword($config, $dn, $user['pwd'], $user['change'], "", true);
            if ($res) {
                $n ++;
            }
        }
    }
    return $n . " utilisateurs ont vont être reinitialisés.";
}

function generate_listing_csv($config, $listing)
{
    $contenu_fichier = "";
    if (is_array($listing) && count($listing) > 0) {
        usort($listing, "trieleve");
        // generation du csv
        $contenu_fichier = "login;lastName;firstName;email;structure;allClasses;activated;code\n";

        foreach ($listing as $user) {
            $contenu_fichier .= $user['cn'] . ";" . $user['nom'] . ";" . $user['pre'] . ";" . $user['email'] . ";" . $user['structure'] . ";" . $user['cla'] . ";" . $user['change'] . ";" . $user['pwd'] . "\n";
        }
    }
    return $contenu_fichier;
}

function generate_listing_pdf($config, $listing, $mail = false, $file = false, $uai = "")
{
    $content = "";
    if (count($listing) > 0) {
        usort($listing, "trieleve");

        $content = "<page backtop='15mm' backbottom='15mm'>";

        foreach ($listing as $user) {
            if ($user['change']) {
                $type = "code d'activation";
            } else {
                $type = "mot de passe";
            }
            if (! isset($struct_preced)) {
                $struct_preced = $user['structure'] ?? "";
            }
            if ($user['structure'] != $struct_preced) {
                $struct_preced = $user['structure'];
                $content .= "</page><page backtop='15mm' backbottom='15mm'>";
                $content .= "<div style='margin-bottom:3mm; border: solid 1px black; padding:3mm; width: 80%; margin: auto'>";
                $content .= "<table style='width:100%'><tr><td style='width:50%'>";
                $content .= "</td>";
                $content .= "<td>" . etab_to_name($config, $user['structure']) . "</td>";
                $content .= "</tr></table>";
                $content .= "</div>";
            }
            if (! isset($classe_preced)) {
                $classe_preced = $user['cla'] ?? "";
            }
            if ($user['cla'] != $classe_preced) {
                $classe_preced = $user['cla'];
                $content .= "</page><page backtop='15mm' backbottom='15mm'>";
            }

            $content .= "<div style='margin-bottom:3mm; border: solid 1px black; padding:3mm; width: 80%; margin: auto'>";

            $content .= "<table style='width:100%'><tr><td style='width:50%'>";
            $content .= $user['nom'] . " ";
            $content .= $user['pre'] . "<br>" . etab_to_name($config, $user['structure']) . "<br>";
            $content .= "Classe : " . $user['cla'] . "</td>";

            $content .= "<td style='width:50%'>";
            $content .= "identifiant  : <span style='font-family:courier'><b>" . $user['cn'] . "</b></span><br>";
            if (! empty($user['or']) && $user['or'] != $user['cn']) {
                $content .= "identifiant initial ENT : <span style='font-family:courier'><b>" . $user['or'] . "</b></span><br>";
            }
            $pwde = $user['pwde'] ?? $user['pwd'];
            if (! empty($pwde)) {
                $content .= $type . " " . $user['mode'] . " : <span style='font-family:courier'><b>" . $pwde . "</b></span>";
                if ($user['pwd'] != $pwde) {
                    $content .= $type . " pour les ordinateurs : <span style='font-family:courier'><b>" . $user['pwd'] . "</b></span>";
                }
            } else {
                $content .= "Email : <span style='font-family:courier'><b>" . $user['email'] . "</b></span>";
            }
            $content .= "</td></tr>";
            $content .= "<tr><td>";
            $content .= "<span style='font-size:50%'>";
            if ($user['change']) {
                $content .= "Le code d'activation permet de changer le mot de passe lors de la première connexion.<br>";
            }
            $content .= "Le mot de passe est strictement personnel et doit être protégé.<br> L'utilisateur est responsable de l'usage qui est fait de son identifiant.</span></td>";
            if (! empty($config['central_url'])) {
                $content .= '<td><span>Addresse de connexion : <a href="' . $config['central_url'] . '">' . $config['central_url'] . '</a></span></td>';
            }
            $content .= "</tr></table>";
            $content .= "</div>";
        }

        $content .= "</page>";
        require_once (dirname(__FILE__) . '/../vendor/autoload.php');
        // require_once ('../vendor/autoload.php');
        // use Spipu\Html2Pdf\Html2Pdf;
        // use Spipu\Html2Pdf\Exception\Html2PdfException;
        // use Spipu\Html2Pdf\Exception\ExceptionFormatter;

        $html2pdf = new Spipu\Html2Pdf\Html2Pdf('P', 'A4', 'fr');
        $html2pdf->pdf->SetDisplayMode('fullpage');
        $html2pdf->writeHTML($content);
        if ($file) {
            if (! is_dir("/tmp/listing")) {
                mkdir("/tmp/listing");
            }
            if (! is_dir("/tmp/listing/" . date("d.m.Y"))) {
                mkdir("/tmp/listing/" . date("d.m.Y"));
            }
            return $html2pdf->output('/tmp/listing/' . date("d.m.Y") . '/listing_' . $uai . '_' . date("d.m.Y") . '.pdf', "F");
        } else {
            if ($mail) {
                $flag = "E";
            } else {
                $flag = "D";
            }
            return $html2pdf->output('listing_' . $uai . '_' . date("d.m.Y") . '.pdf', $flag);
        }
    } else {
        return "";
    }
}
?>